#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/ipc.h>
/*#include <unistd.h>*/
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/asynch.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include "rtv.h"
#include "rtv_ops.h"
#include "rtvdiskio.h"

extern SERVER_OBJECT *rtvdisk_init();
/*
#define TRACE_DIAGS
*/

SERVER_OBJECT  *rtvobj;

#define ENABLE_RTVMODE 0x170
#define DISABLE_RTVMODE 0x171


#define MAX_IDLE_TIME 100*1000
#define STOP_DELAY_TIMEOUT 2000*1000
#define SHUTTLE_RESPONSE 1000*100
#define TRACKING_TIMEOUT 500*1000
char           *get_token();
extern char    *strtok();
extern void     cleanup();

main(int argc, char **argv)
{
    fd_set          readfds;
    static int      request, action;
    static int      cli_addr, cli_len;
    char            server_hostname[30];
    char           *token, *bptr, cmdbuf[132], msgbuf[132];
    static char	   *device= "/dev/fbs/tvtwo0";
    struct timeval  timer;
    int             nbytes, i,n;
    static int      record_error,record_level;
    static int      port = XmitServerPort;
    static int      shuttle_count, comm_stat;
    static int      record_status, ret;
    static int      audio_busy = 0;
    static int      file_status = 0;
    static int      frame_delta, rewind = 0;
    static int      idle_time = 0;
    static int      shutdown_delay = 0;
    static int      shuttle_delay = 0;
    static int      current_frame=0;
    static int      elapse_time=0;
    static int      rtv_enable;
    static int      secs=0;
    static int      qlev;
    static int      name_leng;
    register int    csock;
    RTV_ATTRIBUTES  attrs,play_attrs;
    /*
     * TODO:
     */
    if (argv[1] != NULL)
        device = argv[1];

    gethostname(server_hostname, 30);

    /*
     * Setup signals for cleanup routine.
     */
    signal(SIGHUP, cleanup);
    signal(SIGINT, cleanup);
    signal(SIGQUIT, cleanup);
    signal(SIGTERM, cleanup);
    signal(SIGPIPE, cleanup);

    /*
     * Create and initialize service object.
     */
    rtvobj = rtvdisk_init();
    rtvobj->record_attrs.sid = 0;
    rtvobj->record_attrs.win_x = 100;
    rtvobj->record_attrs.win_y = 100;
    rtvobj->record_attrs.win_h = 480;
    rtvobj->record_attrs.win_w = 640;
    rtvobj->record_attrs.interlace = 1;
    rtvobj->record_attrs.rate = 12;
    rtvobj->record_attrs.qfactor = 100;
    rtvobj->record_attrs.fps = 30;
    rtvobj->record_attrs.toc_count=0;
    rtvobj->record_attrs.frame_count=0;
    rtvobj->record_attrs.audio_slice = 8192;
    rtvobj->record_attrs.audio_sample_rate = 48000;
    rtvobj->record_attrs.bytes_per_sample = 2;
    rtvobj->record_attrs.audio_channels = 2;
    rtvobj->record_attrs.audio_encoding = AUDIO_ENCODING_LINEAR;
    rtvobj->record_attrs.audio_endian = 0;
    rtvobj->timer.tv_sec = 0;
    rtvobj->timer.tv_usec = 5000;
    rtvobj->audio_status=0;
    rtvobj->audio_mode=1;
    rtvobj->ccube_fd=0;
    rtvobj->audioFD=0;
    rtvobj->client=NULL;
    /*
     * Open and initialize Video compression device.
     */
     rtvobj->ccube_fd = rtv_tvtwo_init(&rtvobj->record_attrs,device);
     if (rtvobj->ccube_fd <= 0) {
        fprintf(stderr, "No tvtwo device\n");
	cleanup(0);
     }
     /*
      * Open and initialize ramdisk device.
      */
     ramdisk_init(rtvobj);
     stop_record(rtvobj->ccube_fd);
     stop_play(rtvobj->ccube_fd);
    /*
     * Store verison number in RTV object.
     */
    strncpy(rtvobj->version,"newrtv 1.1  ",12);
    /*
     * Bind to well known port for client connections.
     */
    rtvobj->port = port;
    rtvobj->srvr_sock = init_server_socket(rtvobj->port);
    /*
     * Ccube Initialization was moved from here to the ATTACH request.
     */
                             
    rtvobj->local_host = (char *) malloc(strlen(server_hostname));
    strcpy(rtvobj->local_host, server_hostname);
    rtvobj->action = STOP_RTV;
    /*
     * Start listening for client connection requests.
     */
    FD_ZERO(&readfds);
    listen(rtvobj->srvr_sock, 32);

    while (1) {
        FD_SET(rtvobj->srvr_sock, &readfds);
        for (i = 0; i < MAX_CONNECTIONS; i++)
            if (rtvobj->client_sock[i] != 0)
                FD_SET(rtvobj->client_sock[i], &readfds);
        /*
         * Select detects messages on open sockets and calls network
         * I/O to receive and process messages.
         */
        switch (select(32, (fd_set *) & readfds, (fd_set *) NULL,
                 (fd_set *) NULL, (struct timeval *) &rtvobj->timer)) {
        case -1:
            if (errno != EINTR) {    /* error processing */
                perror("Select error in RTV server!\n");
		fprintf(stderr,"RTV: Client connection failed!\n");
		fflush(stderr);
		rtvobj->errno= errno;
		rtvobj->action=STOP_RTV;
                break;
            }
            break;
        case 0:
             rtvobj->timer.tv_sec = 0;
            /*
             * While no messages are being recieved, process
             * field video data. Poll for more video input at
             * least every 1/30th second.
             */
#ifdef TRACE_TIMEOUT_DIAGS
            elapse_time+= rtvobj->timer.tv_usec;
	    if(elapse_time > TRACKING_TIMEOUT)
	    {
		elapse_time=0;
		secs+=5;
		switch(rtvobj->action)
		{
		case START_RTV_RECORD:
		     fprintf(stderr,"Recording..%d seconds!\n",secs);
		     break;
		case START_RTV_PLAY:
		     fprintf(stderr,"Playback..%d seconds\n",secs);
		     break;
		case STOP_RTV:
		     fprintf(stderr,"RTV Stopped...\n");
		     secs=0;
		     break;
		case PAUSE_RTV_PLAY:
		     fprintf(stderr,"Pause Play or Single Step.\n");
		     secs=0;
		     break;
		case PAUSE_RTV_RECORD:
		     fprintf(stderr,"Pause Record.\n");
		     secs=0;
		     break;
		default:
		     fprintf(stderr,"Action=%d .\n",rtvobj->action);
		     break;
		}
	    }
#endif
            switch (rtvobj->action) {
            case START_RTV_RECORD:
                qlev=rtv_get_record_level(rtvobj->ram_fd);
		if(qlev>=2)
		{
                    rtvobj->errno=rtv_record_frame(rtvobj);
                    if(rtvobj->errno<0)
		    {
			fprintf(stderr,"Record: errno=%d\n",rtvobj->errno);
                        rtvobj->action=STOP_RTV_RECORD;
		        if(rtvobj->client!=NULL && rtvobj->errno<0)
			    rtvobj->client->errno=rtvobj->errno;
		    }
                    else if(rtvobj->que_level>5 || qlev>30) {
                        rtv_pause_record(rtvobj);
                        if (rtvobj->audio_mode) 
                            rtv_stop_audio_record(rtvobj->ram_fd);
                        rtv_stop_record(rtvobj);
                        rtvobj->action = STOP_RTV_RECORD;
                        fprintf(stderr,"Disk I/O late - RTV server stopped!\n"); 
			rtvobj->client->errno=rtvobj->errno= RTV_EIO;
                        break;
                    }
		}
                break;
            case START_RTV_PLAY:
                if (rtv_check_queue(rtvobj) < 20) {
                    rtvobj->errno=rtv_play_frame(rtvobj);
		    if(rtvobj->errno<=0) {
			rtvobj->client->errno=rtvobj->errno;
                        rtvobj->client->action=
                            rtvobj->action = STOP_RTV_PLAY;
                        break;
                    }
                    if (advance_frame(rtvobj) == 0) {
                        rtvobj->action = STOP_RTV;
                        rtvobj->rec_offset = 0;
                        rtvobj->errno= RTV_EIO;
			rtvobj->client->errno=rtvobj->errno;
                        fprintf(stderr, "Frame Advance error!\n");
                        break;
                    }
                }
                break;
            case STOP_RTV_PLAY:
                qlev = rtv_check_queue(rtvobj);
                shutdown_delay += 100;
                if (qlev == 0 || shutdown_delay > STOP_DELAY_TIMEOUT) {
                    rtv_stop_play(rtvobj);
                    rtvobj->action = STOP_RTV;
                    shutdown_delay = 0;
                    if (qlev != 0)
                        fprintf(stderr,"Timeout occurred stopping RTV engine!\n");
		    rtvobj->timer.tv_usec = 20000;
                    break;
                }
                break;
            case PAUSE_RTV_RECORD:
            break;
            case PAUSE_RTV_PLAY:
                 rtvobj->queue_full=check_ioque_level(rtvobj, SYNC_IO);
                break;
            case STOP_RTV:
            case STOP_RTV_RECORD:
		rtvobj->timer.tv_usec = 20000;
	    default:
                break;
            }
            break;
        default:
            rtvobj->timer.tv_usec = 5000;
            rtvobj->timer.tv_sec = 1;
            if (FD_ISSET(rtvobj->srvr_sock, &readfds)) {
                for (i = 0; i < MAX_CONNECTIONS; i++) {
                    if (rtvobj->client_sock[i] == 0) {
                        rtvobj->client_sock[i] = 
                            accept_connection(rtvobj->srvr_sock);
                        break;
                    }
                }
                break;
            }
            for (i = 0; i < MAX_CONNECTIONS; i++)
            {
                if (rtvobj->client_sock[i] > 0 && 
                    (FD_ISSET(rtvobj->client_sock[i], &readfds))) {
    		    rtvobj->timer.tv_sec = 0;
                    csock = rtvobj->client_sock[i];
                    memset(cmdbuf, NULL, 80);
                    recv_msg(csock, cmdbuf, 80);
                    if(cmdbuf[0]== (char)NULL)continue;
#ifdef TRACE_DIAGS
printf("csock=%d cmdbuf=%s\n",csock,cmdbuf);
#endif
                    token = strtok(cmdbuf, ":");

                    while (token != NULL) {
                        rtvobj->mode = parse_request(token);
                        switch (rtvobj->mode) {
                        case RTV_DISK_OPEN:
                             token = strtok(NULL, ":");
                             if (token != NULL) {
                                rtvobj->client->errno=
                                rtvobj->errno = open_rtv_files(rtvobj, token);
                             } else {
                                fprintf(stderr, "NO Filename supplied\n");
                                rtvobj->client->errno=
				    rtvobj->errno= RTV_BADARG; 
				}
                             ack_msg(csock,cmdbuf);
                             break;
                        case RTV_SHM_ATTACH:
                             token = strtok(NULL, ":");
                             rtvobj->shmid[i]=atoi(token);
                             client_obj[i]= 
                                (RTV_OBJECT *)shmat(rtvobj->shmid[i],0,0);
                             if(rtvobj->client==NULL)
				 rtvobj->client=client_obj[i];
    		             rtvobj->timer.tv_usec = 5000;
			     if(rtv_enable==0)
			     {
			         ioctl(rtvobj->ccube_fd,ENABLE_RTVMODE,NULL);
				 rtv_enable=1;
			     }
                             /*
                              * Open and initialize "/dev/audio."
                              */
			     if(rtvobj->audio_status==0)
			     {
                                 rtvobj->audioFD = rtv_audio_init();
#ifdef TRACE_DIAGS
			     fprintf(stderr,"rtv_audio_init: audioFD=%d\n",
				 rtvobj->audioFD);
#endif
                                 if (rtvobj->audioFD <= 0) {
                                     rtvobj->audio_status = 0;
                                 } else {
                                     rtvobj->audio_status = rtvobj->audioFD;
				     rtvobj->audio_mode=1;
                                 }
			     }
                             ack_msg(csock,cmdbuf);
                             break;
                        case RTV_GET_FRAME_COUNT:
                             sprintf(msgbuf, "%d", rtvobj->frame_count);
                             send_msg(csock, msgbuf, strlen(msgbuf));
                             break;
                        case RTV_REWIND:
                             ack_msg(csock, cmdbuf);
                             break;
                        case SET_RTV_ATTRIBUTES:
			     if(rtvobj->client==NULL)
                                 rtvobj->client= client_obj[i];
                             parse_attributes(4, &rtvobj->record_attrs);
                             memcpy((caddr_t)&rtvobj->client->attrs,
                                (caddr_t)&rtvobj->record_attrs,sizeof(RTV_ATTRIBUTES));
                             ack_msg(csock, cmdbuf);
                             break;
                        case RTV_CLOSE:
			   {
			     int length;
			     
			     rtvobj->action=STOP_RTV;
			     if(rtvobj->iomode == RTV_PLAY)
			     {
			       lseek(rtvobj->fld_fd,12,SEEK_SET);
			       length=write(rtvobj->fld_fd,rtvobj->play_attrs,sizeof(RTV_ATTRIBUTES));
			     }
                             close(rtvobj->fld_fd);
                             close(rtvobj->disk_fd);
                             ack_msg(csock, cmdbuf);
                             break;
			   }
                        case CLIENT_EXIT:
			     rtvobj->action=STOP_RTV;
    			     rtvobj->timer.tv_usec = 5000;
                             ack_msg(csock, cmdbuf);
	    		     if(rtvobj->audio_status)
			     {
				 flush_audio(rtvobj->audioFD);
				 close(rtvobj->audioFD);
			     }
                             rtvobj->audio_status = 0;
			     stop_record(rtvobj->ccube_fd);
			     stop_play(rtvobj->ccube_fd);
			     if(rtv_enable)
			     {
			         ioctl(rtvobj->ccube_fd,DISABLE_RTVMODE,NULL);
			         rtv_enable=0;
			     }
			     if(rtvobj->client != client_obj[i])
			     {
				 for(n=0;n<MAX_CONNECTIONS;n++)
				 {
				     if(rtvobj->client_sock[n]!=0)
				     {
                                         shmdt((void *)client_obj[n]);
                                         FD_CLR(rtvobj->client_sock[n],&readfds);
                                         close(rtvobj->client_sock[n]);
                                         rtvobj->client_sock[n] = 0;
				     }
				 }
                             }
			     else
			     {
			         shmdt((void *)client_obj[i]);
                                 FD_CLR(rtvobj->client_sock[i], &readfds);
                                 close(rtvobj->client_sock[i]);
                                 rtvobj->client_sock[i] = 0;
			     }
                             client_obj[i]=NULL;
			     rtvobj->client=NULL;
                             break;
                        case RTV_AUDIO_STATUS:
                             sprintf(msgbuf, "%d", rtvobj->audio_status);
                             send_msg(csock, msgbuf, strlen(msgbuf));
                             break;
                        case RTV_AUDIO_MODE:
                            token = strtok(NULL, ":");
                            rtvobj->audio_mode = atoi(token);
                            ack_msg(csock, cmdbuf);
                            break;
                        case RTV_RECORD:
                             token = strtok(NULL, ":");
                            if (token != NULL) {
                                action = parse_request(token);
                                switch (action) {
                                    /* 
                                     * Initialize ccu be and
                                     * start rtv compression.
                                     */
                                case RTV_INIT:
 				     rtvobj->iomode = RTV_RECORD;
                                     if (rtvobj->action != STOP_RTV)
                                        break;
                                     if(rtvobj->client==NULL)
					 rtvobj->client= client_obj[i];
                                     rtv_init_record(rtvobj);
                                     rtvobj->client->action=
                                         rtvobj->action = INIT_RTV_RECORD;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_START:
    		                     rtvobj->timer.tv_usec = 1000;
                                     if (rtvobj->audio_mode) {
                                        rtv_start_audio_record(rtvobj->ram_fd);
                                     }
                                     rtv_start_record(rtvobj);
				     qlev=0;
                                     rtvobj->client->action=
                                         rtvobj->action = START_RTV_RECORD;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_STOP:
                                     rtv_stop_record(rtvobj);
                                     if (rtvobj->audio_mode) {
                                        rtv_stop_audio_record(rtvobj->ram_fd);
                                     }
                                     rtvobj->client->action=
                                         rtvobj->action = STOP_RTV;
    		                     rtvobj->timer.tv_usec = 5000;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_PAUSE:
                                     if(rtvobj->action == START_RTV_RECORD) {
                                        do 
                                        {
                                           qlev=rtv_get_record_level(rtvobj->ram_fd);
#ifdef TRACE_DIAGS
                                           printf("qlev=%d\n",qlev);
#endif
                                        }while(qlev>=2);
                                        rtv_pause_record(rtvobj);
                                        if (rtvobj->audio_mode) 
                                            rtv_stop_audio_record(rtvobj->ram_fd);
                                        rtvobj->client->action=
                                            rtvobj->action = PAUSE_RTV_RECORD;
                                     }
    				     rtvobj->timer.tv_usec = 10000;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_RESUME:
                                     if (rtvobj->audio_mode) {
                                        rtv_start_audio_record(rtvobj->ram_fd);
                                     }
                                     rtvobj->action=START_RTV_RECORD;
                                     rtv_resume_record(rtvobj);
    				     rtvobj->timer.tv_usec = 1000;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_SET_ORIGIN:
                                     parse_attributes(2, &rtvobj->record_attrs);
                                     set_record_org(rtvobj->ccube_fd,&rtvobj->record_attrs);
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_SET_WINDOW:
                                     if(rtvobj->action == START_RTV_PLAY) {
                                         parse_attributes(4,&attrs);
                                         ack_msg(csock,cmdbuf);
                                         break;
                                     }
                                     parse_attributes(4, &rtvobj->record_attrs);
                                     set_record_win(rtvobj->ccube_fd,&rtvobj->record_attrs);
                                     memcpy((caddr_t)&rtvobj->client->attrs,
                                         (caddr_t)&rtvobj->record_attrs,sizeof(RTV_ATTRIBUTES));
                                     ack_msg(csock, cmdbuf);
                                     break;
                                default:
                                    fprintf(stderr,"Invalid token=%s action=%d\n",
                                           token, action);
                                    ack_msg(csock, cmdbuf);
                                    break;
                                }
                            }
                            break;
                        case RTV_PLAY:
                            token = strtok(NULL, ":");
                            if (token != NULL) {
                                action = parse_request(token);
#ifdef TRACE_DIAGS
                                printf("action=%d\n", action);
#endif
                                switch (action) {
                                case RTV_INIT:
				     rtvobj->iomode = RTV_PLAY;
                                     rtvobj->client= client_obj[i];
                                     if (rtvobj->action != STOP_RTV) {
                                         fprintf(stderr,"Invalid request sequence\n");
                                         rtvobj->errno= 
					     rtvobj->client->errno= RTV_BADARG;
                                         ack_msg(csock, cmdbuf);
                                         break;
                                     }
                                     rtv_init_play(rtvobj);
                                     rtvobj->client->action=
                                         rtvobj->action = INIT_RTV_PLAY;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_START:
                                     if (rtvobj->action == INIT_RTV_PLAY ||
                                         rtvobj->action == STOP_RTV) {
                                         rtv_start_play(rtvobj);
                                     }
				     do {
                                         rtvobj->queue_full=
                                             check_ioque_level(rtvobj, SYNC_IO);
                                     }while(!rtvobj->queue_full);
                                     rtvobj->client->action=
                                        rtvobj->action = START_RTV_PLAY;
    				     rtvobj->timer.tv_usec = 1000;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_STOP:
                                    rtvobj->client->action=
                                        rtvobj->action = STOP_RTV_PLAY;
    		                     rtvobj->timer.tv_usec = 5000;
                                    ack_msg(csock, cmdbuf);
                                    break;
                                case RTV_PAUSE:
                                    if(rtvobj->action == START_RTV_PLAY) {
                                        do {
                                        qlev = rtv_check_queue(rtvobj);
                                        } while (qlev > 0);
                                        rtv_pause_play(rtvobj);
                                        rtvobj->client->action=
                                            rtvobj->action = PAUSE_RTV_PLAY;
                                    }
    				    rtvobj->timer.tv_usec = 5000;
                                    ack_msg(csock, cmdbuf);
                                    break;
                                case RTV_RESUME:
                                     if(rtvobj->action== PAUSE_RTV_PLAY) {
                                        rtv_resume_play(rtvobj);
                                        rtvobj->client->action=
                                            rtvobj->action = START_RTV_PLAY;
                                    }
    				    rtvobj->timer.tv_usec = 1000;
                                    ack_msg(csock, cmdbuf);
                                    break;
                                case RTV_SELECT_FRAME:
                                case RTV_PLAY_SINGLE:
                                case RTV_SELECT_FIELD:
                                    token = strtok(NULL, ":");
                                    rtvobj->frame_selection = atoi(token);
                                    if(action==RTV_SELECT_FIELD) {
                                        token = strtok(NULL, ":");
                                        rtvobj->field_selection=atoi(token);
                                    }
    				    rtvobj->timer.tv_usec = 5000;
#ifdef TRACE_DIAGS
                            printf("SELECT: frame_selection=%d next_frame=%d\n",
                                rtvobj->frame_selection, rtvobj->next_frame);
#endif
                                    if (rtvobj->action == INIT_RTV_PLAY ||
                                        rtvobj->action == STOP_RTV) {
                                        rtv_start_play(rtvobj);
                                    }
                                    rtvobj->client->action=
                                        rtvobj->action = RTV_PAUSE_STEP;
                                    if(rtvobj->frame_selection == 0 && 
                                       rtvobj->next_frame > 1)rewind = 1;
                                    if (rtv_check_queue(rtvobj) < 20 && 
					rtvobj->frame_selection >= 0)
                                        rtvobj->client->errno=
					    rtv_play_selected_frame(rtvobj,action);
                                    ack_msg(csock, cmdbuf);
                                    if (rewind) {
					rtvobj->bufno=0;
					rtvobj->seek_ahead=0;
					rtvobj->writelist= rtvobj->writelist->head;
					rtvobj->readlist= rtvobj->readlist->head;
                                        rtv_fill_queue(rtvobj);
                                        rewind = 0;
                                    }
                                    break;
                                case RTV_SHUTTLE:
                                    token = strtok(NULL, ":");
                                    rtvobj->frame_selection = atoi(token);
#ifdef TRACE_DIAGS
                                    printf("SHUTTLE: frame_selection=%d\n",
                                           rtvobj->frame_selection);
#endif
                                    if (rtvobj->action == INIT_RTV_PLAY ||
                                        rtvobj->action == STOP_RTV) {
                                        rtv_start_play(rtvobj);
                                    }
                                    rtvobj->client->action=
                                        rtvobj->action = RTV_PAUSE_STEP;
                                    if (rtv_check_queue(rtvobj) < 20 &&
					rtvobj->frame_selection > 0)
                                        rtvobj->client->errno=
					    rtv_play_selected_frame(rtvobj, RTV_SHUTTLE);
    				    rtvobj->timer.tv_usec = 5000;
                                    ack_msg(csock, cmdbuf);
                                    break;
                                case RTV_SET_WINDOW:
                                     parse_attributes(4, &play_attrs);
				     rtvobj->play_attrs.win_x=play_attrs.win_x;
				     rtvobj->play_attrs.win_y=play_attrs.win_y;
                                     set_play_win(rtvobj->ccube_fd,&rtvobj->play_attrs);
				     if(rtvobj->client!=NULL)
                                         memcpy((caddr_t)&rtvobj->client->attrs,
                                             (caddr_t)&rtvobj->play_attrs,
                                            sizeof(RTV_ATTRIBUTES));
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case RTV_SET_ORIGIN:
                                     parse_attributes(2, &rtvobj->play_attrs);
                                     set_play_org(rtvobj->ccube_fd,&rtvobj->play_attrs);
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case SET_RTV_PLAY_ATTRIBUTES:
                                     parse_attributes(2, &rtvobj->play_attrs);
                                     set_play_org(rtvobj->ccube_fd,&rtvobj->play_attrs);
				     rtvobj->end_frame = rtvobj->play_attrs.last_valid_frame;
                                     ack_msg(csock, cmdbuf);
                                     break;
                                case GET_RTV_PLAY_ATTRIBUTES:
				     sprintf(msgbuf, "%d:%d", rtvobj->play_attrs.first_valid_frame, 
					     rtvobj->play_attrs.last_valid_frame);
				     send_msg(csock, msgbuf, strlen(msgbuf));
                                     break;
                                default:
                                     fprintf(stderr,"Invalid token=%s action=%d\n",
                                           token, action);
                                     ack_msg(csock, cmdbuf);
                                     break;
                                }
                            }
                            break;
                        default:
                            fprintf(stderr,"Invalid token=%s action=%d\n",
                                           token, action);
                            ack_msg(csock, cmdbuf);
                            break;
                        }
                        token = strtok(NULL, ":");
                    }
                    memset(msgbuf, NULL, 80);
                }
            }
            break;
        }
    }
}
#define WIN_X 0
#define WIN_Y 1
#define WIN_H 2
#define WIN_W 3
#define NONINTER 4
#define Q_FACTOR 5
#define MAX_RATE 6
#define FPS 7
#define FIRST 8
#define LAST 9
static char    *argtbl[] = {"win_x", "win_y", "win_h", "win_w",
"interlace", "qfact", "rate", "fps", "first", "last", NULL};

parse_attributes(int nargs, RTV_ATTRIBUTES * attrs)
{
    int             k, j;
    char           *buf;
    char           *arg;
    int             value;
    for (j = 0; j < nargs; j++) {
        k = 0;
        buf = strtok(NULL, ":");
        while (argtbl[k] != NULL) {
            if (strcmp(buf, argtbl[k]) == 0) {
                arg = strtok(NULL, ":");
                value = atoi(arg);
                switch (k) {
                case WIN_X:
                    attrs->win_x = value;
                    break;
                case WIN_Y:
                    attrs->win_y = value;
                    break;
                case WIN_H:
                    attrs->win_h = value;
                    break;
                case WIN_W:
                    attrs->win_w = value;
                    break;
                case NONINTER:
                    attrs->interlace = value;
                    break;
                case Q_FACTOR:
                    attrs->qfactor = value;
                    break;
                case MAX_RATE:
                    attrs->rate = value;
                    break;
                case FPS:
                    attrs->fps = value;
                    break;
		case FIRST:
		    attrs->first_valid_frame = value;
		    break;
		case LAST:
		    attrs->last_valid_frame = value;
		    break;
                }
            }
            k++;
        }
    }
}
parse_request(char *buf)
{
    int             k;
    if (buf == (char *)NULL)
        return (0);
    for (k = 0; k < CMDTBLSIZ; k++) {
        if (cmdtbl[k].opcode == 0)
            return (-1);

        if (strcmp(buf, cmdtbl[k].string) == 0) {
            return (cmdtbl[k].opcode);
        }
    }
}
