/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)ccube_compress.c 1.6 Copyright (c) 1993 Parallax Graphics Inc"

#include <sys/types.h>
#include <sys/param.h>
#include <sys/cred.h>
#include <sys/errno.h>
#include <sys/cmn_err.h>
#include <sys/open.h>
#include <sys/file.h>
#include <sys/kmem.h>
#include <sys/t_lock.h>
#include <sys/stat.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "tvtwo.h"

#define COMPRESSION 1
#define CCUBE_STATE_TIMEOUT 1
/*
#define TRACE_VINTR
 */

void ccube_svc_v_chan(tvtwo_softc *softc)
{
register int sid;
static int trace=0;
static int seconds=0;
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
QUEUE_INFO *playback_qinfo;

    if ((softc->istat & 0x0100) != 0x0100) 
        softc->v_chan_state = 1; /* Polarity of video */
    else    softc->v_chan_state = 0;

#ifdef TRACE_VINTR
    if(trace++ == 30)
    {
        seconds++;
        cmn_err(CE_WARN," v_chan: secs=%d state=%d\n",
		seconds,softc->ccube_comp_state);
        trace=0;
    }
#endif
    switch(softc->ccube_comp_state)
    {
    case DST_IDLE:
        break;
    case DST_BUSY:
        softc->ccube_comp_state = DST_IOLATE;
        break;
    case DST_IOLATE:
        softc->ccube_comp_state = DST_IOLATE_LATE;
        break;
    case DST_IOLATE_LATE:
	softc->overflow=1;
        softc->ccube_comp_state=DST_FIELD_DONE;
        break;
    case DST_START:
         if( softc->Vidcount++ < 2)
            break;
         if(softc->v_chan_state==1)
            break;
	 softc->ccube_comp_state=DST_READY;
    case DST_READY:
    case DST_FIELD_DONE:
        /*
         * Should be ready to get/put a new field , PAUSE, or STOP.
         */
         if(softc->rtv_dma_state==RECORD)
	 {
             if(softc->record_action == RECORD)
                  if(softc->overflow==0)
		  {
		      softc->field_polarity= softc->v_chan_state;
		      get_next_field(softc);
		  }
		  else
		  {
		      softc->overflow--;
                      softc->ccube_comp_state=DST_FIELD_DONE;
		      return;
		  }
             else if(softc->record_action == PAUSE)
	     {
                  softc->ccube_comp_state=DST_FIELD_DONE;
		  break;
	     }
             else if(softc->record_action == STOP && softc->field_polarity==0)
             {
    /* 
     *  Disable V Channel interrupt to stop compression.
     */
                 softc->imask &= ~ _Int_Vid;
                 XV_WREG(softc->regs,IntMask.l, softc->imask);
                 softc->ccube_comp_state= DST_IDLE;
		 break;
             }
	 }
        break;
     }
}


void ccube_svc_c3(tvtwo_softc *softc )
{

int rval, count;

    rval= process_c3_intr(softc);
    switch (rval) {
    case -1:
        softc->bmask &= ~ _Int_Dma;
        softc->imask &= ~ ( _Int_CCube | _Int_Vid | _Int_Wid );
        XV_WREG(softc->regs,IntMask.l, softc->imask);
        XV_WREG(softc->regs,BIntMask.l, softc->bmask);
        softc->ccube_error= EIO;
        if(softc->rtv_dma_state == RECORD)
        {
         ccube_finish_compress(softc);
        }
        else
        {
        ccube_finish_decomp(softc);
        }
    break;

    case 0:
        softc->bmask &= ~ _Int_Dma;
        softc->imask &= ~ _Int_CCube ;
        if(softc->rtv_dma_state == RECORD)
             ccube_finish_compress(softc);
        else
             ccube_finish_decomp(softc);
    break;

    case 1: 
         break;
    }
}

void ccube_svc_dma(tvtwo_softc *softc )
{

int rval, count;

    rval= process_c3_intr(softc);
    switch (rval) {
    case -1:
        ccube_status(softc);
        softc->bmask &= ~ _Int_Dma;
        softc->imask &= ~  _Int_CCube ;
        softc->ccube_error= EIO;
        if(softc->rtv_dma_state == RECORD)
        {
             ccube_finish_compress(softc);
        }
        else
        {
             ccube_finish_decomp(softc);
        }
    break;

    case 0:
        ccube_status(softc);
        softc->imask &= ~ _Int_CCube ;
        softc->bmask &= ~ _Int_Dma;
        if(softc->rtv_dma_state == RECORD)
         ccube_finish_compress(softc);
        else
             ccube_finish_decomp(softc);
    break;

    case 1: 
         break;
    }
}

disable_interrupts(tvtwo_softc *softc)
{
    XV_WREG(softc->regs,IntMask.l,0x08000);
    XV_WREG(softc->regs,BIntMask.l,0);
}
enable_w_chan_intr(tvtwo_softc *softc)
{
    softc->imask |= _Int_Wid;
    XV_WREG(softc->regs,IntMask.l,softc->imask);
    XV_WREG(softc->regs,BIntMask.l,softc->bmask);
}
enable_v_chan_intr(tvtwo_softc *softc)
{
    softc->imask |=  _Int_Vid  ;
    XV_WREG(softc->regs,IntMask.l,softc->imask);
    XV_WREG(softc->regs,BIntMask.l,softc->bmask);
}

/*
#define TRACE_WINTR
 */
ccube_svc_w_chan(tvtwo_softc *softc)
{
register int sid,video_sync,audio_sync;
QUEUE_INFO *playback_qinfo;
FIELD_INFO *field ;
SHM_FIELD_QUE *shm_que = (SHM_FIELD_QUE *)softc->shm_que;
register int polarity;
static int trace=0,seconds=0;
static int track_state=0;

    if ((softc->istat & 0x0200) != 0x0200) 
        softc->w_chan_state = 0;
    else    softc->w_chan_state = 1;

#ifdef TRACE_WINTR
     if(trace++ == 60)
     {
        seconds++;
        cmn_err(CE_WARN," w_chan: secs=%d state=%d\n",
	    seconds,softc->ccube_decomp_state);
        trace=0;
     }
#endif

    switch(softc->ccube_decomp_state)
    {
    case DST_IDLE:
        break;
    case DST_BUSY:
        softc->ccube_decomp_state = DST_IOLATE;
        break;
    case DST_IOLATE:
        softc->ccube_decomp_state = DST_IOLATE_LATE;
        break;
    case DST_IOLATE_LATE:
        softc->ccube_error = EIO;
        softc->ccube_decomp_state = DST_IDLE;
        break;
    case DST_READY:
    case DST_FIELD_DONE:
    /*
     * Should be ready to put a new field,
     * Set state = DST_BUSY and process next field.
     */
	 if(softc->rtv_dma_state==PLAY)
	 {
             if(softc->play_action == PLAY)
             {
                 playback_qinfo= (QUEUE_INFO *)&shm_que->playback_streams[0];
                 if(playback_qinfo->level>0)
                     put_next_field(softc);
	     }
             else if(softc->play_action == PAUSE && softc->field_polarity == 0)
             {
                 softc->ccube_decomp_state=DST_FIELD_DONE;
		 break;
             }
	     else if(softc->play_action == STOP && softc->field_polarity==0)
	     {
        /* 
         *  Disable V & W Channel interrupts.
         */
                 softc->imask &= ~ _Int_Vid;
                 XV_WREG(softc->regs,IntMask.l, softc->imask);
		 /*
		  * Turn of V Channel.
    		 XV_WREG(softc->regs,VCtl.l, 0x80808);
    		 XV_WREG(softc->regs,VSqzX.l, 0);
    		 XV_WREG(softc->regs,VSqzY.l, 0);
		  */
                 softc->ccube_decomp_state= DST_IDLE;
		 break;
	     }
             else
                 softc->ccube_decomp_state=DST_FIELD_DONE;
        }
        break;
    }
}
