/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */
  
#pragma ident "@(#)rtvmod.c	1.1 Copyright (c) 1993 Parallax Graphics Inc"
  
#include <sys/types.h>
#include <sys/sysmacros.h>
#include <sys/stat.h>           /* S_IFCHR */
#include <sys/stropts.h>
#include <sys/stream.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/kmem.h>
#include <sys/strsun.h>
#include <sys/modctl.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <sys/tihdr.h>
#include <sys/dlpi.h>
#include <sys/debug.h>
#include "krtv.h"


char _depends_on[] = "drv/tvtwo";


#define MAX_AUDIO_QUEUE 512
#define MAX_NET_BUFSIZE (1024)
#define ALLIGN32(ptr) (ptr)
#define AUDIO_RAM_SIZE MAX_AUDIO_QUEUE*1024
#define QUEUE_AUDIO_FRAME 0x101
#define I_SET_AUDIO_RATE 0x102


#define T_Udr_Size    (sizeof(struct T_unitdata_req))

extern SHM_FIELD_QUE *global_shm_que;

typedef struct {
	int frame_no;
	int size;
	int sync_clock;
	caddr_t addr;
	}AUDIO_FRAME;

typedef struct {
	int level;
	caddr_t base;
	int mem_offset;
	AUDIO_FRAME *head;
	AUDIO_FRAME *tail;
	AUDIO_FRAME *input;
	AUDIO_FRAME *output;
	AUDIO_FRAME audio_frame[MAX_AUDIO_QUEUE];
	}AUDIO_QUEUE_INFO;


static int que_level;
static int current_device;
static int action_flag=1;
static u_char audio_hdr[4];
#ifdef DEBUG
static rtaudio_debug = 1;
#define	_STMT(a) a
#define	DEBUGF(args) _STMT(if (rtaudio_debug) cmn_err args)
#else
#define	DEBUGF(args)
#endif

static int
rtaudio_open(queue_t *rq, dev_t *dev, int flag, int sflag, cred_t *cr);


static int
rtaudio_close(queue_t *q, int flag, cred_t *creadp);

static int
rtaudio_wput(queue_t *q, mblk_t *mp);

static int
rtaudio_wsrv(queue_t *q);

static int
rtaudio_rsrv(queue_t *q);


static int
rtaudio_rput(queue_t *q, mblk_t *mp);


/*
 * Streams declarations
 */
struct module_info	rtaudio_info = {
	0x2,		/* module ID number */
	"rtaudio",	/* module name */
	0,		/* min packet size accepted */
	INFPSZ,		/* max packet size accepted */
	0,		/* hi-water mark */
	0		/* lo-water mark */
};

/*
 * Read queue information
 */
static struct qinit rtaudio_rinit = {
	rtaudio_rput,    /*lower read procedure */ 
	rtaudio_rsrv,    /*lower service procedure */
	rtaudio_open,	/* open procedure */
	rtaudio_close,	/* close procedure */
	nodev,		/* for 3bnet only */
	&rtaudio_info,	/* module information */
	NULL		/* module statistics */
};
/*
 * Write queue information
 */
static struct qinit rtaudio_winit = {
	rtaudio_wput,	/* put procedure */
	rtaudio_wsrv, 	/* service procedure */
	rtaudio_open,	/* open procedure */
	rtaudio_close,	/* close procedure */
	nodev,	/* for 3bnet only */
	&rtaudio_info,	/* module information */
	NULL		/* module statistics */
};

/*
 * Stream configuration
 */
static struct streamtab rtaudio_strinfo = {
	&rtaudio_rinit,	/* qinit for read queue */
	&rtaudio_winit,	/* qinit for write queue */
	NULL,		/* mux qinit for read */
	NULL	/* mux qinit for write */
};


int record_count=0;


typedef struct {
	int id;	/* id of link */
	unsigned ltype;	/* persistent or non-persistent link */
	queue_t *netq;	/* linked write queue on lower side of mux */
	queue_t *ctlq;	/* controlling write queue on upper side of mux */
}MUX_STATE;

typedef struct {
	queue_t *rtv_qptr;
	MUX_STATE rtaudio_state;
}RTV_MUX;



/*
 * We only have one instance for this driver, so we don't bother
 * using the ddi_soft_state* routines.
 */
typedef struct {
	dev_info_t	*devi;		/* my devinfo handle */
	queue_t *rq;
	queue_t *wq;
	queue_t *net_rq;
	queue_t *net_wq;
	int	rtv_open;
	int	rtv_mode;
	int	rtv_dst;
	int	recv_ready;
	int	xmit_ready;
	int	stream_id;
	int	monitor;
	int	frame_no;
	char    dst_addr[20];
	int	dst_leng;
	int    AudioFrameSize;
	AUDIO_QUEUE_INFO *audio_que;     
} RTV_SOFTC;

RTV_SOFTC *glb_rtv_softc ;




static struct fmodsw rtv_fsw = {
	"rtaudio",
	&rtaudio_strinfo,
	D_NEW | D_MTQPAIR | D_MP
};

extern struct mod_ops mod_strmodops;

static struct modlstrmod rtv_strmod = {
	&mod_strmodops,	
	"RTV-XVideo STREAMS Mux Module",	/* Module title */
	&rtv_fsw	
};

static struct modlinkage modlinkage = {
	MODREV_1, (void *) &rtv_strmod, NULL
};

_init(void)
{
int e;
	if ((e = mod_install(&modlinkage)) != 0) return (e);
}


/*
 * Build the unload transaction, if possible.
 */

_fini(void)
{
	return (mod_remove(&modlinkage));
}

_info(struct modinfo * modinfop)
{
	return (mod_info(&modlinkage, modinfop));
}

static RTV_SOFTC *rtaudio_init()
{
	RTV_SOFTC *softc;
	mblk_t  *mp;
	caddr_t base;
	AUDIO_QUEUE_INFO *audio_que;
	register int i,cnt;

	softc = (RTV_SOFTC *)kmem_zalloc(sizeof(RTV_SOFTC),KM_SLEEP);
	if(softc == NULL)return((RTV_SOFTC *)NULL);
	softc->rtv_open=0;
	softc->monitor=1;
	softc->AudioFrameSize= 8*1024;
	audio_que= softc->audio_que= 
	    (AUDIO_QUEUE_INFO *)kmem_zalloc(sizeof(AUDIO_QUEUE_INFO),KM_SLEEP);
	audio_que->base= (caddr_t)kmem_zalloc(AUDIO_RAM_SIZE,KM_SLEEP);

	audio_que->mem_offset=0;
	audio_que->output = audio_que->input=audio_que->head= 
	    &audio_que->audio_frame[0];
	audio_que->tail= &audio_que->audio_frame[MAX_AUDIO_QUEUE-2];
	return(softc);
}




static int 
rtaudio_open(queue_t *rq, dev_t *devp, int flag, int sflag, cred_t *credp)
{
    dev_t device;
    RTV_SOFTC *rtv_softc;
    register int i;

    if(rq->q_ptr)
	return(0);

    rtv_softc = rtaudio_init();
    if(rtv_softc==NULL) return(-1);
    rtv_softc->rtv_open=1;
    rtv_softc->rq=rq;
    rtv_softc->wq=WR(rq);
    rq->q_ptr = WR(rq)->q_ptr= rtv_softc;
    rtv_softc->rtv_mode=0;
    audio_hdr[0]=0;
    audio_hdr[1]=0;
    audio_hdr[2]=0x89;
    audio_hdr[3]=0xad;
    global_shm_que->aud_wq= (int *)rtv_softc->wq;
    global_shm_que->aud_rq= (int *)rtv_softc->rq;
    global_shm_que->audio_shm= (int *)rtv_softc->audio_que;
    glb_rtv_softc= rtv_softc;
    qprocson (rq);
    return(0);

}
static int
rtaudio_close(queue_t *q, int flag, cred_t *creadp)
{
	RTV_SOFTC *rtv_softc;
	register mblk_t *mp;
	register int i;

	ASSERT(q);
	rtv_softc= (RTV_SOFTC *)q->q_ptr;
	ASSERT(rtv_softc);
	flushq(q,FLUSHALL);

	qprocsoff(q);
	q->q_ptr = WR(q)->q_ptr = NULL;

        if(rtv_softc->audio_que->base!=NULL)
	    kmem_free(rtv_softc->audio_que->base,AUDIO_RAM_SIZE);
	if(rtv_softc->audio_que!=NULL)
	    kmem_free(rtv_softc->audio_que,sizeof(AUDIO_QUEUE_INFO));
	if(rtv_softc!=NULL)
	    kmem_free(rtv_softc,sizeof(RTV_SOFTC));
	return(0);
}
struct address {
	int     ad_len;
	caddr_t ad_addr;
};
static
do_reply(queue_t *q,int type,mblk_t *mp)
{
struct iocblk *iocbp;

	mp->b_datap->db_type= type;
	mp->b_wptr = mp->b_rptr + sizeof(struct iocblk); 
	iocbp = (struct iocblk *)mp->b_rptr;
	iocbp->ioc_count=0;
	freemsg(mp->b_cont);
	mp->b_cont=NULL;
	qreply(q,mp);
}

static
mblk_t *generate_udr(RTV_SOFTC *softc)
{
    struct T_unitdata_req *udr;
    mblk_t *mp;

	 mp= allocb(T_Udr_Size+softc->dst_leng,BPRI_MED);
	 if(mp == NULL)
	 {
	     printf("allocb failed\n");
	     return((mblk_t *)NULL);
	 }
	 mp->b_datap->db_type=M_PROTO;
	 udr= (struct T_unitdata_req *)mp->b_rptr;
	 udr->PRIM_type=T_UNITDATA_REQ;
	 udr->DEST_length= softc->dst_leng;
	 udr->DEST_offset= T_Udr_Size;
	 udr->OPT_length=0;
	 udr->OPT_offset=0;
	 mp->b_wptr += T_Udr_Size;
	 bcopy(softc->dst_addr,(caddr_t)mp->b_wptr, softc->dst_leng);
	 mp->b_wptr+= softc->dst_leng;
	 return(mp);
}

#define QNAME(q)        ((q)->q_qinfo->qi_minfo->mi_idname)


static int 
rtaudio_wput(queue_t *q, mblk_t *mp)
{
    RTV_SOFTC *rtv_softc;
    caddr_t optr;
    mblk_t *fmp,*ackm,*smp;
    struct address *ap;
    char *bptr,*host;
    dblk_t  *dp;
    int id,leng;
    int success;
    static int audio_trace=0;
    int i,rec_size, prim;
    static int fldcnt=0;
    int records,sample_size;
    struct iocblk *iocp;

    if(q->q_ptr)
	rtv_softc= (RTV_SOFTC *)q->q_ptr;

	switch(DB_TYPE(mp))
	{
	case M_IOCTL: 
	{
	    struct iocblk *iocbp;
	    struct linkblk *linkp;
	    struct copyreq  *cqp;
	    caddr_t ip, base, addr;
	    /*
	     * ioctl. Only channel 0 can do ioctls. Two 
	     * calls are recognized: LINK, and UNLINK
	     */
	    iocbp = (struct iocblk *)mp->b_rptr;
	    switch(iocbp->ioc_cmd) 
	    {
	    case I_STOP_AUDIO:
		 glb_rtv_softc->xmit_ready=0;
		 glb_rtv_softc->recv_ready=0;
		 do_reply(q,M_IOCACK,mp);
		 break;
	    case I_START_AUDIO:
		 glb_rtv_softc->xmit_ready=1;
		 glb_rtv_softc->recv_ready=1;
		 do_reply(q,M_IOCACK,mp);
		 break;
	    case I_LINK_AUDIO:
		 do_reply(q,M_IOCACK,mp);
		 if(global_shm_que->wq!=NULL)
                     glb_rtv_softc->net_wq= (queue_t *)global_shm_que->wq;
		 break;
	    default:
		putnext(q,mp);
		break;
	    }
	    break;
	}
	default:
	    putnext(q,mp);
	    break;
	}
}
static int send_audio_frame(RTV_SOFTC *softc,queue_t *q)
{
	AUDIO_FRAME *audio_frame;
	AUDIO_QUEUE_INFO *audio_qinfo=(AUDIO_QUEUE_INFO *)softc->audio_que;
	mblk_t *amp;
	register int msg_count;
	caddr_t bptr;

	audio_frame= audio_qinfo->output;
	msg_count= audio_frame->size/1024;
	if(msg_count==8)
	{
	    bptr= (caddr_t)audio_frame->addr;
	    while(msg_count>0)
	    {
	        amp=allocb(1024,BPRI_MED);
	        if(amp != NULL)
	        {
		    bcopy(bptr,(caddr_t)amp->b_wptr,1024);
		    amp->b_wptr+= 1024;
		    putnext(q,amp);
	            bptr+= 1024;
		    msg_count--;
	        }
	        else
	           cmn_err(CE_WARN,"! SEND_AUDIO_FRAME: allocb failed\n");
	    }
	}
	audio_qinfo->output++;
	audio_qinfo->level--;
	if(audio_qinfo->output>= audio_qinfo->tail)
	    audio_qinfo->output=audio_qinfo->head;

}
static int rtaudio_wsrv(queue_t *q)
{
    mblk_t *mp;
    RTV_SOFTC *rtv_softc;
    static int trace=0;
    
    ASSERT(q);
    rtv_softc= (RTV_SOFTC *)q->q_ptr;
    ASSERT(rtv_softc);
    while(mp = getq(q))
    {
	switch(DB_TYPE(mp))
	{
	case M_IOCTL: 
	{
	    struct iocblk *iocbp;
	    iocbp = (struct iocblk *)mp->b_rptr;
	    switch(iocbp->ioc_cmd) 
	    {
	    case QUEUE_AUDIO_FRAME:
		 freemsg(mp);
		 send_audio_frame(rtv_softc,q);		 
		 break;
	    case I_SET_AUDIO_RATE:
		 freemsg(mp);
                 if(global_shm_que->frame_rate==25)
	             rtv_softc->AudioFrameSize= 6*1024;
                 else
	             rtv_softc->AudioFrameSize= 8*1024;
		 break;
	    default: 
	         putnext(q,mp);
		 break;
	    }
	}
	    break; 
	default:
	     putnext(q,mp);
	     break;
	}
    }
    return(0);
}

copy_audio_to_ram(RTV_SOFTC *softc,mblk_t *mp,int leng)
{
	AUDIO_FRAME *audio_frame;
	AUDIO_QUEUE_INFO *audio_qinfo;
	static int offset=0;
	caddr_t audio_addr;

	audio_qinfo= (AUDIO_QUEUE_INFO *)softc->audio_que;
	if(offset==0)
	{
	    if(audio_qinfo->mem_offset+ softc->AudioFrameSize>= AUDIO_RAM_SIZE)
	    {
	         audio_qinfo->mem_offset=0;
	    }
	}

	audio_addr= audio_qinfo->base+audio_qinfo->mem_offset+offset;
	bcopy((caddr_t)mp->b_rptr,audio_addr,leng);
	offset+=leng;
	if(offset<softc->AudioFrameSize)return(0);

	audio_addr= audio_qinfo->base+audio_qinfo->mem_offset;
	audio_qinfo->mem_offset+= offset;

	audio_frame= audio_qinfo->input;
	audio_frame->addr= audio_addr;
	audio_frame->size= offset;
	audio_frame->sync_clock= global_shm_que->clock_time;
	offset=0;
	audio_frame->frame_no = softc->frame_no++;
	audio_qinfo->input++;
	audio_qinfo->level++;
	if(audio_qinfo->input>= audio_qinfo->tail)
	{
	        audio_qinfo->input= audio_qinfo->head;
	}
	
	return(0);
}

static int
rtaudio_rput(queue_t *q,mblk_t *mp)
{
    RTV_SOFTC *rtv_softc;
    static int trace=0;

	rtv_softc= (RTV_SOFTC *)q->q_ptr;
	switch(DB_TYPE(mp))
	{
	case M_DATA:
#ifdef RTV_NETWORK_SUPPORT
	    if(rtv_softc->xmit_ready && rtv_softc->net_wq!=NULL)
	    {
	        mp->b_datap->db_type=M_PROTO;
	        putq(rtv_softc->net_wq,mp);
	    }
	    else
		freemsg(mp);
#else
	    if(global_shm_que->audio_mode==RECORD)
		copy_audio_to_ram(rtv_softc,mp,msgsize(mp));
	/* 
	 *Send messages to audio output when monitor is on.
	*/
	    if(global_shm_que->audio_monitor)
		    putq((queue_t *)global_shm_que->aud_wq,mp);
	    else
		    freemsg(mp);
#endif
	    break;
	default:
	    putnext(q,mp);
	    break;
	}
	return(0);
}
static int rtaudio_rsrv(queue_t *q)
{
	register int i;
	mblk_t *mp;
	register dblk_t *dp;
	caddr_t base;

	while(mp = getq(q)) 
	{
	        putnext(q,mp);
	}
}

