#pragma ident "@(#)mpg.h	1.14 93/05/06 SMI"

/*
 * Interface definitions for multi planegroup (mpg) ddx interface.
 */

#ifndef MPG_H
#define MPG_H

#include "X.h"
#include "validate.h"
#include "gcstruct.h"
#include "windowstr.h"
#include "scrnintstr.h"
#include "server/ddx/mi/mibstore.h"

/* max number of planegroups per device */
#define MPG_MAX_PGS 32 

typedef enum {
    MPG_OTHER=0, /* other planegroup */
    MPG_VISIBLE, /* visible planegroup */
    MPG_DRAWABLE /* drawable planegroup */
} mpgType;

typedef enum {
    MPG_NOOP=0,  /* virtual plane */
    MPG_DRAW, 	 /* drawable by client */
    MPG_FILL, 	 /* fill plane */
    MPG_COPY,	 /* virtual-physical separation */
    MPG_FILL_WID /* fill plane with window id */
} mpgOp;

typedef enum {
    MPG_SEPARATE,  /* virtual-physical separation */
    MPG_REPAIR,	    /* repair area, used where gravity causes damage */
    MPG_COVER,	    /* cover siblings and ancestors */
    MPG_EXPOSE	    /* expose siblings and ancestors */
} mpgClipOp;

typedef enum {
    MPG_CURSOR_SHARED,	 /* cursor shares planegroups with windows */
    MPG_CURSOR_DEDICATED /* cursor has dedicated planegroups */
} mpgCursorType;

typedef int mpgValue;
typedef unsigned int mpgPlanes;	 /* bit encoding of planegroup ids */
typedef unsigned int mpgPlaneId; /* planegroup ids are enumerated per device */
typedef RegionPtr *RegionListPtr;

typedef struct _mpgData {
    pointer devPrivate;
} mpgData, *mpgDataPtr;

typedef enum {
    MpgColorLutPartitioned = 0,
    /* Rest are reserved for future use */
} MpgColorLutConfig;

typedef struct _mpgPgInfo {  /* per plane */
    mpgPlaneId id;           /* what planegroup */
    mpgPlanes plane;         /* bit encoded planegroup */
    mpgType type;            /* planegroup type */
    mpgOp op;                /* planegroup preparation */
    DevUnion arg;            /* argument for op */
    struct _mpgPgInfo *next; /* next planegroup */
} mpgPgInfo, *mpgPgInfoPtr;

/* There are certain screen functions that are planegroup dependent.
 * These functions are stored per planegroup. After the mpg wrapper
 * function initializes any private data, it may call the
 * wrapped screen function.
 */
typedef struct _mpgFuncs { 
    void (* GetImage)();               /* pDrawable, sx, sy, w, h, format,
                                        * planemask, pdestbits */
    void (* GetSpans)();               /* pDrawable, wMax, ppt, pwidth,
                                        * nspans, pdstbits */
    void (* MpgScreenSetup)();         /* pScreen */
    Bool (* CreateColormap)();         /* pMap */
    void (* DestroyColormap)();        /* pMap */
    void (* InstallColormap)();        /* pMap */
    void (* UninstallColormap)();      /* pMap */
    void (* ResolveColor)();           /* pVisual */
    void (* CopyWindow)();             /* pWin, ptOldOrg, prgnSrc */
    Bool (* CreateWindow)();           /* pWin */
    Bool (* DestroyWindow)();          /* pWin */
    Bool (* RealizeWindow)();          /* pWin */
    Bool (* PositionWindow)();         /* pWin, x, y */
    Bool (* UnrealizeWindow)();        /* pWin */
    void (* PaintWindowBorder)();      /* pWin, pRgn, which */
    void (* PaintWindowBackground)();  /* pWin, pRgn, which */
    Bool (* ChangeWindowAttributes)(); /* pWin, mask */
    miBSFuncPtr BSfuncs;	       /* backing store functions */
} mpgFuncs, *mpgFuncPtr;

typedef struct _mpgInfo {   /* per visual */
    mpgPlaneId did;	    /* drawable planegroup id */
    mpgPlanes allPlanes;    /* all planegroups for this visual */
    mpgPlanes drawPlanes;   /* drawable+clipping planegroups */
    mpgFuncPtr mpgFuncs;    /* planegroup dependent screen functions */
    mpgDataPtr mpgData;	    /* devPrivate of active planegroup */
    mpgPgInfoPtr mpgPgList; /* shared planegroup list */
    CARD32  whitePixel;
    CARD32  blackPixel;
} mpgInfoRec, *mpgInfoPtr;

typedef struct _mpgPerVisInfo { 
    VisualID vid;		
    mpgInfoPtr mpgInfo;
} mpgPerVisInfo, *mpgPerVisInfoPtr;

typedef struct _mpgPerDepthInfo {
    int	depth;
    Bool	(* createGC)();
    PixmapPtr	(* createPixmap)();
    Bool	(* destroyPixmap)();
    void	(*GetImage)(); /* for pixmaps, otherwise per visual */
    void	(*GetSpans)(); /* likewise */
} mpgPerDepthInfo, *mpgPerDepthInfoPtr;

typedef struct _mpgExposed {
    mpgPlaneId id;     /* planegroup id */
    RegionPtr exposed; /* exposed region */
    struct _mpgExposed *next;
} mpgExposedRec, *mpgExposedPtr;
 
typedef struct _mpgGravity {
    mpgPlanes oldPlanes;
    mpgPlanes newPlanes;
    mpgPlanes copyPlanes;
    RegionPtr pOldRegions[MPG_MAX_PGS];  /* old regions */
    RegionPtr pNewRegions[MPG_MAX_PGS];  /* new regions */
    RegionPtr pCopyRegions[MPG_MAX_PGS]; /* regions to copy */
} mpgGravityRec, *mpgGravityPtr;
 
typedef struct _mpgNode {
    mpgPgInfoPtr pginfo;  /* shared planegroup info */
    RegionPtr clipList;   /* -border-children clip */
    RegionPtr borderClip; /* +border+children clip */
    struct _mpgNode *next;
} mpgNodeRec, *mpgNodePtr;
 
typedef struct _mpgValidate {
    union _valUnion { 		     /* validate union */
        struct _before { 	     /* before validate */
            Bool ancestor; 	     /* affected ancestor */
            Bool visChanged;         /* visibility state is changed */
            Bool drawCovered;        /* drawable client region is covered */
            Bool drawExposed;        /* drawable client region is exposed */
            Bool drawBorderCovered;  /* drawable border region is covered */
            Bool drawBorderExposed;  /* drawable border region is exposed */
        } before;
        struct _after { 	     /* after validate */
            RegionRec exposed;       /* exposed drawable client region */
            RegionRec borderExposed; /* exposed drawable border region */
        } after;
    } valunion;
    mpgExposedPtr otherExposed;
} mpgValidateRec, *mpgValidatePtr;
 
typedef struct _mpgPrivGC { /* per gc */
    GCOps *ops;
    GCFuncs *funcs;
} mpgPrivGC, *mpgPrivGCPtr;

typedef struct _mpgPrivWin { /* per window */
    mpgInfoPtr mpgInfo;	     /* shared per visual planegroup info */
    mpgPlanes treePlanes;    /* own+children's total planes */
    mpgValidatePtr valdata;  /* mpg window validation data */
    mpgNodePtr pglist; 	     /* shared and unique planegroup info */
} mpgPrivWin, *mpgPrivWinPtr;

typedef struct _mpgPrivScreen { /* per screen */
    VisualID rootVID;		/* internal root visual id */
    mpgPlanes dispPlanes;       /* displayable planegroups */
    int numPlanegroups;         /* total number of planegroups */
    PixmapPtr pixmap;           /* pointer to drawable for each planegroup */
    Bool (*CloseScreen)();      /* screen close function */
    void (*mpgPGSwitch)();      /* planegroup switch function */
    void (*CopyPlanes)();       /* custom screen planes copy function */
    void (*PreparePlanes)();    /* custom window planes preparation function */
    mpgPerVisInfoPtr mpgVisInfo;
    mpgPerDepthInfoPtr mpgDepthInfo;
} mpgPrivScreen, *mpgPrivScreenPtr;

extern int mpg_max_pgs;
extern int mpgGCPrivateIndex;
extern int mpgWindowPrivateIndex;
extern int mpgScreenPrivateIndex;
extern Bool noMinExp; /* no optimized damage */
extern Bool altview; /* alternative view */

extern void mpgPrepareWid(WindowPtr pWin);

/* exported functions */ 
extern void mpgGetImage();
extern void mpgGetSpans();

extern Bool mpgCreateGC();
extern int mpgValidateTree();

extern Bool mpgScreenInit();
extern Bool mpgCloseScreen();
extern Bool mpgGetScreenState();
extern Bool mpgInsertPlanegroup();

extern Bool mpgDestroyPixmap();
extern PixmapPtr mpgCreatePixmap();
extern RegionPtr mpgPixmapToRegion();

extern void mpgResolveColor();
extern Bool mpgCreateColormap();
extern void mpgDestroyColormap();
extern void mpgInstallColormap();
extern void mpgUninstallColormap();
extern int  mpgListInstalledColormaps(); 

extern void mpgCopyWindow();
extern void mpgPaintWindow();
extern Bool mpgCreateWindow();
extern Bool mpgDestroyWindow();
extern Bool mpgRealizeWindow();
extern Bool mpgPositionWindow();
extern Bool mpgUnrealizeWindow();
extern void mpgWindowExposures();
extern Bool mpgChangeWindowAttributes();
 
extern void mpgSaveAreas();
extern void mpgRestoreAreas();
extern DrawablePtr mpgDrawable();
extern void mpgCheckAndDropCursor();
extern mpgInfoPtr getMpgInfoFromVisual();

#define mpg_union(a,b) ((a)|(b)) 
#define mpg_bit_encoded(i) (1<<(i))
#define mpg_intersect(a,b) ((a)&(b))
#define mpg_subtract(a,b) ((a)&(~(b)))
#define mpg_subset(a,b) ((a)==((a)&(b)))

#define ReshapeExposed ((mpgExposedPtr)1)
#define mpgUnmapValData ((mpgValidatePtr)1)
#define NullRegionList ((RegionListPtr)NULL)

#define mpg_priv_win(pWin) ((mpgPrivWinPtr)((pWin)->devPrivates[mpgWindowPrivateIndex].ptr))
#define mpg_priv_scr(pScreen) ((mpgPrivScreenPtr)((pScreen)->devPrivates[mpgScreenPrivateIndex].ptr))

#define mpg_info(pWin) (mpg_priv_win(pWin)->mpgInfo)

#define mpg_pg_id(pg) ((pg)->pginfo->id)
#define mpg_pg_plane(pg) ((pg)->pginfo->plane)
#define mpg_pg_type(pg) ((pg)->pginfo->type)
#define mpg_pg_op(pg) ((pg)->pginfo->op)
#define mpg_pg_val(pg) ((pg)->pginfo->arg.uval)
#define mpg_pg_list(pWin) (mpg_priv_win(pWin)->pglist)

#define mpg_valdata(pWin) (mpg_priv_win(pWin)->valdata)
#define mpg_ancestor(pWin) (mpg_valdata(pWin)->valunion.before.ancestor)

#define mpg_drawable_id(pWin) (mpg_info(pWin)->did)
#define mpg_window_id(pWin) (mpg_priv_win(pWin)->wid)
#define mpg_all_planes(pWin) (mpg_info(pWin)->allPlanes)
#define mpg_draw_planes(pWin) (mpg_info(pWin)->drawPlanes)
#define mpg_tree_planes(pWin) (mpg_priv_win(pWin)->treePlanes)
#define mpg_view_planes(pWin) mpg_subtract(mpg_all_planes(pWin), mpg_draw_planes(pWin))

#ifndef NOTDEF
/*
 * For performance and simplification reasons,
 * all draw planes are physical and there's only one
 * visible plane per window assigned to the first mpgNode.
 */
#define mpg_vdraw_planes(pWin) ((mpgPlanes)0)
#define mpg_pdraw_planes(pWin) mpg_draw_planes(pWin)
#define mpg_vis_planes(pWin) ((mpgPlanes)((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE)?mpg_pg_plane(mpg_pg_list(pWin)):0))
#define mpg_pvis_planes(pWin) ((mpgPlanes)(((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE) && mpg_pg_op(mpg_pg_list(pWin)))?mpg_pg_plane(mpg_pg_list(pWin)):0))
#define mpg_vvis_planes(pWin) ((mpgPlanes)(((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE) && !mpg_pg_op(mpg_pg_list(pWin)))?mpg_pg_plane(mpg_pg_list(pWin)):0))
#define mpg_vis_cliplist(pWin) ((RegionPtr)((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE)?mpg_pg_list(pWin)->clipList:NULL))
#define mpg_vis_borderclip(pWin) ((RegionPtr)((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE)?mpg_pg_list(pWin)->borderClip:NULL))
#endif /* NOTDEF */
 
/* NOTE: virtual view planes are always visible */
#define mpg_pview_planes(pWin) mpg_subtract(mpg_view_planes(pWin), mpg_vvis_planes(pWin))
#define mpg_vview_planes(pWin) mpg_subtract(mpg_view_planes(pWin), mpg_pview_planes(pWin))
#define mpg_phys_planes(pWin) mpg_union(mpg_pdraw_planes(pWin), mpg_pview_planes(pWin))
#define mpg_virt_planes(pWin) mpg_union(mpg_vdraw_planes(pWin), mpg_vview_planes(pWin))
#define mpg_expo_planes(pWin) mpg_union(mpg_draw_planes(pWin), mpg_vis_planes(pWin))

#define mpg_funcs(pWin) (mpg_info(pWin)->mpgFuncs)
#define mpg_screen_setup(pWin) (* mpg_funcs(pWin)->MpgScreenSetup)((pWin)->drawable.pScreen)
#define mpg_create_window(pWin) (* mpg_funcs(pWin)->CreateWindow)(pWin)
#define mpg_destroy_window(pWin) (* mpg_funcs(pWin)->DestroyWindow)(pWin)
#define mpg_realize_window(pWin) (* mpg_funcs(pWin)->RealizeWindow)(pWin)
#define mpg_unrealize_window(pWin) (* mpg_funcs(pWin)->UnrealizeWindow)(pWin)
#define mpg_position_window(pWin, x, y) (* mpg_funcs(pWin)->PositionWindow)(pWin, x, y)
#define mpg_change_window_attributes(pWin, mask) (* mpg_funcs(pWin)->ChangeWindowAttributes)(pWin, mask)
#define mpg_paint_window_border(pWin, pRegion, what) (* mpg_funcs(pWin)->PaintWindowBorder)(pWin, pRegion, what)
#define mpg_paint_window_background(pWin, pRegion, what) (* mpg_funcs(pWin)->PaintWindowBackground)(pWin, pRegion, what)
#define mpg_window_exposures(pWin, prgn, other_exposed) (* mpg_funcs(pWin)->WindowExposures)(pWin, prgn, other_exposed)
#define mpg_get_spans(pDrawable, wMax, ppt, pwidth, nspans, pdstStart) (* mpg_funcs(pWin)->GetSpans) (pDrawable, wMax, ppt, pwidth, nspans, pdstStart)
#define mpg_get_image(pDrawable, sx, sy, w, h, format, planeMask, pdstLine) (* mpg_funcs(pWin)->GetImage)(pDrawable, sx, sy, w, h, format, planeMask, pdstLine)
#define mpg_save_areas(pPixmap, prgnSave, xorg, yorg, pWin) (* mpg_funcs(pWin)->BSfuncs->SaveAreas)(pPixmap, prgnSave, xorg, yorg)
#define mpg_restore_areas(pPixmap, prgnRestore, xorg, yorg, pWin) (* mpg_funcs(pWin)->BSfuncs->RestoreAreas)(pPixmap, prgnRestore, xorg, yorg)

#define mpg_root_vid(pScreen) (mpg_priv_scr(pScreen)->rootVID)
#define mpg_disp_planes(pScreen) (mpg_priv_scr(pScreen)->dispPlanes)
#define mpg_copy_planes(pScreen, pRegions, planes, dx, dy) (* mpg_priv_scr(pScreen)->CopyPlanes)(pScreen, pRegions, planes, dx, dy)
#define mpg_prepare_planes(pWin, pRegions, pRegion, planes, node) (* mpg_priv_scr(pWin->drawable.pScreen)->PreparePlanes)(pWin, pRegions, pRegion, planes, node)
#endif /* MPG_H */
