/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: c3lib.c,v $
 * Revision 1.2  1995/12/05  23:18:41  kevin
 * Fix plxv_ccube_init to check for ccube availability
 *
 * Revision 1.1  1995/11/27  22:46:59  dist
 * Initial revision
 *
 * Revision 1.20  1993/09/30  02:41:31  rocky
 * SQUASH
 *
 * Revision 1.18  93/06/14  16:54:42  xan
 * added IsLastQTable to get and put image
 * 
 * Revision 1.17  93/05/19  23:39:36  rocky
 * *** empty log message ***
 * 
 * Revision 1.16  93/05/19  23:30:26  rocky
 * tcomp/C3GetImage fix (added lseek to 0)
 * also fixed Q table intermittent loading
 * 
 * Revision 1.15  92/07/22  12:33:02  rocky
 * fixed big memory leak in cl550htable
 * 
 * Revision 1.14  92/03/06  11:19:44  dave
 * added eoi stuff here in stead of driver
 * 
 * Revision 1.13  91/12/19  10:49:48  dave
 * fixed hight of decompressed images and did some dec changes
 * 
 * Revision 1.12  91/11/15  11:09:54  dave
 * added chan to C3PutImage parameters, 1 = W else V
 * 
 * Revision 1.11  91/11/15  09:35:12  dave
 * added buffer alignment
 * 
 * Revision 1.10  91/10/30  12:20:50  mears
 * fixed qtable stuff
 * 
 * Revision 1.9  91/10/25  16:33:51  dave
 * fixed GETWORD
 * 
 * Revision 1.8  91/10/23  09:55:44  mears
 * new rev
 * 
 * 
 * Revision 1.6  91/08/14  17:20:31  dave
 * changed things to handle gc'd qtables
 * 
 * Revision 1.5  91/08/14  12:22:14  mears
 * blanking info in nvram
 * 
 * Revision 1.4  91/07/28  04:52:13  eric
 * stuff
 * 
 * Revision 1.3  91/07/26  20:24:44  eric
 * fd as first parameters
 * 
 * Revision 1.2  91/07/26  20:13:22  eric
 * needed fd args
 * 
 * Revision 1.1  91/07/26  20:07:37  eric
 * Initial revision
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: c3lib.c,v 1.2 1995/12/05 23:18:41 kevin Exp $ Parallax Graphics Inc";
#endif

#ifdef DRIVER
static int _c3lib_ferd = 0;
#else  /* the rest if the file */

#include <stdio.h>
#include <fcntl.h>

#include "xvideo.h"
#include "c3.h"
#include "qtab.h"
#include "huff.h"
#include "plxv.h"
#include "plxv_xpoint.h"

#include <math.h>
#include <unistd.h>

/* flags returned by do_marker and plxv_create_c3table	*/

#define	NOMARKER 0		/* no marker or bad marker */
#define	QMARKER 1		/* quatize marker */
#define	HMARKER 2		/* hufman marker */

#define QTABLE_SIZE 128     /* fixed at 128 */

/*
 *  #define XV_DIAG 1 
 */
#include "xvdebug.h"

int loopsperusec = 8;
#define DELAY(n)     { volatile int N = (n * loopsperusec); while (--N > 0); }


#define NUM_LAST_QTABLE_SAMPLES 8
static short LastQTable[NUM_LAST_QTABLE_SAMPLES];  /* samples from last Q table */

static unsigned short *lastqtab=0;                 /* last pointer, old way */

/******************************************************************************/
/*                                                                            */
/*  The visibility matrices are Psychovisual Weighting values from the Databook*/
/*                                                                            */
/******************************************************************************/
    
static unsigned int visibility[128]={
      16, 11, 10, 16, 24, 40, 51, 61, 
      12, 12, 14, 19, 26, 58, 60, 55,
      14, 13, 16, 24, 40, 57, 69, 56,
      14, 17, 22, 29, 51, 87, 80, 62,
      18, 22, 37, 56, 68,109,103, 77,         /* Luminance Componant Matrix   */
      24, 35, 55, 64, 81,104,113, 92,
      49, 64, 78, 87,103,121,120,101,
      72, 92, 95, 98,112,100,103, 99,   
                                       
      17, 18, 24, 47, 99, 99, 99, 99,  
      18, 21, 26, 66, 99, 99, 99, 99,     
      24, 26, 56, 99, 99, 99, 99, 99,        
      47, 66, 99, 99, 99, 99, 99, 99,         /* Chrominance Componant Matrix */
      99, 99, 99, 99, 99, 99, 99, 99,        
      99, 99, 99, 99, 99, 99, 99, 99,        
      99, 99, 99, 99, 99, 99, 99, 99,        
      99, 99, 99, 99, 99, 99, 99, 99};
    
        
/******************************************************************************/
/*                                                                            */
/*  These matrix values are the precalculated DCT coefficients. The first 64  */
/*    values are used by the compression luminance and chrominance            */
/*    calculations, while the last 64 are used for the decompression          */  
/*    calculations.                                                           */
/*                                                                            */
/******************************************************************************/
          
static double KFactor[128]={
        2048.0000000,11362.630859,5351.682129 ,9632.773438 ,
        2048.0000000,6436.413574 ,2216.739258 ,2260.167969 ,
        11362.630859,63041.683594,29691.984375,53444.160156,
        11362.630859,35710.246094,12298.823242,12539.772461,
        5351.682129 ,29691.984375,13984.619141,25171.648438,
        5351.682129 ,16819.158203,5792.619141 ,5906.103516 ,
        9632.773438 ,53444.160156,25171.648438,45307.769531,
        9632.773438 ,30273.685547,10426.438477,10630.706055,
        2048.000000 ,11362.630859,5351.682129 ,9632.773438 ,
        2048.000000 ,6436.413574 ,2216.739258 ,2260.167969 ,
        6436.413574 ,35710.246094,16819.158203,30273.685547,
        6436.413574 ,20228.230469,6966.723633 ,7103.210449 ,
        2216.739258 ,12298.823242,5792.619141 ,10426.438477,
        2216.739258 ,6966.723633 ,2399.381592 ,2446.388428 ,
        2260.167969 ,12539.772461,5906.103516 ,10630.706055,
        2260.167969 ,7103.210449 ,2446.388428 ,2494.316162 ,

        64.000000   ,88.770554   ,83.620033   ,75.256042   ,
        64.000000   ,50.284481   ,34.636551   ,17.657562   ,
        88.770554   ,123.128288  ,115.984314  ,104.383125  ,
        88.770554   ,69.746574   ,48.042278   ,24.491743   ,
        83.620033   ,115.984314  ,109.254837  ,98.326752   ,
        83.620033   ,65.699837   ,45.254837   ,23.070717   ,
        75.256042   ,104.383125  ,98.326752   ,88.491737   ,
        75.256042   ,59.128292   ,40.728275   ,20.763098   ,
        64.000000   ,88.770554   ,83.620033   ,75.256042   ,
        64.000000   ,50.284481   ,34.636551   ,17.657562   ,
        50.284481   ,69.746574   ,65.699837   ,59.128292   ,
        50.284481   ,39.508263   ,27.213764   ,13.873458   ,
        34.636551   ,48.042278   ,45.254837   ,40.728275   ,
        34.636551   ,27.213764   ,18.745169   ,9.556205    ,
        17.657562   ,24.491743   ,23.070717   ,20.763098   ,
        17.657562   ,13.873458   ,9.556205    ,4.871711    };
            

extern C3_WREGS	*c3_wreg;
extern C3_RREGS	*c3_rreg;

extern XV_video_format *plxv_io_format[4];

extern int C3HBlank;
extern int C3HSync;
extern int C3VBlank;
extern int C3VSync;

XV_video_format c3_format;

#define RGB422 1 /* */

#define	C3HBLANK	64		/* 32 */
#define C3HSYNC		16		/* 16 */
#define C3VBLANK	22		/* 8 */
#define C3VSYNC		2		/* 8 */

#define VLSI_LEAF 0x100
#define COMP	1 
#define YDC	0
#define CDC	1
#define	YAC	16
#define CAC	17
#define CL550HTSIZE (0x1c80/4)

static unsigned short *lhufftab = 0;
#define IN 1
#define OUT 2
static int direction = 0;

#define GETSHORT(arg1,arg2) arg1 = ((unsigned char) *arg2++) << 8; arg1 += ((unsigned char) *arg2++)

static unsigned short *lqtab = 0;
static unsigned short *lhtab = 0;


unsigned short CL550_YDC[256];
unsigned short CL550_CDC[256];
unsigned short CL550_YAC[1024];
unsigned short CL550_CAC[1024];

unsigned short *GetCL550Qtable();
unsigned short *GetCL550Htable();

#ifdef NOT
int no_ccube = 0;
#endif

extern int no_ccube;			/* Set in plxv_init_video()  */

static int corrupted = 1;

unsigned char
*plxvMakeQ(factor, buf)
unsigned int factor;
char *buf;
{
    register unsigned char *obuf, *ibuf;
    unsigned char *rbuf;
    register int i, temp;
    int j;
 
    rbuf = obuf = (unsigned char *) malloc(sizeof(DQTmarker));
    ibuf = (unsigned char *) &DQTmarker;
 
/* move the first four chars */
 
    *obuf++ = *ibuf++;
    *obuf++ = *ibuf++;
    *obuf++ = *ibuf++;
    *obuf++ = *ibuf++;
 
    for(j = 2; j--;)
    {
        *obuf++ = *ibuf++;  /* move one then scale the rest */
        for(i = 64; i--;)
        {
        temp = (((unsigned int) *ibuf++) * factor) / 50;
        *obuf++ = (temp < 256) ? temp : 255;
        }
    }
    return rbuf;
}

get_qtables(factor,compress,decompress)
int factor;
unsigned short **compress, **decompress;
{
    unsigned char *dqtptr;
    int dqtsize;
 
    /*
     * Adjust the pointer and size of the DQT marker segment data so
     * the buffer and size is what is expected by GetCL550Qtable().
     */
 
	if ( no_ccube )
	{
		ERRORF((fd, "Attempt to get_qtables when no ccube present\n"));
		return 0;
	}
    dqtptr = (unsigned char *) plxvMakeQ(factor, &DQTmarker);
 
    dqtptr += 4;
 
    dqtsize = sizeof(DQTmarker) -4;
 
    *compress = (unsigned short *) GetCL550Qtable(1, dqtptr, dqtsize);
    *decompress = (unsigned short *) GetCL550Qtable(0, dqtptr, dqtsize);
}

/*****************************************************************************
 *
 * Get compressed image.
 *
 ****************************************************************************/


C3GetImage(fd,x0,y0,w0,h0,x1,y1,w1,h1,scale,buf,len,qtab,htab)
int fd;
int x0,y0,w0,h0;
int x1,y1,w1,h1;
int scale;
unsigned int *buf;
unsigned int len;	/* size of buffer in bytes */
unsigned short *qtab, *htab;
{
	unsigned int dmacount;
	unsigned char *cp;
	int offset;

    DEBUGF(1, (fd, "C3GetImage entry\n"));
	if ( no_ccube )
	{
		ERRORF((fd, "Attempt to GetImage when no ccube present\n"));
		return 0;
	}

	plxv_wf_done();

	if(qtab && (qtab!=lastqtab || !IsLastQTable(qtab)))
	{
	    LoadCL550Qtable(qtab); 
	}
	lastqtab= qtab;

	C3InitCompress();

	if(htab)
	{
	  if(htab != lhtab)
	  {
	    lhtab = htab;
	    LoadCL550Htable(htab);
	  }
	}
	else
	{
	  if(lhtab != defaultCL550CompHtable)
	  {
	    lhtab = defaultCL550CompHtable;
	    LoadCL550Htable(defaultCL550CompHtable); 
	  }
	}

	corrupted = 1;      /* jpgeg put image inline q/h tables now corrupted */

	SetC3GetImageTiming(w1,h1,0); /* Load Window Registers */

	SetC3IoFormat(w1,h1,0);  /* channel W to output 1 frame through C port */

#if 0
	/* Load Window Registers */

	C3_WREG(HPeriod, w1 + C3HBLANK  -1);
	C3_WREG(HSync, C3HSYNC);
	C3_WREG(HDelay, C3HBLANK/2);
	C3_WREG(HActive, 2*(w1/8) -1);
	C3_WREG(VSync, C3VSYNC);
	C3_WREG(VPeriod, h1 + C3VBLANK  -1);
	C3_WREG(VDelay, C3VBLANK/2);	
	C3_WREG(VActive, (h1/8));
	C3_WREG(HControl,  0);
	C3_WREG(VControl,  0);

	/* Set up video channel W to output one frame through the C port */

	c3_format.active_pixels = w1;
	c3_format.active_lines = h1;
	c3_format.blank_lines = C3VBLANK;
	c3_format.reset_line = C3VSYNC;
	c3_format.start_line = C3VBLANK/2;
	plxv_io_format[PORT_C] = &c3_format;
#endif

	plxv_xpoint_c(0x60616);
	plxv_xpoint_w(OUTPUT_C,0);

#if 0
	plxv_output_w(x0,y0,w0,h0,x1,y1+c3_format.start_line,w1,h1,1,1);
#else
	if(xv_cp->main.board_code == SBUS_4)
		offset = 1;
	else
		offset = -2;
	plxv_output_w(x0,y0,w0,h0,x1-1,y1+c3_format.start_line+offset,w1,h1,1,1);
#endif

	/* Must reset with lseek or kernal will eventually bail out when seek 
	   pointer wraps around the system maximum for a stream like read */

	lseek(fd, 0, 0);          

	if ((dmacount = read(fd, buf, len)) == -1)
	{
	    perror("C3GetImage");
	    cp = (unsigned char *) buf;
	}
	else
	{
		register int j, k;
		j = dmacount >> 2;
		k = j + (((len - dmacount) > 128) ? 32 : (len - dmacount) >> 2);
		if(j > 0)
		{
			for(; (j < k) && (buf[j] != -1); ++j);
			dmacount = j << 2;
		}
	    cp = (unsigned char *) &buf[dmacount >> 2];
	}

	*cp++ = 0xff;
	*cp++ = 0xff;
	*cp++ = 0xff;
	*cp++ = 0xd9;

        dmacount+=4 ;
	return(dmacount);
}

/****************************************************************************
 *
 * Put compressed image.
 * 
 ***************************************************************************/

static unsigned char restart[5] = { 0xff, 0xff, 0xff, 0xff, 0xd0 };  /* restart marker */

C3PutImage(fd,x0,y0,w0,h0,x1,y1,w1,h1,scale,buf,len,qtab,htab,chan)
int fd;
int x0,y0,w0,h0;
int x1,y1,w1,h1;
unsigned int *buf;
unsigned int len;	/* size of buffer in bytes */
unsigned short *qtab, *htab;
int chan;
{
	char s[20];
	unsigned int saved[2], numsaved;
	unsigned int *savedloc;
	int count, tlen;
	int flags;
	unsigned int *sbuf, *origbuf;
	static int reload_counter = 0;

	DEBUGF(1, (fd, "C3PutImage entry\n"));
	if ( no_ccube ) 
	{
		ERRORF((fd, "Attempt to PutImage when no ccube present\n"));
		return 0;
	}

	plxv_wf_done();

    origbuf = buf;		/* keep orignal buffer pointer */
	tlen = len;			/* save the len to see if load q tab */

	flags = domarker(&buf,&len);

	if (len < 4)
	{
	    if(flags & QMARKER)
		{
			lqtab = 0;		/* loaded a q table */
			lastqtab = 0;
		}
	    if(flags & HMARKER)
			lhtab = 0;		/* loaded an h table */
		return(tlen);		/* return if nothing but markers */
	}

	if(qtab && (!(flags&QMARKER) && 
	   (qtab!=lastqtab || !IsLastQTable(qtab)) || (reload_counter >= 30))) 
	{
	      LoadCL550Qtable(qtab);
		  lastqtab= qtab;
		  reload_counter = 0;
	}
	else if(flags & QMARKER)
		lastqtab = 0;
    else
		++reload_counter;

	C3InitDecompress();

	if(flags & HMARKER)
	    lhtab = 0;
	else
	{
	  if(htab)
	  {
	    if(htab != lhtab)
	    {
		LoadCL550Htable(htab);
		lhtab = htab;
	    }
	  }
	  else
	  {
	    if(lhtab != defaultCL550DecompHtable)
	    {
	        LoadCL550Htable(defaultCL550DecompHtable);
	    	lhtab = defaultCL550DecompHtable;
	    }
	  }
	}

	/* Check for miss-aligned data: alloc and copy to good buffer */

	sbuf = buf;
	numsaved = 0;
	if((int) sbuf & 3)
	{
	  if(sbuf != origbuf)       /* if there is a header insert restart marker */
	  {
		int nummark;
		savedloc = (unsigned int *)((unsigned int) sbuf & ~3);
		if(((unsigned int) sbuf & 3) == 1)
		{
			numsaved = 2;
			--savedloc;
			nummark = 5;
			saved[0] = savedloc[0];
			saved[1] = savedloc[1];
		}
        else
		{
			numsaved = 1;
			nummark = (unsigned int) sbuf & 3;
			saved[0] = savedloc[0];
		}
		len = (len + nummark) & ~3;
		sbuf = savedloc;
		memcpy(savedloc, &restart[5 - nummark], nummark);
	  }
	  else                  /* bummer, alloc and copy to good buffer */
	  {
	    register unsigned char *tbuf, *ssbuf;
	    register int c;
#ifdef mips
	    sbuf = tbuf = (unsigned char *)(((int) buf) & ~3);
	    ssbuf = (unsigned char *)buf;
#else
	    tbuf = (unsigned char *) malloc((len + 3) & ~3);
	    sbuf = (unsigned int *) tbuf;
	    ssbuf = (unsigned char *)buf;
#endif
	    for(c = 0; c < len; ++c)
		*tbuf++ = *ssbuf++;
	    len = (len + 3) & ~3;
	  }
	}

        /* Set up video channel W or V to input one frame through C port */

	c3_format.active_pixels = w1;
	c3_format.active_lines = h1 + 2;
	c3_format.blank_lines = C3VBLANK;
	c3_format.reset_line = C3VSYNC;
	c3_format.start_line = C3VBLANK/2;

	plxv_io_format[PORT_C] = &c3_format;
#if 0
	plxv_xpoint_c(CE_HIGH);
#else
	plxv_xpoint_c(0x90); /* ??? */
#endif

	if(chan)
	{
            plxv_xpoint_w(INPUT_C,1);
            plxv_flash_w_squeeze(x0,y0+c3_format.start_line,
				 w0,h0 + 2,x1,y1- 1,w1,h1 + 2,1,1);
	}
	else
	{
            plxv_xpoint_v(INPUT_C,1);
            plxv_flash_v_squeeze(x0,y0+c3_format.start_line,
				 w0,h0 + 2,x1,y1- 1,w1,h1 + 2,1,1);
	}


	/* Load Window Registers */

	SetC3PutImageTiming(w0,h0,0);             /* Load Window Registers */

#if 0
	C3_WREG(HPeriod, w0 + C3HBLANK  -1);
	C3_WREG(HSync, C3HSYNC);
	C3_WREG(HDelay, C3HBLANK/2);
	C3_WREG(HActive, 2 * (w0/8) -1);
	C3_WREG(VSync, C3VSYNC);
	C3_WREG(VPeriod, h0 + C3VBLANK  -1 );
	C3_WREG(VDelay, C3VBLANK/2 - 9);
	C3_WREG(VActive, (h0/8));
	C3_WREG(HControl, ~0);
	C3_WREG(VControl, ~0);
#endif

	/* Do the write operation */

	if ((count = write(fd, sbuf, len)) == -1 )
	    perror("C3PutImage");
    if(numsaved)
	{
		savedloc[0] = saved[0];
		if(numsaved > 1)
			savedloc[1] = saved[1];
	}
#ifndef mips
    else
	  if(sbuf != buf)
	    free(sbuf);
#endif
	C3_WREG(S_reset, ~0);

	DEBUGF(1, (fd, "C3PutImage exit\n"));
	return(count);
}
#define InitBufSize 8*1024
/*
 * Compression qtable initialzation routine. Uses C3GetImage
 * to initialize and load boards ccube tables.
 */

plxv_ccube_init(int fd)
{
char *buf;
static int factor=80;
int size;
unsigned short *compress, *decompress;

        if ( no_ccube )
     {
        ERRORF((fd, "Attempt to plxv_ccube_init() when no ccube present \n")) ;
        return 0;
     }
     get_qtables(factor,&compress,&decompress);
	 buf= (char *)malloc(InitBufSize);
     size=C3GetImage(fd,0,0,320,240,0,0,320,240,0,buf,InitBufSize,compress,NULL);
	 free(buf);
}
/****************************************************************************

/****************************************************************************
 * 
 * Check the specified buffer to see if there are any marker codes at the 
 * beginning of it.  If any marker codes exist, process them, and adjust 
 * the start and length of the buffer to strip off the marker codes.
 *
 * Only the following marker codes are processed:
 *
 *	DQT - 	Define quantization table.  The Luma and Chroma tables must be
 *		defined; i.e. there must be two table definitions present in
 *		the marker segment.
 *
 * 	DHT -	Define Huffman table.  All four Huffman tables (YDC, YAC, CDC,
 *		CAC) must be defined; i.e. there must be four table definitions
 * 		present in the marker segment.
 *
 * 	SOF0 -	Start of Frame.  Data is checked only for consistency with our
 *		JPEG implementation.
 *
 * 	SOS -	Start of Scan.  Data is checked only for consistency with our
 *		JPEG implementation.
 *
 * All other marker codes are skipped.
 *
 ****************************************************************************/

static unsigned char pv_qtab[2 * 65];

static unsigned char pv_htab[1024];
static int curhtab;
static int newh;
static int newq;

static void save_dht_marker(buf, len)
char *buf;
int len;
{
	register int i;
	register unsigned char *dp, *sp;
	i = 0;
	sp = (unsigned char *)buf;
	dp = &pv_htab[curhtab];
	if((corrupted == 0) && (lhtab == 0) && (newh == 0))
	{
		for(; i < len; ++i)
		{
			if(*dp++ != *sp++)
			{
				newh = 1;
				--dp;
				--sp;
				break;
			}
		}
	}
	else
		newh = 1;
	if(newh)
		memcpy(dp, sp, len - i);
	curhtab += len;
}

static void save_dqt_marker(buf, len)
char *buf;
int len;
{
	register int i;
	register unsigned char *dp, *sp;

	i = 0;
	sp = (unsigned char *)buf;

	if(len == (2 * 65)) 
		dp = pv_qtab;
    else if(len <= 65)
	{
		switch(buf[0])
		{
			case 0:
				dp = pv_qtab;
				break;
			case 1:
				dp = &pv_qtab[65];
				break;
			default:
				fprintf(stderr,
						"WARNING, YUV422 only: 2 quantization tables max.\n");
				return;
		}
	}
	else
	{
		fprintf(stderr, "    Bad DQT marker length\n");
		return;
	}

	if((corrupted == 0) && (lastqtab == 0) && (newq == 0))
	{
		for(; i < len; ++i)
		{
			if(*dp++ != *sp++)
			{
				newq = 1;
				--dp;
				--sp;
				break;
			}
		}
	}
	else
		newq = 1;
	if(newq)
		memcpy(dp, sp, len - i);
}

static domarker(bufp,lenp)
int **bufp,*lenp;
{
	unsigned char *buf;
	int len;
	int mlen;
	unsigned char mcode;
	int flags;

	flags = 0;
	buf = (unsigned char *) *bufp;
	len = *lenp;

	newq = newh = curhtab = 0;

	/*
	 * Loop while the next two bytes indicate a marker code
	 */

	while ((len > 0) && (*buf == 0xff) && (*(buf+1) != 0) && (*(buf+1) != 0xff)) {
		/*
		 * Get the marker code and the length of the marker segment.
		 * (The length is meaningless for standalone markers: SOI, EOI,
		 * RST, TEM.)
		 */
		mcode = *(buf+1);
		buf +=2;
		GETSHORT(mlen, buf);
		mlen -= 2;

		/*
		 * Process the marker code
		 */

		switch (mcode) {
		case 0xDB:			/* Define Quant. Table */
			save_dqt_marker(buf,mlen);
			flags |= QMARKER;
			break;
		case 0xDA:
			sos_marker(buf,mlen);
			break;
		case 0xC0:
			sof0_marker(buf,mlen);
			break;
		case 0xC4:
			save_dht_marker(buf,mlen);
			flags |= HMARKER;
			break;
		default:
			break;
		}


		/* 
		 * Adjust the pointer to and length of the data buffer,
		 * stripping off the processed marker segment.
		 */

		if ((mcode == 0xd8) ||/* (mcode == 0xd9) ||*/ (mcode == 0x01) ||
			((mcode & 0xf8) == 0xd0)) {
			buf -= 2;
			len -= 2;
		} else {
			buf += mlen;
			len -= mlen+4;
		}
		*bufp = (int *) buf;
		*lenp = len;
	}
	if((flags & QMARKER) && newq)
    {
		dqt_marker(pv_qtab, 2 * 65);
		corrupted = 0;
	}
	if((flags & HMARKER) && newh)
    {
		dht_marker(pv_htab, curhtab);
		corrupted = 0;
	}
	return(flags);
}

plxv_create_c3table(dir, buf, len, retbuf)
int dir;
register unsigned char *buf;
register int len;
unsigned short *retbuf[];
{
	int flags=0, mlen;
	unsigned char mcode, *end;

	if(len <= 0)
		return(flags);
	/*
	 * Get the marker code and the length of the marker segment.
	 * (The length is meaningless for standalone markers: SOI, EOI,
	 * RST, TEM.)
	 */

	curhtab = 0;
	for(end = buf + len; buf < end;)
	{
		mcode = *(buf+1);
		buf +=2;
		GETSHORT(mlen, buf);
		mlen -= 2;
	
		/*
		 * Process the marker code
		 */
	
		switch (mcode)
		{
			case 0xDB:			/* Define Quant. Table */
				save_dqt_marker(buf, mlen);
				flags |= QMARKER;
				break;
			case 0xC4:
				save_dht_marker(buf, mlen);
				flags |= HMARKER;
				break;
			default:
				break;
		}
		if ((mcode == 0xd8) || (mcode == 0x01) ||
			((mcode & 0xf8) == 0xd0))
			buf -= 2;
		else
			buf += mlen;
	}

	if((flags & QMARKER) && (flags & HMARKER))
	{
	    fprintf(stderr,
			"plxv_create_c3table: both Q and H markers in buffer\n");
		return(0);
	}

	if(flags & QMARKER)
		*retbuf = GetCL550Qtable(dir, pv_qtab, 2 * 65);
	else if(flags & HMARKER)
		*retbuf = GetCL550Htable(dir, pv_htab, curhtab);

    corrupted = 1;		/* jpgeg put image inline q/h tables now corrupted */
	return(flags);
}

plxv_create_c3tables(dir, buf, len, qretbuf, hretbuf)
int dir;
register unsigned char *buf;
register int len;
unsigned short *qretbuf[], *hretbuf[];
{
	int flags=0, mlen;
	unsigned char mcode, *end;

	if(len <= 0)
		return(0);

	/*
	 * Get the marker code and the length of the marker segment.
	 * (The length is meaningless for standalone markers: SOI, EOI,
	 * RST, TEM.)
	 */

	curhtab = 0;
	for(end = buf + len; buf < end;)
	{
		if(*buf++ != 0xff)
			break;
		mcode = *buf++;
		GETSHORT(mlen, buf);
		mlen -= 2;
	
		/*
		 * Process the marker code
		 */
	
		switch (mcode)
		{
			case 0xDB:			/* Define Quant. Table */
				save_dqt_marker(buf, mlen);
				flags |= QMARKER;
				break;
			case 0xC4:
				save_dht_marker(buf, mlen);
				flags |= HMARKER;
				break;
			default:
				break;
		}
		if ((mcode == 0xd8) || (mcode == 0x01) ||
			((mcode & 0xf8) == 0xd0))
			buf -= 2;
		else
			buf += mlen;
	}
	if(flags & QMARKER)
		*qretbuf = GetCL550Qtable(dir, pv_qtab, 2 * 65);
	if(flags & HMARKER)
		*hretbuf = GetCL550Htable(dir, pv_htab, curhtab);
	return(flags);
}


/*****************************************************************************
 *
 * Process a Start of Scan Marker Code.  Check to see that component 1 is Y
 * and components 2 and 3 are C (U and V) by examining the table identifier
 * fields. 
 *
 *****************************************************************************/

static sos_marker(buf,len)
char *buf;
int len;
{
	int 	i,err=0;
	char 	n, 		/* Number of components 	*/
		cs, 		/* Component identifier		*/
		td, 		/* DC Huffman table identifier	*/
		ta;		/* AC Huffman table identifier	*/

	n = *buf++;

	for (i=0; i<n; i++) {
		cs = *buf++;
		td = ta = *buf++;
		td = (td & 0xf0) >> 4;
		ta &= 0xf;

		switch (cs) {
		case 1:	
			if ((td != 0) || (ta != 0)) err=1;
			break;
		case 2:
			if ((td != 1) || (ta != 1)) err=1;
			break;
		case 3:
			if ((td != 1) || (ta != 1)) err=1;
			break;
		default:
			err = 1;
			break;
		}
		if (err) printf("SOS: bad component spec\n");
		err = 0;
	}
}

/******************************************************************************
 *
 * Process a Start of Frame Marker Code.  Check to see that the image data
 * contains three components of eight bits each and that the q-table specs
 * are consistent with our implementation.
 * 
 * Also check that the image size is no bigger than 1024x1024, since a wider
 * image will overflow the 8K static RAM front-ending the CL550.
 *
 ******************************************************************************/

static sof0_marker(buf,len)
char *buf;
int len;
{
	int i,err=0;
	char 	p,	/* Number of bits per component		*/
		n;	/* Number of components			*/
	short 	x,	/* X size of image, in pixels		*/
		y;	/* Y size of image, in pixels		*/
	char 	c, 	/* component identifier			*/
		h, 	/* number of h samples per MDU		*/
		v, 	/* number of v samples per MDU		*/
		tq;	/* quantization table for this component */

	p = *buf++;
	GETSHORT(y,buf);
	GETSHORT(x,buf);
	n = *buf++;

	if ((x>1024) || (y>1024)) printf("SOF: 1024x1024 is max image size\n");
	if ((n != 3) || (p != 8)) err = 1;

	for (i=0; i<n; i++) {
		c = *buf++;
		h = v = *buf++;
		h = (h & 0xf0) >> 4;
		v &= 0xf; 
		tq = *buf++;

		switch (c) {
		case 1:
			if ((h != 2) || (v != 1)) err=1;
			break;
		case 2:
			if ((h != 1) || (v != 1)) err=1;
			break;
		case 3:
			if ((h != 1) || (v != 1)) err=1;
			break;
		default:
			err = 1;
			break;
		}
		if (err) printf("SOF: Image must be coded in YUV 422 mode\n");
		err = 0;
	}
}

/******************************************************************************
 *
 * Process a Define Huffman Table Marker Code.
 *
 ******************************************************************************/

static dht_marker(buf,len)
char *buf;
int len;
{
	unsigned short *CL550Htable;


	CL550Htable = GetCL550Htable(0, buf, len);
	LoadCL550Htable(CL550Htable);
/*
	lhufftab = CL550Htable;
*/
	free(CL550Htable);
}

/*****************************************************************************
 * 
 * Process a Define Quantization Table Marker Code.  
 *
 *****************************************************************************/

static dqt_marker(buf,len)
char *buf;
int len;
{
	unsigned short *CL550Qtable;

	if (len != 2*65) { 
		printf("    Bad DQT marker length\n");
		return;
	}

	CL550Qtable = GetCL550Qtable(0, buf, len);
	LoadCL550Qtable(CL550Qtable);

/*
	lqtab = CL550Qtable;
*/

	free(CL550Qtable);
}

/*****************************************************************************
 *
 * Load CL550 Huffman Table.  Expects an array of CL550HTSIZE shorts to be
 * loaded into the CL550 huffman tables.  It loads all four huffman tables
 * in a single shot.
 *
 *****************************************************************************/

LoadCL550Htable(table)
unsigned short *table;
{
	register int i;

	DEBUGF(1, (fd, "LoadCL550Htable entry\n"));
	if ( no_ccube ) 
	{
		ERRORF((fd, "Attempt to load Huffman table when no ccube present\n"));
		return 0;
	}

	C3_WREG(Huff_Enable, -1);

	for (i=0;i<CL550HTSIZE;i++)
	{
		DELAY(1);
		C3_WREG(HuffYAC[i],table[i]);
	}
	DELAY(1);
	C3_WREG(Huff_Enable, 0);
	DEBUGF(1, (fd, "LoadCL550Htable exit\n"));
}

/*****************************************************************************
 *
 * Load CL550 Quantization Table A.  (B is unused).  Expects an array of 
 * 128(QTABLE_SIZE) shorts to be loaded into the CL550 QuantA registers.
 *
 *****************************************************************************/

LoadCL550Qtable(table)
unsigned short *table;
{
	register int i,j,inc=(QTABLE_SIZE/NUM_LAST_QTABLE_SAMPLES);

	DEBUGF(1, (fd, "LoadCL550Qtable entry\n"));
	if ( no_ccube ) 
	{
		ERRORF((fd, "Attempt to load Q table when no ccube present\n"));
		return 0;
	}

	C3_WREG(QuantSelect, 0);

	for (i=j=0;i<QTABLE_SIZE;i++) {
		DELAY(1);
	    if(!(i%inc)) LastQTable[j++]= table[i];    /* for IsLastQTable() */

	    C3_WREG(QuantA[i], table[i]);
	}

	lastqtab= table;
	DELAY(1);

	C3_WREG(QuantSelect, 1);
	DEBUGF(1, (fd, "LoadCL550Qtable exit\n"));
}

/****************************************************************************** 
 *
 * Generate a CL550 Huffman table load image from a DHT marker code segment.
 * The load image is generated differently for compression and decompression.
 * The marker segement passed to this routine should be stripped of the 
 * marker code itself and length field.  The rest of the buffer and the length
 * are passed as arguments.
 *
 ******************************************************************************/

unsigned short *GetCL550Htable(compFlag, JPEGbits, JPEGlen)
short compFlag;
unsigned char *JPEGbits;
int	JPEGlen;
{
	unsigned short *YDC_table,*CDC_table,*YAC_table,*CAC_table;
	unsigned i,j,k, num_values;
	char bits_char[16];
	unsigned bits[32];

	int     which;
	int	start;
    
	unsigned short		*CL550Htable;
	unsigned char           BitsArray[16];
	unsigned char           ValuesArray[256];
	short                   num;
	short           *CoderCodes;
	short           *DecoderCodes;

	DEBUGF(1, (fd, "GetCL550Htable entry\n"));
	if ( no_ccube ) 
	{
		ERRORF((fd, "Attempt to get Huffman table when no ccube present\n"));
		return 0;
	}

    CL550Htable = (unsigned short *) calloc(CL550HTSIZE, sizeof(short));
    YAC_table = CL550Htable;
    YDC_table = (unsigned short *) ((int) CL550Htable + 0xc00/2);
    CAC_table = (unsigned short *) ((int) CL550Htable + 0x1000/2);
    CDC_table = (unsigned short *) ((int) CL550Htable + 0x1c00/2);
    
    start = (int) JPEGbits;  
    while ((int) JPEGbits < start + JPEGlen) {

	which = *JPEGbits++;

	for (i=0;i<16;i++) BitsArray[i] = *JPEGbits++;

        for( i=0, num_values=0; i<16; i++)
            num_values  += BitsArray[i];

	for (i=0; i< num_values; i++) ValuesArray[i] = *JPEGbits++;

        if (compFlag == COMP)
        {
            /***   Generate CL550B Coder Tables ***/

            buildCodeTablesCL550B(BitsArray, ValuesArray, &num, &CoderCodes);

	    if(!CoderCodes)
	    {
		printf("Error: GetCL550Htable: mem not alloc'd for Coder table!\n");
		printf("       CL550Htable not valid!\n");
		return(CL550Htable);
	    }

	    switch (which) {
		case YDC:	
		    for (i=0;i<num;i++) YDC_table[i] = CoderCodes[i];
		    break;
		case CDC:	
		    for (i=0;i<num;i++) CDC_table[i] = CoderCodes[i];
		    break;
		case YAC:	
		    for (i=0;i<num;i++) YAC_table[i] = CoderCodes[i];
		    break;
		case CAC:	
		    for (i=0;i<num;i++) CAC_table[i] = CoderCodes[i];
		    break;
		default:
		    break;
	    } 
	    free(CoderCodes);
        }
        else
        {
            /***   Generate CL550B Decoder Tables ***/

            buildTreesCL550B(BitsArray, ValuesArray, &num, &DecoderCodes);

	    if(!DecoderCodes)
	    {
		printf("Error: GetCL550Htable: mem not alloc'd for Decoder table!\n");
		printf("       CL550Htable not valid!\n");
		return(CL550Htable);
	    }

	    switch (which) {
		case YDC:
		    for (i=0;i<num;i++) YDC_table[i] = DecoderCodes[i];
		    break;
		case CDC:
		    for (i=0;i<num;i++) CDC_table[i] = DecoderCodes[i];
		    break;
		case YAC:
		    for (i=0;i<num;i++) YAC_table[i] = DecoderCodes[i];
		    break;
		case CAC:
		    for (i=0;i<num;i++) CAC_table[i] = DecoderCodes[i];
		    break;
		default:
		    break;
	    }
	    free(DecoderCodes);
        }
     }  /* end of table loop */

	DEBUGF(1, (fd, "GetCL550Htable exit\n"));
	return(CL550Htable);
}


/******************************************************************************/
/*                                                                            */
/*  buildCodeTablesCL550B(BitsArray, ValuesArray, OutputSize, OutputCodes)    */
/*      builds the huffman code tables for the CL550 processor, using bits and*/
/*      values as its intput.                                                 */
/*                                                                            */
/******************************************************************************/

static buildCodeTablesCL550B(BitsArray, ValuesArray, OutputSize, OutputCodes)
unsigned char       BitsArray[16];  /* Number of codes for each code length i */
unsigned char       ValuesArray[];  /* Values represented by each code        */
short               *OutputSize;  /* Number of 9 bit table entries        */
short	            **OutputCodes;    /* Code tables suitable for CL550B      */
                                      /*    download                          */
{
    short       i,j,k;
    short       codeSizesBySize[257], codeSizesByValue[257];
    short       codeValuesBySize[257], codeValuesByValue[257];
    short       num_codes,  *CodeTable;
    

	DEBUGF(1, (fd, "buildCodeTablesCL550B entry\n"));
	if ( no_ccube )
	{
		ERRORF((fd, "Attempt to build code tables when no ccube present\n"));
		return 0;
	}

    /*** allocate memory, if any available, for ouput ***/
    
    CodeTable       = (short*)malloc(sizeof(short)*2048L);
    *OutputCodes    = CodeTable;
    if( !CodeTable )
        return;
    

    /***   Generate Huffman Codes from minimal info form  ***/

    genCodeSizes(   BitsArray, codeSizesBySize, &num_codes);
    generateCodes(  codeSizesBySize, codeValuesBySize );
    orderCodes(     ValuesArray,
                        codeValuesBySize,
                        codeSizesBySize,
                        codeValuesByValue,
                        codeSizesByValue,
                        &num_codes);

    /*** Generate VLSI coder data ***/

            CL550_coder(    num_codes,
                            codeValuesByValue,
                            codeSizesByValue,
                            CodeTable);
    if (num_codes < 20)
        *OutputSize = num_codes*2;
    else
        *OutputSize = 512;
                        
}

/******************************************************************************/
/*                                                                            */
/*  CL550_coder(len, codes, sizes, CodeTablePtr) actually builds the code     */
/*     tables                                                                 */
/*                                                                            */
/******************************************************************************/
    

static CL550_coder(len, codes, sizes, CodeTablePtr)
short   len;
short   codes[], sizes[];
short       *CodeTablePtr;
{
    register    short   i, j;
    unsigned short  shortlen, tempcode, test;
    unsigned long   size,revCode,bigCodeWord;
    
    
	DEBUGF(1, (fd, "CL550_coder entry\n"));
    for (i=0; i<len; i++)
        {
        tempcode = codes[i];
        shortlen = sizes[i];
        revCode = 0;
        if (shortlen)
            {
            size  = shortlen;
            for (j=0;(unsigned short)j<shortlen;j++)
                {
                revCode = revCode <<1;
                if (tempcode &1)
                    revCode |= 1;
                tempcode = tempcode >>1;
                }
            if( shortlen == 16 )
                bigCodeWord = 0x20000l | revCode;
            else if( shortlen == 15 )
                bigCodeWord = 0x30000l | revCode;
            else if( shortlen == 14 )
                bigCodeWord = 0x38000l | revCode;
            else
                bigCodeWord = size<<13 | revCode;
            }
        else
            bigCodeWord = 0;
        *CodeTablePtr++ = bigCodeWord&0x1FF;
        *CodeTablePtr++ = (bigCodeWord>>9)&0x1FF;
        }
        if (len > 20)
            for (i=0;i<512-len*2;i++)
                *CodeTablePtr++ = 0x0000L;
	DEBUGF(1, (fd, "CL550_coder exit\n"));
}


/******************************************************************************/
/*                                                                            */
/*  buildTreesCL550B(BitsArray, ValuesArray, OutputSize, OutputTree) builds   */
/*      the output tree from the bits and values                              */
/*                                                                            */
/******************************************************************************/


static buildTreesCL550B(BitsArray, ValuesArray, OutputSize, OutputTree)
unsigned char    BitsArray[16];     /* Number of codes for each code length i */
unsigned char    ValuesArray[];     /* Values represented by each code        */
short            *OutputSize;       /* Number of 9 bit table entries          */
short    **OutputTree;   /* Decode tree  suitable for CL550B download */
{
    short       codeSizesBySize[257], codeSizesByValue[257];
    short       codeValuesBySize[257], codeValuesByValue[257];
    short       num_codes;
    
    short       *TreeTable;
    
/*** allocate memory, if any available, for ouput ***/
    
	DEBUGF(1, (fd, "buildTreesCL550B entry\n"));
    TreeTable       = (short*)malloc(sizeof(short)*2048L);
    *OutputTree     = TreeTable;
    if( !TreeTable )
        return;
    

/***   Generate Huffman Codes from minimal info form  ***/
    genCodeSizes(       BitsArray, codeSizesBySize, &num_codes);
    generateCodes(      codeSizesBySize, codeValuesBySize );

/*** Make decoder tree ***/

    make_Cl550_tree(    num_codes,
                            ValuesArray,
                            codeValuesBySize,
                            codeSizesBySize,
                            TreeTable,
                            OutputSize);
}
    

/******************************************************************************/
/*                                                                            */
/*  Generate a two bit grab decoding tree for chip 2 (CL550B)                 */
/*                                                                            */
/*  Each addressable word in the decode tree contains 18 bits( a high 9 & a   */
/*     low 9).                                                                */
/*  Each 9 bits is a node.                                                    */ 
/*  Each node contains:                                                       */
/*    1 - A leaf flag                                                         */
/*    2 - Either;                                                             */
/*            A. Next state, if this is a branch ( leaf flag not set), or     */
/*            B. 8 bit value represented by the huffman code.                 */
/*  Two 18 bit words can be addressed by a state.                             */
/*  MS of the 2 bit grab determine which of two words.                        */
/*  LS of the 2 bit grab determine whether high 9 or low 9.                   */
/*                                                                            */
/******************************************************************************/

static make_Cl550_tree(len, values, codes, sizes, tree, treeSize)
short     len, codes[], sizes[];
short   tree[];
short               *treeSize;
unsigned char   values[];
{
    short   *base;
    register    short       grab2;
    short       code, size, nextState;
    short       i;
    
	DEBUGF(1, (fd, "make_Cl550_tree entry\n"));
    for (i=0;i<2048;i++)
        tree[i] = 0;
    nextState = 1;
    for (i=0;i<len;i++)
        {
        size=sizes[i];
            
        base = tree;                      /* start @ root of tree             */
        code = codes[i] << (16-size);     /* get code we wanna insert         */
        while( size>0 )                   /* no rest till this code has a leaf*/
            {                             /* grab next two bits               */
            grab2 = (code < 0)?2:0;       /* first one,                       */
            if ( (code =code <<1) < 0 )   /* then the other                   */
                grab2 += 1;
            code = code << 1;                      

            if (size>2)                   /* then not a leaf, but u node that */
                { /* size > 2 */
                base += grab2;            /* point to node                    */
                if (*base)                /* branch already defined ??        */
                    base = tree + 4*(*base);/* yes...branch, so to speak, out */
                else
                    {                      /* no ... define a new branch      */
                    *base = nextState;
                    base = tree + 4*nextState;
                    nextState++;
                    }
                size -= 2;
                } /* size > 2 */
            else
                { /* size is 1 or 2 */      /* then we got us a leaf          */
                if (size == 2)              /* either even or odd             */
                    {
                    base += grab2;
                    *base = values[i] | VLSI_LEAF;/* done + value represented */
                    }
                else
                    {               /* size is 1, we've got one too many bits */
                    grab2 &= 2;
                    base += grab2;
                    *base = values[i] | VLSI_LEAF; /* done + value represented*/
                    base ++;
                    *base = values[i] | VLSI_LEAF; /* done + value represented*/
                    }
                break;
                }
            }
        }
    *treeSize = nextState * 4;
	DEBUGF(1, (fd, "make_Cl550_tree exit\n"));
}

/******************************************************************************/
/*                                                                            */
/*   Constructs an array containing the sizes of each Huffman code.           */
/*   Sorted in increasing code size.                                          */
/*                                                                            */
/******************************************************************************/

static genCodeSizes( bits, codeSize, numberCodes )
unsigned    char        bits[];
short       codeSize[], *numberCodes;
  {
    short   p,i,j;
    short   numCodesIthSize;

     for( i=0, p=0; i<16;)
       {
          numCodesIthSize = bits[i++];
          for( j=0; j<numCodesIthSize; j++)
               codeSize[p++] = i;
        }

     codeSize[p] = 0;
     *numberCodes = p;
  }   /* end of function size_table */


/******************************************************************************/
/*                                                                            */
/*   Generates the huffman codes for each codesize.                           */
/*                                                                            */
/******************************************************************************/
static generateCodes( sizeArray, codeArray )
short    sizeArray[], codeArray[];
{
    short    p, code, currentSize, nextSize;

     p = 0;
     code = 0;
     currentSize = sizeArray[0];

     while ( 1 )
       {
          do                                    /** inc while sizes are same **/
              codeArray[p++] = code++;
          while ( sizeArray[p]==currentSize );

          if ( !(nextSize = sizeArray[p]) )     /** terminate with zero size **/
             break; 
        do                                   /** shift code up for next size **/
           {
             code = code << 1;
              currentSize++;
            }
         while ( currentSize != nextSize );
        }
     
} 


/******************************************************************************/
/*                                                                            */
/*    Sorts by value represented                                              */
/*                                                                            */
/******************************************************************************/
static orderCodes( huffval, huffcode, huffsize, codeByval, sizeByval, numCodes )
  unsigned char   huffval[];
  short             huffcode[], huffsize[], sizeByval[], codeByval[];
  short         *numCodes;
  {
    short    i,index;
    unsigned maxval;
    
    /*** find max value ***/
    maxval = 0;
    for( i=0; i<*numCodes; i++)
        maxval = huffval[i]>maxval?huffval[i]:maxval;
        
    /*** init sizeByval to zero for all values ***/
    for( i=0; i<maxval; i++)
        sizeByval[i] = 0;
        
     /*** sort ***/
     for( i=0; i<*numCodes; i++)
       {
          index = huffval[i];
          codeByval[index] = huffcode[i];
          sizeByval[index] = huffsize[i];
        }
        
    *numCodes = maxval+1;

  }

/******************************************************************************/
/*                                                                            */
/*    Generate a CL550 Quantization table for either compression or 	      */
/*    decompression.  This routine takes a DQT marker segment, less the DQT   */
/*    marker code and length.						      */
/*                                                                            */
/******************************************************************************/

unsigned short *GetCL550Qtable(compflag, buf, len)
int compflag;
char *buf;
int len;
{
#if 0
    char 	pq,	/* quantizer precision 0=8, 1=16 	*/
		tq;	/* quantizer table number		*/
    unsigned char c;
    int i,j,index;
    int qshift;
    int start;
    unsigned int compCoef1[QTABLE_SIZE];
    unsigned int decompCoef1[QTABLE_SIZE];
    static short visibility[QTABLE_SIZE];
#endif
    unsigned short *CL550Qtable;

	DEBUGF(1, (fd, "GetCL550Qtable entry\n"));
    CL550Qtable = (unsigned short *) malloc(QTABLE_SIZE * sizeof(short));   

#if 1
	++buf;
	C3PrepHardwareQTable(buf, CL550Qtable, compflag);
	buf += 65;
	C3PrepHardwareQTable(buf, &CL550Qtable[64], compflag);
#else

	start = (int) buf;
	while ((int) buf < start + len) {
		pq = tq = *buf++;
		pq = (pq >> 4) & 0xf;
		tq &= 0xf;
	
		if (pq != 0) printf("8 Bit Q table precision supported only\n");
		if (tq > 1) printf("YUV422 only: 2 quantization tables max.\n");

		for (i=0;i<64;i++) {
			c = *buf++;
			index = tq * 64 + i;
			visibility[index] = (int) c;
		}
	}

    if (compflag) {

        /* calculate the Q table values for compression */
    
        for ( i=0; i<64; i++ ) {

            /* compression luminance */

            compCoef1[i] = KFactor[i] / visibility[i] + .5;  
            if ( compCoef1[i] > 32767. ) compCoef1[i] = 32767.;

            /* compression chrominance */
            
            compCoef1[i+64] = KFactor[i] / visibility[i+64] + .5; 
            if ( compCoef1[i+64] > 32767. ) compCoef1[i+64] = 32767.;
        }
        
        for ( i=0; i<8; i++ )
            for ( j=0; j<8; j++ ) {
                 CL550Qtable[i*8+j]=compCoef1[i+j*8];
                 CL550Qtable[i*8+j+64]=compCoef1[i+j*8+64];
            }

        } else {

        /* calculate the Q table values for decompression */
    
        for ( i=0; i<64; i++ ) {
            decompCoef1[i] = KFactor[i+64] * visibility[i]
                * pow( (double)2,(double)count_bits(visibility[i]) ) *.5 + .5;   

            decompCoef1[i] &= 0xfffc; 
            
            qshift = 7 - count_bits( visibility[i] );

            decompCoef1[i] = (qshift<<13) + (decompCoef1[i]>>2); 
                                         
            /* decompression chrominance */
            
            decompCoef1[i+64] = KFactor[i+64] * visibility[i+64]
                * pow( (double)2,(double) count_bits(visibility[i+64]) ) * .5 + .5; 
    
            if (decompCoef1[i+64] > 32767.) decompCoef1[i+64] = 32767.;

            decompCoef1[i+64] &= 0xfffc; 
            qshift = 7 - count_bits( visibility[i+64] );

            decompCoef1[i+64] = (qshift<<13) + (decompCoef1[i+64]>>2);
        }
        
        for ( i=0; i<8; i++ )
            for ( j=0; j<8; j++ ) {
                 CL550Qtable[i*8+j]=decompCoef1[i+j*8];
                 CL550Qtable[i*8+j+64]=decompCoef1[i+j*8+64];
            }
     }

#endif
	DEBUGF(1, (fd, "GetCL550Qtable exit\n"));
     return(CL550Qtable);
}

/******************************************************************************/
/*                                                                            */
/*  count_bits( quant )  finds the number of leading zeros for the given      */
/*     integer.                                                               */
/*                                                                            */
/******************************************************************************/

static count_bits( quant )
unsigned short quant;
{
    int i;
    unsigned char mask;

    for (i=0, mask=0x80; i<8; i++,mask>>=1)
        if (mask & quant) return(i);
    
}

/* ------ this is the new stuff.  Robert Palmer. */

/*****************************************************************************
 * Initialize Get Compressed Image for the RTV driver.
 ****************************************************************************/

RtvInitC3GetImage(x0,y0,w0,h0,x1,y1,w1,h1)
int x0,y0,w0,h0;
int x1,y1,w1,h1;
{
	int offset;
	int dw;

	if ( no_ccube )
	{
	    ERRORF((fd,"Attempt to RtvInitC3GetImage when no ccube present\n"));
	    return 0;
	}

	SetC3IoFormat(w1,h1, 1); 

	plxv_wf_done();
	plxv_xpoint_c(0x60616);
	plxv_xpoint_w(OUTPUT_C, 1); 

	if(xv_cp->main.board_code == SBUS_4)
		offset = 1;
	else
		offset = -2;

#ifdef RTV_DEBUG
	fprintf(stderr,"RtvInitC3GetImage: x1=%d y1=%d w1=%d h1=%d\n",
		x1,y1+c3_format.start_line+offset,w1,h1);
#endif

	plxv_output_w(x0,y0,w0,h0,x1,y1+c3_format.start_line+offset,w1,h1,0,0);
}

/****************************************************************************
 * Initialize Put Compressed Image for driver RTV.
 ***************************************************************************/

RtvInitC3PutImage(x0,y0,w0,h0,x1,y1,w1,h1)
int x0,y0,w0,h0;
int x1,y1,w1,h1;
{
	SetC3IoFormat(w1,h1,1); 

	plxv_wf_done();
        plxv_xpoint_c(0x90); 
        plxv_xpoint_v(INPUT_C,1);
/*
	XV_WREG(VSqzY.b.factor, 4095);
	XV_WREG(VSqzY.b.inter, 0);
	XV_WREG(VSqzY.b.ltor, 0);
	XV_WREG(VSqzY.b.ttob, 1);
	XV_WREG(VStart.l,4);
        XV_WREG(VStop.l, 260)
	XV_WREG4(VCtl.l,4);
 */
}

SetC3IoFormat(width,height,interlaced)
{
	DEBUGF(1, (fd, "SetC3IoFormat entry\n"));
	c3_format.active_pixels = width;
	c3_format.active_lines = height/(interlaced ? 2 : 1);
	c3_format.blank_lines = C3VBlank;
	c3_format.reset_line = C3VSync;           /* only one used hlib */
	c3_format.start_line = C3VBlank/2; 
	plxv_io_format[PORT_C] = &c3_format;
	DEBUGF(1, (fd, "SetC3IoFormat exit\n"));
}

/* Low overhead Q & H table compares. Checks every 16th table entry for a
 * reasonable match to the previous Qtable.
 */

IsLastQTable(table) 
short *table;
{
    static short last[10];
    int i, difference=0;

    for(i=0; i<8; i++,table+=16) {
        if(last[i] != *table) difference=1;
	last[i]= *table;
    } 

    return((!difference));
}

IsLastHTable(table) 
short *table;
{
    static short last[10];
    int i, difference=0;

    for(i=0; i<8; i++,table+=16) {
        if(last[i] != *table) difference=1;
	last[i]= *table;
    } 

    return((!difference));
}

do_coef()
{
	
	int i;

/*
	printf("do_coef:\n");
*/
	/* Load DCT coefficients */

	DEBUGF(1, (fd, "do_coeff entry\n"));
    XV_RREG(i,Bt.blu.reg0);
	/* plxv_switch_context(xv); */

	C3_WREG(DCT[0],0x00005A82);
	DELAY(10);
	C3_WREG(DCT[1],0x00007FFF);
	DELAY(10);
	C3_WREG(DCT[2],0x000030FC);
	DELAY(10);
	C3_WREG(DCT[3],0x00007642);
	DELAY(10);
	C3_WREG(DCT[4],0x00005A82);
	DELAY(10);
	C3_WREG(DCT[5],0x00007FFF);
	DELAY(10);
	C3_WREG(DCT[6],0x000030FC);
	DELAY(10);
	C3_WREG(DCT[7],0x00007642);
	DELAY(10);
	C3_WREG(DCT[9],0x00000000);
	/*
	printf("exit do_coef\n");
	 */
	DEBUGF(1, (fd, "do_coeff exit\n"));
}

/*****************************************************************************
 *
 * Prepare the CL550 for compression.  Called by C3GetImage.
 *
 *****************************************************************************/

static C3InitCompress()
{
	char s[20];
	int i1,i2;


	C3_WREG(S_reset, ~0);

	/* Load configuration registers */

#ifdef RGB422
	C3_WREG(Config, 0x38);
#else
	C3_WREG(Config, 0x28); 
#endif
	C3_WREG(DRQ_Enable,0);
	C3_WREG(NMRQ_Enable,0); 

#ifdef RGB422
	C3_WREG(Init_1, 0x003c);
	C3_WREG(Init_2, 0x003d);
	C3_WREG(Init_3, 0x007b);
	C3_WREG(Init_4, 0x00f1);
	C3_WREG(Init_5, 0x000a);
	C3_WREG(Init_6, 0x0000);
	C3_WREG(Init_7, 0x0017);
	C3_WREG(QuantSync, 0x0400);
#else
	C3_WREG(Init_1, 0x0042);
	C3_WREG(Init_2, 0x0001);
	C3_WREG(Init_3, 0x0081);
	C3_WREG(Init_4, 0x00f7);
	C3_WREG(Init_5, 0x0000);
	C3_WREG(Init_6, 0x0000);
	C3_WREG(Init_7, 0x001d);
	C3_WREG(QuantSync, 0x0406);
#endif
	C3_WREG(QuantYC, 0x2099);
	C3_WREG(QuantAB, 0x0000);
	C3_WREG(QuantSelect, 1);
#ifdef RGB422
	C3_WREG(VideoLatency, 0x0185);
#else
	C3_WREG(VideoLatency, 0x017f);
#endif

	/* Load coder configuration registers */

	C3_WREG(CodecSeq, 0x00cc);
	C3_WREG(CodecCompH, 0x0088);
	C3_WREG(CodecCompL, 0x0044);
	C3_WREG(CoderAttr, 0x00004);
	C3_WREG(CodingIntH, 0x0000);
#ifdef RGB422
	C3_WREG(CodingIntL, 0x0002);
#else
	C3_WREG(CodingIntL, 0x001);
#endif
	C3_WREG(DecLength, 0x0008);
	C3_WREG(CodeOrder, 0x0001);

	/* Load RGB-YUV conversion matrix */

	C3_WREG(R2Y,0x00000133);
	C3_WREG(G2Y,0x00000259);
	C3_WREG(B2Y,0x00000074);
	C3_WREG(R2U,0x00000F54);
	C3_WREG(G2U,0x00000EAD);
	C3_WREG(B2U,0x000001FF);
	C3_WREG(R2V,0x000001FF);
	C3_WREG(G2V,0x00000E53);
	C3_WREG(B2V,0x00000FAE);

}

/*****************************************************************************
 *
 * Prepare the CL550 for decompression.  This is called by C3PutImage.
 *
 *****************************************************************************/
static C3InitDecompress()
{
	C3_WREG(S_reset, ~0);

	/* Load configuration registers */

#ifdef RGB422
	C3_WREG(Config, 0x100 + 0x38);
#else
	C3_WREG(Config, 0x100 + 0x28); 
#endif

	C3_WREG(DRQ_Enable,0);
	C3_WREG(NMRQ_Enable,0);

	C3_WREG(Init_1, 0x003e);
	C3_WREG(Init_2, 0x0037);
	C3_WREG(Init_3, 0x01ff);
	C3_WREG(Init_4, 0x0049);
	C3_WREG(Init_5, 0x0005);
	C3_WREG(Init_7, 0x0022);
	C3_WREG(QuantSync, 0x043e);
	C3_WREG(QuantYC, 0x2033);
	C3_WREG(QuantAB, 0x0000);
	C3_WREG(QuantSelect, 1);
	C3_WREG(VideoLatency, 0x017f);

	/* Load coder configuration registers */

	C3_WREG(CodecSeq, 0x00cc);
	C3_WREG(CodecCompH, 0x0088);
	C3_WREG(CodecCompL, 0x0044);
	C3_WREG(CoderAttr, 0x00004);
	C3_WREG(CodingIntH, 0x0000);
#ifdef RGB422
	C3_WREG(CodingIntL, 0x0002);
#else
	C3_WREG(CodingIntL, 0x001);
#endif
	C3_WREG(DecLength, 0x0008);
	C3_WREG(CodeOrder, 0x0001);

	/* Load RGB-YUV conversion matrix */

	C3_WREG(R2Y,0x00000400);
	C3_WREG(G2Y,0x00000FFE);
	C3_WREG(B2Y,0x000005C9);
	C3_WREG(R2U,0x00000400);
	C3_WREG(G2U,0x00000EA2);
	C3_WREG(B2U,0x00000D23);
	C3_WREG(R2V,0x00000400);
	C3_WREG(G2V,0x0000071B);
	C3_WREG(B2V,0x00000FFD);

}

#endif    /* NOT DRIVER */
