/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: dmsdlib.c,v 1.1 1995/11/27 22:46:59 dist Exp $ Parallax Graphics Inc";
#endif

junkprint()
{
}

#ifdef DRIVER
static int driver_null_variable = 0;
#else /* the reset of the file */

#include "math.h"
#include "string.h"
#include "xvideo.h"
#include "plxv.h"
#include "plxv_xpoint.h"

/*******************  DMSD addresses *******************/
static char dmsd_7191_address[2] = {0x45,0x47};
static char dmsd_7192_address[2] = {0x70,0x71};
static char dmsd_tuner_address[2] = {0x00,0x01}; /* known wrong !! */

/******************  save settings & save time *********/

static int hue_save[4] = {0,0,0,0};
static int sat_save[4] = {0,0,0,0};
static int bright_save[4] = {0x8000,0x8000,0x8000,0x8000};
static int gain_save[4] = {0x8000,0x8000,0x8000,0x8000};
static int inverse_save[4] = {0,0,0,0};
static int sel_yuv_save[4] = {0,0,0,0};  /* also used for YC mode */
static int sel_625_save[4] = {0,0,0,0};
static int sel_bw_save[4]  = {0,0,0,0};

/* static variable for the RGB option CARD */
/*  RGB vector is R,G,B.... YUV vector is RY,Y,BY   */
static float yuv_to_rgb[] = {1.14,1.0,0.0,
                             -0.581,1.0,-0.395,
                              0.0,1.0,2.032};

static float rgb_to_bw[]     = { 0.299,  0.587,  0.114,
				 0.299,  0.587,  0.114,
				 0.299,  0.587,  0.114};

static float yuv_to_rgb_bw[] = { 0.0 ,1.0   ,0.0,
                                 0.0 ,1.0   ,0.0,
                                 0.0 ,1.0   ,0.0};

static float identity_matrix[] =   {1.0,0.0,0.0,
                                    0.0,1.0,0.0,
                                    0.0,0.0,1.0};


/**************end of RGB statics *****************/

static char delim[] = {"	 ,"};
				
#ifdef DEBUG
#define dprintf printf
#else
#define dprintf junkprint
#endif

#define IO_TYPE_CMP(a,b) ((a->flags & IO_CHAN_TYPE) == b)

XV_video_format loop_format = {640,482,43,7,21};
XV_video_format *plxv_io_format[5] = {0,0,0,0,&(loop_format)};
int plxv_opt_cntl_reg = 0;

XV_io_port *plxv_io_port(sel)
int sel;
{
    XV_io_port *iop;
    switch(sel)
    {
      case PORT_A:       /* main board */
        iop = &(xv_cp->opt.a); break;
      case PORT_B:       /* option board a port*/
        iop = &(xv_cp->opt.b); break;
      case PORT_C:       /* main board b port*/
	dprintf("error: io_port ptr not valid on C port\n");
	return((XV_io_port *) 0);
      case PORT_D:       /* main board b port*/
        iop = &(xv_cp->main.d); break;
      default:
	dprintf("error: io_port is out of range\n");
	return((XV_io_port *) 0);
    }
    return(iop);
}

#ifndef DRIVER

/*******************************  DMSD STUFF  *************************/
plxv_init_dmsd()
{
    int i,i1,j,k;
    XV_io_port *iop;
    unsigned long xpoint_val;
    
    for (i=0;i<4;i++)
    {
	if ( i == PORT_C ) continue;	/* skip the "C" port */
	iop = plxv_io_port(i);


	dprintf(" configuring %d:",i);
	plxv_io_format[i] = ((iop->flags &IO_CHAN_TYPE) ?&(iop->format[0]) :
			(XV_video_format *) 0);

	/* break out the individual types of channels */
	switch((iop->flags)& (IO_CHAN_TYPE)) 
	{
	  case IO_DMSD:
	    /* this channel is a DMSD input */
	    dprintf("is dmsd... unit: %d",iop->config.dmsd.unit);

	    if (( i == PORT_D) || (xv_cp->opt.board_code))
	    {   /* Iff board is really there !! */
		plxv_init_dmsd_7191(i);
	    	plxv_input_adjust(i,&(iop->factory));
	    }

	    switch (i)
	    {
	      case PORT_A:
	      case PORT_B:
		xpoint_val = (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH 
			      | ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
		break;
	      case PORT_D:
/* squash */
		if(xv_cp->main.board_code == SBUS_4)
		{
		  xpoint_val = (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH 
			      | ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
		}
		else
		{
		  xpoint_val = (D_CE_HIGH | D_SYNC_HIGH  | D_ODD_LOW  
				| ODD_INMODE_A  |  SHUTTLE_EN );
		}
		break;
	    }
	    break;
	  case IO_COMP_OUT:
	    dprintf("is COMP_OUT...");
	    xpoint_val = (iop->config.gen.xpoint_bits);
	    break;
	  case IO_RGB_IN:
	    dprintf("is RGB_IN...");
	    if (plxv_input_config(i,"RGB 525 COLOR NONINVERTED"))
	    {
		printf("plxv_init_dmsd: error configuring RGB_IN channel\n");
		break;
	    }
	    plxv_input_adjust(i,&(iop->factory));
	    xpoint_val = (iop->config.gen.xpoint_bits);
	    break;
	  case IO_SONYRGB_IN:
	    dprintf("is SONYRGB_IN...");
	    if (plxv_input_config(i,"RGB 525 COLOR NONINVERTED"))
	    {
		printf("plxv_init_dmsd: error configuring SONYRGB_IN channel\n");
		break;
	    }
	    plxv_input_adjust(i,&(iop->factory));
	    xpoint_val = (iop->config.gen.xpoint_bits);
	    break;
	  case IO_RGB_OUT:
	    plxv_output_config(i,"RGB 525 DISABLE");
	    dprintf("is RGB_OUT...");
	    xpoint_val = (iop->config.gen.xpoint_bits);
	    break;
	  case IO_GEN_IN:
	  case IO_GEN_OUT:
	    /* this channel is a general input/output */
	    xpoint_val = (iop->config.gen.xpoint_bits);
	    dprintf("is GEN_IO...");
	    break;
	  default:
	    /* there is nothing attached to this channel */
	    dprintf("is not used");
	    xpoint_val = 0;
	}

	/* now load the crosspoint switch */
	switch (i)
	{
	  case PORT_A:
	    plxv_xpoint_a(xpoint_val);
	    break;
	  case PORT_B:
	    plxv_xpoint_b(xpoint_val);
	    break;
	  case PORT_D:
/* squash */
	    if(xv_cp->main.board_code == SBUS_4)
	    {
	      plxv_xpoint_a(xpoint_val);
	    }
	    else
	    {
	      plxv_xpoint_d(xpoint_val);
	    }
	    break;
	} 
	dprintf("\n");
    }  /* end of i loop */
}

plxv_init_dmsd_7191(port)
int port;
{
    int i,paddr,sub_addr = 0;
    unsigned char *cptr;
    XV_io_port *port_ptr;

    if ( ( port_ptr = plxv_io_port(port)) == NULL)
    {
	printf("plxv_init_7191: attempt to init invalid port\n");
	return(-1);
    }
    if ( !IO_TYPE_CMP(port_ptr,IO_DMSD))
    {
	printf("plxv_init_7191: attempt to init non DMSD port %d\n",port);
	return(-1);
    }
    paddr = dmsd_7191_address[port_ptr->config.dmsd.unit];
    cptr = &(port_ptr->config.dmsd.val[0]);
    for (i=0;i<23;i++)
    {
	if (sub_addr == 0x12) sub_addr = 0x14;  /* no regs 0x12 & 13 */
	if (plxv_i2cw(paddr,sub_addr++,*cptr++))
	{   /* a write error has occured */
	    printf("plxv_init_dmsd_7191: i2c write error\n");
	    return(-1);
	}
    }

    /* these are for the 7191B */
    if (plxv_i2cw(paddr,0x12,0))
    {   /* a write error has occured */
	printf("plxv_init_dmsd_7191: i2c write error subaddr 0x12\n");
	return(-1);
    }
    if (plxv_i2cw(paddr,0x13,0))
    {   /* a write error has occured */
	printf("plxv_init_dmsd_7191: i2c write error subaddr 0x13\n");
	return(-1);
    }
    return(0);
}
#endif /* DRIVER */

/********************************************************/
/* Routine to load the given knobrecord into DMSD       */
/* pointed at by the config record			*/
/* nominal values for the knob record are 0x80		*/
/* knobrec simply offsets the CONFIG VALUES		*/
/********************************************************/


plxv_set_knobs(port,kptr)
int port;
XV_knob_rec  *kptr;
{
    plxv_input_adjust(port,kptr);
}

plxv_input_adjust(port,kptr)
int port;
XV_knob_rec *kptr;
{
    int paddr;
    XV_io_port *port_ptr;

    if ( ( port_ptr = plxv_io_port(port)) == NULL)
    {
	printf("plxv_input_adjust: attempt to adjust invalid port\n");
	return(-1);
    }

    hue_save[port] = kptr->hue;
    sat_save[port] = kptr->saturation;

    if ( IO_TYPE_CMP(port_ptr,IO_DMSD))
    {
        paddr = dmsd_7191_address[port_ptr->config.dmsd.unit];

        /* Saturation config.dmsd.val[17] */
	if (plxv_i2cw(paddr,0x11,kptr->saturation))
	{
	    printf("plxv_input_adjust: error writing to 7191\n");
	    return(-1);
	}

	/* Hue */
	if (plxv_i2cw(paddr,0x7,kptr->hue - 0x80))
	{
	    printf("plxv_input_adjust: error writing to 7191\n");
	    return(-1);
	}
    } else if ( IO_TYPE_CMP(port_ptr,IO_RGB_IN) ||
		IO_TYPE_CMP(port_ptr,IO_SONYRGB_IN))
    {   /* do matrix manipulation to get hue and sat on the RGB input */
	int i,j;
	i = kptr->hue - 0x80;  j = kptr->saturation;
	if (sel_bw_save[port] == 0) plxv_load_bt281_hue_sat(i,j, sel_yuv_save[port]);
    } else
    {
	printf("plxv_input_adjust: unknown channel type \n");
    }

    /* brightness and contrast  */
    plxv_load_input_clt(port,kptr->bright,kptr->contrast,0);
}

/********************************************************/
/* Routine to SAVE the given knobrecord into NVRAM      */
/* pointed at by the io_port record.                     */
/* Note: this routine also updates the XV_structure	*/
/* sel: 0 = factory, 1= user1, 2= user2			*/
/********************************************************/
plxv_save_knobs(port,sel,kptr)
int port,sel;
XV_knob_rec  *kptr;
{
    int nv_addr,i;
    unsigned char *cp,*cp1;
    XV_io_port *iop,*port_ptr;

    if (sel > 2)
    {
	printf("plxv_save_knobs: select out of range\n");
	return(-1);
    }

    if ( ( port_ptr = plxv_io_port(port)) == NULL)
    {
	printf("plxv_save_knobs: attempt to save on invalid port\n");
	return(-1);
    }

    switch(port)
    {
      case PORT_D:	/* main board  (d)*/
	cp = &(xv_cp->main.d.factory.bright);
	nv_addr = NV_POSITION(main,d.factory);
	nv_addr += sel * (sizeof(XV_knob_rec));
	if (plxv_nv_write(0,kptr,nv_addr,sizeof(XV_knob_rec)))
	{   /* error writing the control board ram */
	    printf("plxv_save_knobs: error writing controller nvram\n");
	    return (-1);
	}
	break;
      case PORT_A:	/* option card (a) */
	cp = &(xv_cp->opt.a.factory.bright);
	nv_addr = NV_POSITION(opt,a.factory);
	nv_addr += sel * (sizeof(XV_knob_rec));
	if (plxv_nv_write(1,kptr,nv_addr,sizeof(XV_knob_rec)))
	{   /* error writing the control board ram */
	    printf("plxv_save_knobs: error writing option nvram\n");
	    return (-1);
	}
	break;
      case PORT_B:	/* option card (b) */
	cp = &(xv_cp->opt.b.factory.bright);
	nv_addr = NV_POSITION(opt,b.factory);
	nv_addr += sel * (sizeof(XV_knob_rec));
	if (plxv_nv_write(1,kptr,nv_addr,sizeof(XV_knob_rec)))
	{   /* error writing the control board ram */
	    printf("plxv_save_knobs: error writing option nvram\n");
	    return (-1);
	}
	break;
    }

    /* now update memory image of NV */
    cp += sel * (sizeof(XV_knob_rec));
    cp1 = (unsigned char *)kptr;
    for (i=0;i<4;i++) *cp++ = *cp1++;

    /* bye */
    return(0);
}



/************************************************/
/* nominal for offset and gain are 0x80         */
/* range s.b limited to 0 -> 0xff, but not clipd*/
/* (these are brightness and contrast controls  */
/* this is a SLOW process, so its skipped when	*/
/* the values loaded are the same as last time	*/
/************************************************/

plxv_load_input_clt(port,bright,gain,must)
int port,bright,gain,must;
{
    int clt[0x100];
    int i,j,k,temp;
    float a,b,c;
    XV_io_port *port_ptr;

    if ( !must && (bright_save[port] == bright) && (gain_save[port] == gain))
    {   /* duplicate load, skip */
	dprintf("skipping duplicate input_clt load \n");
	return(0);
    }

    bright_save[port] = bright;  gain_save[port] = gain;

    /* calculate the coeffiecents */
    a = (float)(bright - 0x80) * 0.6;
    b = (float) (gain - 0x80) /128.0 + 1.00;

    /* possibly reverse video if inverse[unit] is set */
    if ( inverse_save[port])
    {
	a = 256.0 - a;
	b = -b;
    }

    /* now load the 256 location mapping ram */
    for (i=0;i,i<0x100;i++)
    {
	c = a + (b * (float)i) + 0.5;
	k = clip((int)c);
	clt[i] = k;
/*	dprintf("%3d  ",k); if ( (i & 0xf) == 0xf) dprintf("\n"); /* */
    }
    

    if ( ( port_ptr = plxv_io_port(port)) == NULL)
    {
	printf("plxv_input_adjust: attempt to adjust invalid port\n");
	return(-1);
    }

    if ( IO_TYPE_CMP(port_ptr,IO_DMSD))
    {   /* load the 7192 clt */
	plxv_load_7192_clt(&(port_ptr->config.dmsd),clt);
    }
    else if ( IO_TYPE_CMP(port_ptr,IO_RGB_IN) ||
	      IO_TYPE_CMP(port_ptr,IO_SONYRGB_IN))
    {
	/* load the 281 clt (R,G & B the same for rgb) */
	/* load the 281 clt (just g for yuv) */
	if (sel_yuv_save[port])
		plxv_load_bt281_clt(clt,0x2);
	else
		plxv_load_bt281_clt(clt,0x7);
    }
    else
    {
	printf("plxv_load_input_clt: unknown input type\n");
	return(-1);
    }

    

}
plxv_load_7192_clt(rptr,clt)
int *clt;
XV_dmsd_config *rptr;
{
    int i,j,k,temp;
    int unit;
    float a,b,c;

    unit = rptr->unit;

    /* set the 7192 to bypass mode to hide the changes (about .1 sec) */
    /* this is done with a 7191 mode bit GPSW1, at bit 1 of register  */
    /* at sub-address 0xe					      */

    temp = rptr->val[14];
    plxv_i2cw(dmsd_7191_address[unit],0xe,~2 & temp);  /* a mode bit */
    plxv_i2cw(dmsd_7192_address[unit],0,0x3a);   /* enable write to vluts */

    /* now load the 256 location mapping ram inside the 7192 */
    if((unit==0 && (xv_cp->main.misc_flags & SAA7192A)) ||
       (unit==1 && (xv_cp->opt.misc_flags & SAA7192A))) /* chk for rev A */
    {
	i2c_set_addr(dmsd_7192_address[unit],4);
	i2cwd(0);
	i2cack(3);
    }
    else
    {
	i2c_set_addr(dmsd_7192_address[unit],1);
    }

    for (i=0;i<0x100;i++)  
    {
    	i2cwd(*clt++);
	i2cack(3);
    }
    /*set the 7192 back into operational mode */
    plxv_i2cw(dmsd_7192_address[unit],0,0xfa);   /* enable read to vluts */
    plxv_i2cw(dmsd_7191_address[unit],0xe,2 | temp); /*disable bypass */
}

static int clip(val)
int val;
{
    if (val < 0) return(0);
    if (val > 0xff) return(0xff);
    return(val);
}

/* DMSD status routine: returns a longword with the following bitfields */
/* bit value:		meaning:
	 1		chroma locked
	0x20	    60 hz vertical (NTSC) (if zero, then PAL) - if AUTO ??
		   - this bit is valid from last correct line lock
	0x40	    NOT Line locked (sync locked )		
*/
int plxv_input_status(port,val)
int port;
int *val;
{
    int paddr,i,j,k,l;
    XV_io_port *port_ptr;

    if ( ( port_ptr = plxv_io_port(port)) == NULL)
    {
	printf("plxv_input_status: attempt to stat invalid port\n");
	return(-1);
    }

    if ( IO_TYPE_CMP(port_ptr,IO_DMSD))
    {
	paddr = dmsd_7191_address[port_ptr->config.dmsd.unit];
	if (plxv_i2crdir(paddr,&j))
	{
	    printf("plxv_dmsd_status: i2c read error\n");
	    return(-1);
	}
	if (((j >> 6) & 1) == 0)
	{
	    dprintf("line locked\n");
	}
	else
	{
	    dprintf("line not locked\n");
	}
	if (((j >> 5) & 1) == 1) dprintf("60 hz\n"); else dprintf("50 hz\n");
	if ((j &1) == 1)
	{
	    dprintf("chroma locked\n");
	}
	else
	{
	    dprintf("chroma not locked\n");
	}
    }
    else if ( IO_TYPE_CMP(port_ptr,IO_RGB_IN) ||
	      IO_TYPE_CMP(port_ptr,IO_SONYRGB_IN))
    {
	XV_RREG(i,Ext.reg3);		/* read epld readback register */
	if (i & 2)
	    dprintf("no signal detected\n");
	else
	{
	    if (i & 1) 
		dprintf("50 hz signal detected\n");
	    else 
		dprintf("60 hz signal detected\n");
	}
	k = (~i & 1) << 5;
	l = (i & 2) << 5;
	j = k | l;
    }
    j &= 0x61;
    *val = j;
    return(0);
}

#ifndef DRIVER /* */

plxv_output_config(port,cmd_str)
int port;
char *cmd_str;
{
    int output_option_reg = 0;
    int return_val = 0;
    int i,j,k,l,temp;
    char *cp1,*cp2,*cp3;
    XV_io_port *iop;
    int chan_type;

    if ( ( iop = plxv_io_port(port)) == NULL)
    {
	printf("plxv_output_config: attempt to config invalid port\n");
	return(-1);
    }

    if ((iop->flags & IO_OUTPUT) == 0)
    {   /* This channel doesn't output */
	printf("plxv_output_config: attempt to use input port as output\n");
	return(-1);
    }

    cp1 = (char *)malloc(strlen(cmd_str)+1);
    strcpy (cp1,cmd_str);
    cp2 = strtok(cp1,delim);

    output_option_reg = 0;

    while (cp2 != NULL)
    {
	dprintf("command is: %s...\n",cp2);
	if (strcmp(cp2,"525") == 0)
	{
	    plxv_io_format[port] = &(iop->format[0]);
	    plxv_opt_cntl_reg &= ~2;
	    output_option_reg = 1;
	} 
	else if (strcmp(cp2,"625") == 0)
	{
	    plxv_io_format[port] = &(iop->format[1]);
	    plxv_opt_cntl_reg |= 2;
	    output_option_reg = 1;
	} 
	else if (strcmp(cp2,"RGB") == 0)
	{
	    if ( IO_TYPE_CMP(iop,IO_RGB_OUT))
	    {
		plxv_opt_cntl_reg &= ~0x20;
		output_option_reg = 1;
	    }  
	    else
	    {
		return_val = -1;
	    }
	} 
	else if (strcmp(cp2,"YUV") == 0)
	{
	    if ( IO_TYPE_CMP(iop,IO_RGB_OUT))
	    {
		plxv_opt_cntl_reg |= 0x20;
		output_option_reg = 1;
	    }  
	    else
	    {
		return_val = -1;
	    }
	} 
	else if (strcmp(cp2,"ENABLE") == 0)
	{
	    plxv_opt_cntl_reg |= 0x8;
	    output_option_reg = 1;
	}
	else if (strcmp(cp2,"DISABLE") == 0)
	{
	    plxv_opt_cntl_reg &= ~0x8;
	    output_option_reg = 1;
	}
	else if (strcmp(cp2,"GENLOCK") == 0)
	{
	    plxv_opt_cntl_reg |= 1;
	    output_option_reg = 1;
	    set_option_rsaddr(3,6);
	    XV_RREG(temp,Ext.reg1);
	    dprintf("genlock before = 0x%x\n",temp);
	    temp |= 0x4;
	    set_option_rsaddr(3,6);
	    XV_WREG(Ext.reg1,temp);
	    dprintf("genlock after = 0x%x\n",temp);
	} 
	else if (strcmp(cp2,"FREERUN") == 0)
	{
	    plxv_opt_cntl_reg &= ~1;
	    output_option_reg = 1;
	    set_option_rsaddr(3,6);
	    XV_RREG(temp,Ext.reg1);
	    dprintf("genlock before = 0x%x\n",temp);
	    temp &= 0xfffb;
	    set_option_rsaddr(3,6);
	    XV_WREG(Ext.reg1,temp);
	    dprintf("genlock after = 0x%x\n",temp);
	} 
	else
	{   /* BAD COMMAND */
	    printf("plxv_output_config: Bad command: %s\n",cp2);
	    return(-1);
	}
	cp2 = strtok(NULL,delim);
    }
    if (output_option_reg) XV_WREG(Ext.reg2,plxv_opt_cntl_reg);
    free(cp1);
    return(return_val);
}

plxv_input_config(port,cmd_str)
int port;
char *cmd_str;
{
    int output_option_reg = 0;
    int return_val = 0;
    int unit;
    int i,j,k,l,temp,paddr;
    char *cp1,*cp2;
    XV_io_port *iop;

    if ( ( iop = plxv_io_port(port)) == NULL)
    {
	printf("plxv_input_config: attempt to config invalid port\n");
	return(-1);
    }
    if ((iop->flags & IO_OUTPUT) != 0)
    {   /* This channel doesn't input */
	printf("plxv_input_config: attempt to use output port as input\n");
	return(-1);
    }

    dprintf("plxv_input_config:\n");
    cp1 = (char *)malloc(strlen(cmd_str)+1);
    strcpy (cp1,cmd_str);
    cp2 = strtok(cp1,delim);

    while (cp2 != NULL)
    {
	dprintf("command is: %s...\n",cp2);
	if (strcmp(cp2,"525") == 0)
	{
	    plxv_io_format[port] = &(iop->format[0]);
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN) ||
		 IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		/* disconnect sync */
		plxv_read_bt253(0,&temp);	
		sleep(1);
		plxv_load_bt253(0,temp & 0x8f);
		/* load bt261 registers */
		sel_625_save[port] = 0;
		plxv_init_bt261(port,sel_625_save[port]);
		plxv_opt_cntl_reg &= ~0x10;
	 	output_option_reg = 1;
		/* reconnect sync */
		plxv_load_bt253(0,temp);
	    } 
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		temp = iop->config.dmsd.val[0xf];
		temp &= ~0x80;		/* disable auto mode */
		temp |= 0x40;		/* set fsel to 60hz */
		temp &= ~0x20;		/* clear secam noise reduction */
		iop->config.dmsd.val[0xf] = temp;
		paddr = dmsd_7191_address[iop->config.dmsd.unit];
		plxv_i2cw(paddr,0xf,temp);
	    }
	} 
	else if (strcmp(cp2,"625") == 0)
	{
	    plxv_io_format[port] = &(iop->format[1]);
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN))
	    {
		/* disconnect sync */
		plxv_read_bt253(0,&temp);	
		sleep(1);
		plxv_load_bt253(0,temp & 0x8f);
		/* load bt261 registers */
		sel_625_save[port] = 1;
		plxv_init_bt261(port,sel_625_save[port]);
		plxv_opt_cntl_reg |= 0x10;
	 	output_option_reg = 1;
		/* reconnect sync */
		plxv_load_bt253(0,temp);
	    }
	    else if ( IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		dprintf("plxv_input_config: SONYCAM mode\n");
		/* disconnect sync */
		plxv_read_bt253(0,&temp);	

		plxv_load_bt253(0,temp | 0x70);
		/* load bt261 registers */

		plxv_load_bt261(0x4,226);
		plxv_load_bt261_short(0x8,67);
		plxv_load_bt261_short(0xa,0);
		plxv_load_bt261_short(0xc,76);
		plxv_load_bt261_short(0xe,112);
		plxv_load_bt261_short(0x10,895);
		plxv_load_bt261_short(0x12,144);
		plxv_load_bt261_short(0x14,226);
		plxv_load_bt261_short(0x16,678);
		plxv_load_bt261_short(0x18,417);
		plxv_load_bt261_short(0x1a,488);
		if(xv_cp->opt.board_code == OPT_4)
		{
			plxv_load_bt261_short(0x1c,907);
		}
		else
		{
			plxv_load_bt261_short(0x1c,905);
		}

		plxv_opt_cntl_reg |= 0x10;
	 	output_option_reg = 1;

		/* reconnect sync */
		sleep(1);
		plxv_load_bt253(0,0x60);
 	    }
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		temp = iop->config.dmsd.val[0xf];
		temp &= ~0x80;		/* disable auto mode */
		temp &= ~0x40;		/* set fsel to 50hz */
		iop->config.dmsd.val[0xf] = temp;
		paddr = dmsd_7191_address[iop->config.dmsd.unit];
		plxv_i2cw(paddr,0xf,temp);
	    }
	} 
	else if (strcmp(cp2,"PAL") == 0)
	{
	    dprintf("switching to PAL mode\n");
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN) ||
		 IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		dprintf("no PAL mode for RGB input\n");
		return_val = -1;
	    }
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		/* select pal mode */
		temp = iop->config.dmsd.val[0xd];
		temp &= ~1;
		iop->config.dmsd.val[0xd] = temp;
		paddr = dmsd_7191_address[iop->config.dmsd.unit];
		plxv_i2cw(paddr,0xd,temp);
		/* set secam cross color reduction to off */ 
		temp = iop->config.dmsd.val[0xf];
		temp &= ~0x20;
		iop->config.dmsd.val[0xf] = temp;
		plxv_i2cw(paddr,0xf,temp);
	    }
	}
	else if (strcmp(cp2,"SECAM") == 0)
	{
	    dprintf("switching to SECAM mode\n");
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN) ||
		 IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		dprintf("no SECAM mode for RGB input\n");
                return_val = -1;
            }
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
            {
		/* select secam mode */
		temp = iop->config.dmsd.val[0xd];
                temp |= 1;
                iop->config.dmsd.val[0xd] = temp;
                paddr = dmsd_7191_address[iop->config.dmsd.unit];
                plxv_i2cw(paddr,0xd,temp);

                /* set secam cross color reduction to on */ 
                temp = iop->config.dmsd.val[0xf];
                temp |= 0x20;
		dprintf("setting 0xf to %x\n",temp);
                iop->config.dmsd.val[0xf] = temp;
                plxv_i2cw(paddr,0xf,temp);
            }   
	}
	else if (strcmp(cp2,"AUTO") == 0)
	{
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN) ||
		 IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		dprintf("no auto mode for RGB input\n");
		return_val = -1;
	    } 
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		temp = iop->config.dmsd.val[0xf];
		temp |= 0x80;		/* enable auto mode */
		temp &= ~0x20;		/* clear secam noise reduction */
		iop->config.dmsd.val[0xf] = temp;
		paddr = dmsd_7191_address[iop->config.dmsd.unit];
		plxv_i2cw(paddr,0xf,temp);
	    }
	} 
	else if (strcmp(cp2,"BW") == 0)
	{
	    sel_bw_save[port] = 1;
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN) ||
		 IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		dprintf("setting RGB_IN to bw\n");
		if (sel_yuv_save[port])
		{
		    plxv_load_bt281_csc(yuv_to_rgb_bw);
		}
		else
		{
		    plxv_load_bt281_csc(rgb_to_bw);
		}
	    } 
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		/* kill the color demodulator */
		paddr = dmsd_7191_address[iop->config.dmsd.unit];
		plxv_i2cw(paddr,0x8,0);

		/* remove the chroma trap */
		temp = iop->config.dmsd.val[0x6];
		temp |= 0x80;
		iop->config.dmsd.val[0x6] = temp;
		plxv_i2cw(paddr,0x6,temp);
	    }
	}
	else if (strcmp(cp2,"COLOR") == 0)
	{
	    sel_bw_save[port] = 0;
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN) ||
		 IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		dprintf("reload RGB_IN to color\n");
		plxv_load_bt281_hue_sat(
		    hue_save[port]-0x80,sat_save[port],sel_yuv_save[port]);
	    } 
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		dprintf("setting DMSD  back to color\n");
		if (sel_yuv_save[port] == 0 )
		{   /* restore color mode on compostie input */
		    /* now set bpf & chroma trap, etc */
		    temp = iop->config.dmsd.val[0x6];
		    temp &= ~0x80;		/* use chroma trap */
		    iop->config.dmsd.val[0x6] = temp;
		    paddr = dmsd_7191_address[iop->config.dmsd.unit];
		    plxv_i2cw(paddr,0x6,temp);
		    plxv_i2cw(paddr,0x8,iop->config.dmsd.val[0x8]);
	        }
		else
		{
		    /* now set bpf & chroma trap, etc */
		    temp = iop->config.dmsd.val[0x6];
		    temp |= 0x80;		/* don't use chroma trap */
		    iop->config.dmsd.val[0x6] = temp;
		    paddr = dmsd_7191_address[iop->config.dmsd.unit];
		    plxv_i2cw(paddr,0x6,temp);
		    plxv_i2cw(paddr,0x8,iop->config.dmsd.val[0x8]);
		}
		if ( xv_cp->opt.board_code == OPT_1 )
		{
		    /* set the color kill bit which selects unused yc input */
		    set_option_rsaddr(3,6);	/* bt473 command register */
		    XV_RREG(temp,Ext.reg1);
		    temp &= ~4;
		    XV_WREG(Ext.reg1,temp);
		}
	    }
	}
	else if (strcmp(cp2,"COLORKILL") == 0)
	{
	    sel_bw_save[port] = 1;
	    if ( IO_TYPE_CMP(iop,IO_RGB_IN) ||
		 IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		dprintf("setting RGB_IN to bw\n");
		if (sel_yuv_save[port])
		{
		    plxv_load_bt281_csc(yuv_to_rgb_bw);
		}
		else
		{
		    plxv_load_bt281_csc(rgb_to_bw);
		}
	    }
	    else if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		if (port == PORT_D)
		{
		    dprintf("killing main card DMSD choma\n");
		    /* select YC mode with digital grounded chroma */
		    paddr = dmsd_7191_address[iop->config.dmsd.unit];
		    temp = iop->config.dmsd.val[0xe];
		    temp &= ~0x80;
		    iop->config.dmsd.val[0xe] = temp;
		    plxv_i2cw(paddr,temp);
		    /* now set bpf & chroma trap, etc */
		    temp = iop->config.dmsd.val[0x6];
		    temp |= 0x80;	
		    iop->config.dmsd.val[0x6] = temp;
		    plxv_i2cw(paddr,0x6,temp);
		}
		else
		{
		    /* kill the color demodulator */
		    paddr = dmsd_7191_address[iop->config.dmsd.unit];
		    plxv_i2cw(paddr,0x8,0);
		    /* set the color kill bit which selects unused yc input */
		    set_option_rsaddr(3,6);	/* bt473 command register */
		    XV_RREG(temp,Ext.reg1);
		    temp |= 4;
		    XV_WREG(Ext.reg1,temp);
		}
	    }
	    else
	    {
		printf("plxv_input_config: unknown channel type on port %d\n"
			,port);
		return_val = -1;
	    }
	}
	else if (strcmp(cp2,"RGB") == 0)
	{
	    if (IO_TYPE_CMP(iop,IO_RGB_IN) ||
		IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		/* setup input color space converter for rgb */
		sel_yuv_save[port] = 0;
		plxv_init_bt281(sel_yuv_save[port]);
		plxv_init_bt253(port,sel_yuv_save[port]);
		plxv_load_bt281_csc(identity_matrix);
		set_option_rsaddr(3,6);
		XV_RREG(temp,Ext.reg1);
		dprintf("isel_yuv before = 0x%x\n",temp);
		temp &= 0xfffd;
		set_option_rsaddr(3,6);
		XV_WREG(Ext.reg1,temp);
		dprintf("isel_yuv after = 0x%x\n",temp);
	    }
	    else
	    {
	        dprintf("plxv_input_config: no RGB mode on port %d \n",port);
		return_val = -1;
	    }
	} 
	else if (strcmp(cp2,"YUV") == 0)
	{
	    if (IO_TYPE_CMP(iop,IO_RGB_IN) ||
		IO_TYPE_CMP(iop,IO_SONYRGB_IN))
	    {
		/* setup input color space converter for yuv */
		sel_yuv_save[port] = 1;
		plxv_init_bt281(sel_yuv_save[port]);
		plxv_init_bt253(port,sel_yuv_save[port]);
		plxv_load_bt281_csc(yuv_to_rgb);
		set_option_rsaddr(3,6);
		XV_RREG(temp,Ext.reg1);
		dprintf("isel_yuv before = 0x%x\n",temp);
		temp |= 0x2;
		set_option_rsaddr(3,6);
		XV_WREG(Ext.reg1,temp);
		dprintf("isel_yuv after = 0x%x\n",temp);
	    }
	    else
	    {
	        dprintf("plxv_input_config: no YUV mode on port %d \n",port);
		return_val = -1;
	    }
	} 
	else if (strcmp(cp2,"COMPOSITE") == 0)
	{
	    if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		if ( port == PORT_D)
		{
		}
		else if ( xv_cp->opt.board_code == OPT_1 )
		{   /* this is the VIO card, and has YC mode */
		    if (xv_cp->opt.board_rev != 0)
		    {   /* this rev has yc input on "1" (seperate from comp) */
			temp = iop->config.dmsd.val[14];
			temp &= ~0x1;
			iop->config.dmsd.val[14] = temp;
			paddr = dmsd_7191_address[1];	/* option card */
    			plxv_i2cw(paddr,0xe,temp); 
			printf("plxv_input_config: not implemented\n");

			/* now set bpf & chroma trap, etc */
			temp = iop->config.dmsd.val[0x6];
			temp &= ~0x80;		/* use chroma trap */
			iop->config.dmsd.val[0x6] = temp;
			/*
			paddr = dmsd_7191_address[iop->config.dmsd.unit];
			*/
			plxv_i2cw(paddr,0x6,temp);
		    }
		}
	    }
	    else
	    {
	        dprintf("no COMPOSITE mode on channel %d \n",port);
		return_val = -1;
	    }
	} 
	else if (strcmp(cp2,"YC") == 0)
	{
	    if (IO_TYPE_CMP(iop,IO_DMSD))
	    {
		if ( port == PORT_D)
		{
		    dprintf("plxv_input_config: no YC on port %d",port);
		    return_val = -1;
		}
		else if ( xv_cp->opt.board_code == OPT_1 )
		{   /* this is the VIO card, and has YC mode */
		    if (xv_cp->opt.board_rev == 0)
		    {   /* this rev has yc input on "0" (with comp) */
		    }
		    else
		    {   /* this rev has yc input on "1" (seperate from comp) */
			temp = iop->config.dmsd.val[14];
			temp |= 0x1;    /* set the mode bit for hw switch */
			iop->config.dmsd.val[14] = temp;
			paddr = dmsd_7191_address[1];	/* option card */
    			plxv_i2cw(paddr,0xe,temp); 

			/* now set bpf & chroma trap, etc */
			temp = iop->config.dmsd.val[0x6];
			temp |= 0x80;	
			iop->config.dmsd.val[0x6] = temp;
			plxv_i2cw(paddr,0x6,temp);
		    }
		}
	    }
	    else
	    {
	        dprintf("plxv_input_config: no YC mode on port %d \n",port);
		return_val = -1;
	    }
	} 
	else if (strcmp(cp2,"NONINVERTED") == 0)
	{   /* UNreverse video, update the flag & do the operation */
	    if ((IO_TYPE_CMP(iop,IO_RGB_IN)) ||
		(IO_TYPE_CMP(iop,IO_SONYRGB_IN)) ||
		(IO_TYPE_CMP(iop,IO_DMSD)))
	    {
		inverse_save[port] = 0;
		plxv_load_input_clt(port,bright_save[port],gain_save[port],1);
	    }
	    else
	    {
		printf("plxv_input_config: unknown channel type\n");
		return_val = -1;
	    }
	}
	else if (strcmp(cp2,"INVERTED") == 0)
	{   /* reverse video, update the flag & do the operation */
	    if ((IO_TYPE_CMP(iop,IO_RGB_IN)) ||
		(IO_TYPE_CMP(iop,IO_SONYRGB_IN)) ||
		(IO_TYPE_CMP(iop,IO_DMSD)) )
	    {
		inverse_save[port] = 1;
		plxv_load_input_clt(port,bright_save[port],gain_save[port],1);
	    }
	    else
	    {
		printf("plxv_input_config: unknown channel type\n");
		return_val = -1;
	    }
	}
	else
	{   /* BAD COMMAND */
	    printf("plxv_input_config: Bad command: %s\n",cp2);
	    return(-1);
	}
	cp2 = strtok(NULL,delim);
    }
    if (output_option_reg) XV_WREG(Ext.reg2,plxv_opt_cntl_reg);
    free(cp1);
    return(return_val);
}

#endif /* DRIVER */



/****************************/
/* matrix stuff for 281 csc */
/****************************/

/* m3 = m1 * m2 (square matricies of order "order" */
/* m[0] m[1] .... m[order-1]
   m[order].......m[2order-1]
   ..........................
               ..m[order*order-1]
*/

static mat_mult(order,m1,m2,m3)
int order;
float *m1,*m2,*m3;
{
    int i,j,k;

    for (i=0;i<order;i++)
    {
	for (j=0;j<order;j++)
	{
	    m3[order*i + j] = 0;
	    for (k=0;k<order;k++)
	    {
	        m3[order*i + j] += m1[order*i + k]  * m2[order*k + j];
	    }
	}
    }
}
	   
/* returned the cofactor (det of rol/col deleted matrix * parity[entry]) */
/* m1 matrix has order "order" */
static float cofactor(order,entry,m1)
int entry;
float *m1;
{
    int i,j,k;
    int a,b,c,d;
    int row,col;
    float s[16];
    float determinant();
    float cof;

    k = 0;
    row = entry / order;
    col = entry % order;
    for (i=0;i<order;i++)
    {
        for (j=0;j<order;j++)
	{
	    if ((i != row) && (j != col))
	    s[k++] = m1[order * i + j];
	}
    }
    cof = determinant(order-1,s);
    if (entry & 1)
	cof *= -1;
    return(cof);
}

static float determinant(order,m1)
int order;
float *m1;
{
    float det;
    float cofactor();
    int i;
	
    if (order == 1) return(m1[0]);
    det = 0;
    for (i=0;i<order;i++)
    {
	det += m1[i] * (cofactor(order,i,m1));
    }
    return(det);
}

/* m2 = m1 ^ -1 */
static mat_invert(order,m1,m2)
int order;
float *m1,*m2;
{
    int i,j,k;
    float d;

    d = determinant(order,m1);
    for (i=0;i<order;i++)
    {
	for (j=0;j<order;j++)
	{
	    m2[order * i + j] = cofactor(order,order * j + i,m1) / d;
	}
    }
}

static plxv_load_bt281_hue_sat(hue,sat,sel_yuv)
int hue,sat,sel_yuv;
{
    float hue_sat_yuv[9];
    float hue_sat_rgb[9];
    float hue_mat[9];
    float sat_mat[9];
    float inter1[9],inter2[9],inter3[9];
    float inter4[9];
    float rgb_to_yuv[9];
    int i;
    float to_rads;

    to_rads = 3.1415926 / 128.0;
    for (i=0;i<9;i++)
    {
	hue_mat[i] = 0;
	sat_mat[i] = 0;
    }

/* hue matrix */
    hue_mat[0] = cos((float)hue * to_rads);
    hue_mat[2] = -sin((float)hue * to_rads);
    hue_mat[4] = 1.0;
    hue_mat[6] = sin((float)hue * to_rads);
    hue_mat[8] = cos((float)hue * to_rads);
/* saturation matrix */
    sat &= 0xff;
    sat_mat[0] = (float)(sat) / 128.0;
    sat_mat[4] = 1.0;
    sat_mat[8] = (float)(sat) / 128.0;
    
    mat_invert(3,yuv_to_rgb,rgb_to_yuv);

    if (sel_yuv)
    {
        mat_mult(3,sat_mat,hue_mat,inter1);
        mat_mult(3,yuv_to_rgb,inter1,hue_sat_rgb); 
        plxv_load_bt281_csc(hue_sat_rgb);
    }
    else
    {
        mat_mult(3,hue_mat,rgb_to_yuv,inter1);
        mat_mult(3,sat_mat,inter1,inter2);
        mat_mult(3,yuv_to_rgb,inter2,hue_sat_rgb);
        plxv_load_bt281_csc(hue_sat_rgb);
    }
}

#endif /*NOT DRIVER*/

