/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: i2clib.c,v $
 * Revision 1.1  1995/11/27  22:46:59  dist
 * Initial revision
 *
 * Revision 1.1  1995/10/30  17:27:33  kevin
 * Initial revision
 *
 * Revision 1.1  1995/10/13  21:35:04  kevin
 * Initial revision
 *
 * Revision 1.1  1995/03/30  19:42:32  kevin
 * Initial revision
 *
 * Revision 1.5  93/09/30  02:46:01  rocky
 * SQUASH, NVRAM timing
 * 
 * Revision 1.4  91/10/23  09:55:57  mears
 * new rev
 * 
 * 
 * Revision 1.3  91/08/06  15:09:45  mears
 * increased i2c speed 2x with DELAY_FALCTOR
 * 
 * Revision 1.2  91/07/26  19:06:51  eric
 * new from bill
 * 
 * Revision 1.1  91/06/12  16:06:30  eric
 * Initial revision
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: i2clib.c,v 1.1 1995/11/27 22:46:59 dist Exp $ Parallax Graphics Inc";
#endif

/* i2c wait routine (must be modified in compiled envirnment 
   this routine takes no parameters since we can use a single maximum
   required delay everywhere in the i2c code.
*/

#include "xvideo.h"
#include "plxv.h"


static char nv_address[3] = {0x50,0x51,0x52};
#ifdef DEBUG
#define dprintf(a) printf(a)
#else
#define dprintf(a) 
#endif

#ifdef sun
#define DELAY_FACTOR 5
#else
#define DELAY_FACTOR 4
#endif

#ifdef NOTDEF
plxv_nv_delay2(usec)
int usec;
{
    register int i,j,k;

    k = DELAY_FACTOR * usec;	/* this is about 3 X slow */
    for (i=0; i<k; i++) j += i;
}
#endif

plxv_get_timer_val()
{
    unsigned int i;
    XV_RREG(i,TmrCnt.l);
    return(i);
}

plxv_nv_delay(usec)
unsigned int usec;
{
    unsigned int baseval,newval,delta;
    baseval = plxv_get_timer_val();
/*
    XV_RREG(baseval,TmrCnt.l);
*/
    /* use scaled integer math */
    delta = ((usec*10000)/111111) + 2;

    while(1)
    {
	newval = plxv_get_timer_val();
/*
	XV_RREG(newval,TmrCnt.l);
*/
	if(newval>=(unsigned int)(baseval+delta))break;
    }
}

/* i2c start of message */
i2csom()
{
/*
XV_WREG(MiscMode.b.blu,7);
XV_WREG(MiscMode.b.blu,0);
*/
	XV_WREG(I2C.b.cen,0);		/* Jon enable clock output */
	XV_WREG(I2C.b.data, 1);		/* Make sure data is high before */
	XV_WREG(I2C.b.enbl, 1);		/* Set Data direction for write */
	XV_WREG(I2C.b.clk, 1);		/* Make sure clock is high	*/
	plxv_nv_delay(5);		/* SCL high time = 5us		*/
	XV_WREG(I2C.b.data, 0);		/* High-to-low transition on SDA */
	plxv_nv_delay(5);		/* Wait for Tsu (SCL>SDA)=5us	*/
	XV_WREG(I2C.b.clk, 0);		/* High-to-low transition on SCL */
	plxv_nv_delay(4);		/* Wait for Th (SCL<SDA) =4us	*/
}

/* send i2c slave address */
i2cpa(addr,read_flag)
int addr,read_flag;
{
        /* address is a 7-bit value */
	int i;
	for (i=0;i<7;i++) {
 	    XV_WREG(I2C.b.data, (addr & 0x40)>>6);  /* MSB	*/
	    plxv_nv_delay(1);	/* Wait for Tsu (SDA>SCL)=250ns	*/
	    XV_WREG(I2C.b.clk,1);	/* Low-to-high SCL transition	*/
	    plxv_nv_delay(5);	/* SCL high time = 5us		*/
	    XV_WREG(I2C.b.clk,0);	/* High-to-low SCL transition	*/
	    plxv_nv_delay(5);	/* Wait for Th (SCL<SDA)=5us	*/
	    addr <<= 1;
	}
	XV_WREG(I2C.b.data, read_flag);	
	plxv_nv_delay(1);
	XV_WREG(I2C.b.clk, 1);
	plxv_nv_delay(5);
	XV_WREG(I2C.b.clk, 0);
XV_WREG(I2C.b.enbl,0);
	plxv_nv_delay(5);
}

/* i2c acknowledge */
i2cack(context)
int context;
{
    /* context is used to tell this routine what is trying to be ack'ed
       so that a more informative error message can be generated
	context		meaning
	1		primary-address
	2		sub-address
	3		data-write

	this routine will return the value of context iff there is and
	error.  otherwise it will return 0.
    */ 

    int readback;	
    XV_WREG(I2C.b.enbl, 0);		/* Set the data direction for read   */
    plxv_nv_delay(10);			
    XV_WREG(I2C.b.clk,1);		/* Low-to-high SCL transition	     */
    plxv_nv_delay(5);		/* Wait for SCL Low Time=5us	     */
    XV_RREG(readback,I2CStat.b.data);	/* Jon */
    if ((readback & 1) == 1)
    {
	switch (context) 
	{
	    case 1: dprintf("i2c error writing slave address\n");
		break;
	    case 2: dprintf("i2c error writing sub-address\n");
		break;
	    case 3: dprintf("i2c error writing data\n");
		break;
	}
    }
    XV_WREG(I2C.b.clk,0);		/* High-to-low SCL transition	    */
    plxv_nv_delay(5);
/*   XV_WREG(I2C.b.enbl, 1);	*/
    plxv_nv_delay(5);		/* Wait for SCL Low Time=5us	    */
    if ((readback & 1) == 1) return(context);
    return(0);
}
 
/* i2c sub-address */
i2csa(saddr)
int saddr;
{
    int i;

XV_WREG(I2C.b.enbl,1);
    for (i=0;i<8;i++)
    {
	XV_WREG(I2C.b.data, (saddr & 0x80)>>7);  /* MSB		*/
	plxv_nv_delay(1);	/* Wait for Tsu (SDA>SCL)=250ns	*/
	XV_WREG(I2C.b.clk,1);	/* Low-to-high SCL transition	*/
	plxv_nv_delay(5);	/* SCL high time = 5us		*/
	XV_WREG(I2C.b.clk,0);	/* High-to-low SCL transition	*/
if (i == 7) XV_WREG(I2C.b.enbl,0);
	plxv_nv_delay(5);	/* Wait for Th (SCL<SDA)=5us	*/
	saddr<<=1;
    }
}

/* i2c end of message */
i2ceom()
{
    XV_WREG(I2C.b.enbl, 1);		/* enable output */ 
    plxv_nv_delay(10);	
    XV_WREG(I2C.b.data,0);
    plxv_nv_delay(1);	
    XV_WREG(I2C.b.clk,1);
    plxv_nv_delay(5);	
    XV_WREG(I2C.b.data,1);		/* set data high */
    plxv_nv_delay(5);		/* Wait for SCL Low Time=5us	*/
}


/* i2c write data */
i2cwd(data)
int data;
{
    int i;

XV_WREG(I2C.b.enbl,1);
    for (i=0;i<8;i++) 
    {
	XV_WREG(I2C.b.data, (data & 0x80)>>7);  /* MSB		*/
	plxv_nv_delay(1);	/* Wait for Tsu (SDA>SCL)=250ns	*/
	XV_WREG(I2C.b.clk,1);	/* Low-to-high SCL transition	*/
	plxv_nv_delay(5);	/* SCL high time = 5us		*/
	XV_WREG(I2C.b.clk,0);	/* High-to-low SCL transition	*/
if (i == 7) XV_WREG(I2C.b.enbl,0);
	plxv_nv_delay(5);	/* Wait for Th (SCL<SDA)=5us	*/
	data<<=1;
    }
}

/* send start, i2c slave address, and i2c sub-address */
/* error returns: 0 good, 1= no addr ack, 2 = no sub-addr ack */
i2c_set_addr(addr,saddr)
int addr,saddr;
{
    int i;
    i2csom();
    i2cpa(addr,0);  /* writing */
    if ((i=i2cack(1))) 
    {
	i2ceom();
	return(i);
    }
    i2csa(saddr);
    if ((i=i2cack(2)))
    {
	i2ceom();
	return(i);
    }
    return(0);
}

/* send i2c slave and sub-address then send second som and read byte */
plxv_i2cr(addr, saddr,data)
int addr,saddr,*data;
{
    /* i2c devices with more that 1 readable sub-address require
       a write cycle to pre-set the sub-address pointer prior to
       reading a data byte */

    int i,j,rb;

    i2c_set_addr(addr,saddr);
    i2csom();
    i2cpa(addr,1); /* reading */
    if (i2cack(1))
    {
	i2ceom();
	return(-1);  /* was 0 */
    }
    /* readback */
    XV_WREG(I2C.b.enbl, 0);	/* Set the data direction for read */
    plxv_nv_delay(5);		/* Wait for Tsu (SDA>SCL)=250ns	*/
    j=0;
    for (i=0;i<8;i++) 
    {
        j <<= 1;		/* this does nothing on first iter */
        XV_WREG(I2C.b.clk,1);	/* Low-to-high SCL transition	*/
        plxv_nv_delay(5);		/* Let the readback data settle	*/
        XV_RREG(rb,I2CStat.b.data);	/* Jon */
        j |= (rb & 1);
        XV_WREG(I2C.b.clk,0);	
        plxv_nv_delay(5);		/* Wait for SCL Low Time=5us	*/
    }
    i2ceom();
    *data = j;
    return(0);		/* added */
}

/* send i2c slave address then read data byte without sending sub-address */
plxv_i2crdir(addr,data)
int addr,*data;
{
    /* use this routine is no sub-address is require on a read or if the
       sub-address pointer has been previously set 
    */

    int i,j,rb;

    i2csom();
    i2cpa(addr,1); /* reading */
    if (i2cack(1))
    {
	i2ceom();
	return(-1);  /* was 0 */
    }
    /* readback */
    XV_WREG(I2C.b.enbl, 0);	/* Set the data direction for read */
    plxv_nv_delay(10);		/* Wait for Tsu (SDA>SCL)=250ns	*/
    j=0;
    for (i=0;i<8;i++) 
    {
        j <<= 1;		/* this does nothing on first iter */
	XV_WREG(I2C.b.clk,1);	/* Low-to-high SCL transition	*/
	plxv_nv_delay(5);		/* Let the readback data settle	*/
        XV_RREG(rb,I2CStat.b.data);	/* Jon */
if (i == 7) XV_WREG(I2C.b.enbl,0);
	j |= (rb & 1);
	XV_WREG(I2C.b.clk,0);	
	plxv_nv_delay(5);		/* Wait for SCL Low Time=5us	*/
    }
    i2ceom();
    *data = j;
    return(0);		/* added */
}

/* send slave address, sub-address and data byte */
plxv_i2cw(addr, saddr, data)
unsigned int addr,saddr,data;
{
    int i,i1;

    if ((i1 = i2c_set_addr(addr,saddr))) return(i1);
    i2cwd(data);
    i = i2cack(3);
    i2ceom();
    return(i);
}

nv_clock()
{
    /* routine to generate test clock for timing setup */
    printf("\B running clock test\n");
XV_WREG(I2C.b.cen,0);		/* Jon */
    while(1)
    {
	plxv_nv_delay(5);
	XV_WREG(I2C.b.clk,1);
	plxv_nv_delay(5);
	XV_WREG(I2C.b.clk,0);	/* High-to-low SCL transition	*/
    }
}

/****************************************************************/
/* nvram unit  assignement:
	0 - main board
	1 - option card
	2 - tuner box
*/

plxv_nv_write(unit,ptr,nv_addr,count)
int unit,nv_addr,count;
char *ptr;
{
    char *j;
    int i,k,paddr;

    paddr = nv_address[unit];
    j = ptr;
    for (i=0; i<count; i++)
    {
	k = plxv_i2cw(paddr,nv_addr++,*j++);
	if (k) return(k);
	for(k=0;k<3000;k++) plxv_nv_delay(10);
    }
    return(0);
}

plxv_nv_read(unit,ptr,nv_addr,count)
int unit,nv_addr,count;
char *ptr;
{
    char *j;
    int i,k,paddr,data;

    paddr = nv_address[unit];
    j = ptr;
    for (i=0; i<count; i++)
    {
        k = plxv_i2cr(paddr,nv_addr++,&data);
	*j++ = data;
	if (k) return(k);
    }
    return(0);
}
