#include <stdio.h>
#include "Xos.h"
#include "misc.h"
#include "X.h"
#include "input.h"
#include "opaque.h"
#include <sys/signal.h>

#ifdef MEMBUG_TRACE

static unsigned long mem_bug_free_count    = 0;
static unsigned long mem_bug_alloc_count   = 0;
static unsigned long mem_bug_realloc_count = 0;


#define MEM_FAIL_SCALE 100000
long Memory_fail = 0;
pointer minfree = NULL;

static void ReleaseBlock();
extern Bool Must_have_memory;


#define FIRSTMAGIC	0x11aaaa11
#define SECONDMAGIC	0x22aaaa22
#define FREEDMAGIC	0x33aaaa33
#define FREEFILL	0xfd
#define ALLOCFILL       0xfe

#define MAGIC_LENGTH	4
typedef unsigned long*		magic_type;


typedef struct _MallocHeader	*MallocHeaderPtr;

typedef struct _MallocHeader
{
	char		*fileName;
	unsigned int	lineNum;
	unsigned int	amount;
	unsigned int	time;
	unsigned int    pad;

	MallocHeaderPtr	prev;
	MallocHeaderPtr	next;

	magic_type	magic[MAGIC_LENGTH];
} MallocHeaderRec;



typedef struct _MallocTrailer
{
	magic_type	magic[MAGIC_LENGTH];
} MallocTrailerRec, *MallocTrailerPtr;



unsigned long	MemoryAllocTime;
unsigned long	MemoryAllocBreakpoint = ~0;
unsigned long	MemoryActive = 0;
unsigned long	MemoryValidate=0;

MallocHeaderPtr	MemoryInUse;

int membug_mallocs = 0;

#define request(amount)		\
	((amount) + sizeof (MallocHeaderRec) + sizeof (MallocTrailerRec))

#define Header(ptr)		\
	((MallocHeaderPtr) (((char *) ptr) - sizeof (MallocHeaderRec)))

#define Trailer(ptr)		\
	((MallocTrailerPtr) (((char *) ptr) + Header(ptr)->amount))



/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/
typedef struct _memdebugRec
{
        char            *fileName;
        unsigned int    lineNum;
        unsigned int    amount;
        unsigned int    count;
} *memdebugPtr, memdebugRec;

memdebugPtr memdebugList = NULL;
int memdebugNum = 0;

void
print_memory()
{
    MallocHeaderPtr	head, tmp;
    int i;

    for (head = MemoryInUse; head; head = head->next) {
        for (i = 0; i < memdebugNum; i++) {
            if (memdebugList[i].lineNum == head->lineNum) {
                if (strcmp(memdebugList[i].fileName, head->fileName) == 0) {
                    memdebugList[i].amount = memdebugList[i].amount + head->amount;
                    memdebugList[i].count++;
                    break;
                }
            }
        }
        if (i == memdebugNum) {
            memdebugList = (memdebugPtr)realloc(memdebugList, sizeof(memdebugRec) * (memdebugNum + 1));
            memdebugList[memdebugNum].fileName = head->fileName;
            memdebugList[memdebugNum].lineNum = head->lineNum;
            memdebugList[memdebugNum].amount = head->amount;
            memdebugList[i].count = 1;
            memdebugNum++;
        }
    }

    printf("=====================================================================\n");
    printf("\n\n   Filename     Line Number  count  amount\n");
    printf("---------------------------------------------------------------------\n");
    for (i = 0; i < memdebugNum; i++) {
        printf("%15s %8d %8d %9d\n", memdebugList[i].fileName, memdebugList[i].lineNum, memdebugList[i].count, memdebugList[i].amount);
    }
    printf("=====================================================================\n");
}

/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

void set_magic(magic, magic_id)
magic_type magic;
unsigned long      magic_id;
{
	int i;

	for (i=0; i<MAGIC_LENGTH; i++)
		magic[i] = magic_id;
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

Bool same_magic(magic, magic_id)
magic_type magic;
unsigned long      magic_id;
{
	int i;

	for (i=0; i<MAGIC_LENGTH; i++)
		if (magic[i] != magic_id) return(FALSE);

	return(TRUE);
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

#define ULONGS_PER_LINE	4

void dump_header(ptr)
unsigned long *ptr;
{
	unsigned long *p;
	int i, count, length;

	ErrorF("----------- Malloc Header --------------\n");
	ErrorF("There should be four 0x11aaaa11.\n\n");

	p = (unsigned long *) (Header(ptr)->magic);
	length = MAGIC_LENGTH;
	for (i=length, count=ULONGS_PER_LINE; i; p++, i--)
	{
		ErrorF("  0x%x", *p);
		if (--count==0 || i==1)
		{
			ErrorF("\n");
			count = ULONGS_PER_LINE;
		}
	}

	ErrorF("\n");
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

void dump_trailer(ptr)
unsigned long *ptr;
{
	unsigned long *p;
	int i, count, length;

	ErrorF("----------- Malloc Trailer --------------\n");
	ErrorF("There should be four 0x22aaaa22.\n\n");

	p = (unsigned long *) (Trailer(ptr)->magic);
	length = sizeof(MallocTrailerRec) >> 2;
	for (i=length, count=ULONGS_PER_LINE; i; p++, i--)
	{
		ErrorF("  0x%x", *p);
		if (--count==0 || i==1)
		{
			ErrorF("\n");
			count = ULONGS_PER_LINE;
		}
	}

	ErrorF("\n");
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

static unsigned long *SetupBlock(ptr, amount, fileName, lineNum)
unsigned long  *ptr;
unsigned long   amount;
char   *fileName;
unsigned long   lineNum;
{
	MallocHeaderPtr	head = (MallocHeaderPtr) ptr;
	MallocTrailerPtr	tail = (MallocTrailerPtr) (((char *) ptr)
	    + amount + sizeof (MallocHeaderRec));

	MemoryActive += amount;

	set_magic(head->magic, FIRSTMAGIC);
	set_magic(tail->magic, SECONDMAGIC);

	head->fileName = fileName;
	head->lineNum  = lineNum;
	head->amount   = amount;
	head->time     = MemoryAllocTime++;

	/* Attach ``head'' into linked list ``MemoryInUse'' */

	head->next = MemoryInUse;
	head->prev = 0;
	if (MemoryInUse)
		MemoryInUse->prev = head;
	MemoryInUse = head;

	return (unsigned long *)(((char *) ptr) + sizeof (MallocHeaderRec));
}




/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/



void MemError(err_msg, ptr, fileName, lineNum, callerName)
char    *err_msg;
pointer ptr;
char    *fileName;
unsigned long   lineNum;
char    *callerName;
{
	MallocHeaderPtr	head;

	printf("\n\nXserver Memory Error: see /tmp/sunfile\n\n\n");

	ErrorF("Detected during %s at [%s #%d]: %s, memory_address=0x%x\n\n",
	    callerName, fileName, lineNum, err_msg, ptr);

	dump_header(ptr);

	head = Header(ptr);
	ErrorF("Memory malloc or realloc at [%s #%d], amount=%d\n\n",
	    head->fileName, head->lineNum,
	    head->amount);

	dump_trailer(ptr);

	abort();	/* Cause core dump */
}




/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

void ValidateNode(ptr, fileName, lineNum, callerName)
pointer ptr;
char    *fileName;
unsigned long   lineNum;
char    *callerName;
{
	MallocHeaderPtr	head = Header(ptr);

	if (ptr < minfree)
		MemError("Pointer in static storage",
		    ptr, fileName, lineNum, callerName);

	if (((pointer) head) < minfree)
		MemError("Header in static storage",
		    ptr, fileName, lineNum, callerName);

	if (same_magic(head->magic,FREEDMAGIC))
		MemError("Memory freed", ptr, fileName, lineNum, callerName);

	if (!same_magic(head->magic,FIRSTMAGIC))
		MemError("Header not valid (Is this malloc'ed memory?)",
		    ptr, fileName, lineNum, callerName);

	if (!same_magic(Trailer(ptr)->magic,SECONDMAGIC))
		MemError("Trailer trashed", ptr, fileName, lineNum, callerName);
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

void ValidateAllActiveMemory(fileName, lineNum, callerName)
char    *fileName;
unsigned long   lineNum;
char    *callerName;
{
	MallocHeaderPtr	head;

	for (head = MemoryInUse; head; head = head->next)
		ValidateNode(head + 1, fileName, lineNum, callerName);
}



/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

unsigned long total_mallocs = 0;

unsigned long *membug_alloc (amount, fileName, lineNum)
unsigned long amount;
char  *fileName;
unsigned long lineNum;
{
	register pointer  ptr;

	if(!amount)
		return (unsigned long *)NULL;

#ifdef DEBUG
     fprintf(stderr,"Xalloc : FileName= %s, LineNum=%d, amount=%d\t\t",fileName,lineNum,amount);
#endif
	mem_bug_alloc_count++;

        if (membug_mallocs) {
            total_mallocs++;
            if (total_mallocs == membug_mallocs) {
                print_memory();
                total_mallocs = 0;
            }
        }

	/* aligned extra on long word boundary */
	amount = (amount + 7) & ~7;

	if (MemoryValidate)
		ValidateAllActiveMemory (fileName, lineNum, "xalloc");

	if (ptr = (pointer)malloc(request(amount))) {
#ifdef DEBUG
	fprintf(stderr,"ptr=x%x\n",(char *)ptr);
#endif
		/* fill in ALLOCFILL, helps in tracking uninitialised memory */
		{ 
			int c = request(amount);
			pointer tmptr = ptr;
			for(;c>0;c-=4) {
 			  *tmptr++ = ALLOCFILL;
 			  *tmptr++ = ALLOCFILL;
 			  *tmptr++ = ALLOCFILL;
 			  *tmptr++ = ALLOCFILL;
			}
		}
		return SetupBlock (ptr, amount, fileName, lineNum);
	}

	if (Must_have_memory)
		FatalError("Out of memory");

	return (unsigned long *)NULL;
}




/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

unsigned long *membug_calloc (amount, fileName, lineNum)
unsigned long amount;
char  *fileName;
unsigned long lineNum;
{
	unsigned long   *ret;
#ifdef DEBUG
  fprintf(stderr,"calloc : FileName=%s, LineNum = %d, Amount = %d\t", fileName,lineNum, amount);
#endif
	ret = membug_alloc (amount, fileName, lineNum);
	if (ret) bzero ((char *) ret, (int) amount);
#ifdef DEBUG
fprintf(stderr,"ptr = x%x\n", ret);
#endif
	return ret;
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

unsigned long *membug_realloc (ptr, amount, fileName, lineNum)
pointer ptr;
unsigned long   amount;
char    *fileName;
unsigned long   lineNum;
{
	int c;
	pointer tmptr;
	int prev_amount;
	pointer mptr;
	MallocHeaderPtr	head;

	mem_bug_realloc_count++;

#ifdef DEBUG
fprintf(stderr,"Realloc : FileName=%s, LineNum = %d, Amount = %d\t",fileName,lineNum, amount);
#endif

	if (MemoryValidate)
		ValidateAllActiveMemory (fileName, lineNum, "xrealloc");

	if (! amount)
	{
		membug_free(ptr, fileName, lineNum);
		return (unsigned long *)NULL;
	}
	amount = (amount + 7) & ~7;
	head = Header(ptr);

	if (ptr)
	{
		/* simulate realloc with malloc, bcopy and free */
		if (!same_magic(head->magic,FIRSTMAGIC))
			MemError("membug_realloc : Bad header", 
				  ptr, fileName, lineNum, "membug_realloc");
		if (!same_magic(Trailer(ptr)->magic,SECONDMAGIC))
			MemError("membug_realloc : Bad trailer", 
				  ptr, fileName, lineNum, "membug_realloc");

        	head = Header(ptr);
#ifdef DEBUG
fprintf(stderr," oldPtr = x%x\t\t",head);
#endif
	 	prev_amount = request(head->amount);
		ReleaseBlock(ptr, fileName, lineNum, "xrealloc");
		if(prev_amount > request(amount)) {
		    /* pad the exceeding mem with the FREEFILL */
			c = prev_amount - request(amount);
                        tmptr = (char*)head;
			tmptr += request(amount);
  			for(;c>0;c-=4) {
				*tmptr++ = FREEFILL;
				*tmptr++ = FREEFILL;
				*tmptr++ = FREEFILL;
				*tmptr++ = FREEFILL;
			}
                }
		
		ptr = (pointer)realloc((char *)head, request(amount));
#ifdef DEBUG
fprintf(stderr,"newPtr = x%x\n",ptr);
#endif

		if(ptr) {
        	   if(prev_amount < request(amount)) {  
			c = request(amount) - prev_amount +
			   	sizeof(MallocTrailerRec);
 			tmptr = (ptr + prev_amount - sizeof(MallocTrailerRec));
  			for(;c>0;c-=4) {
				*tmptr++ = ALLOCFILL;
				*tmptr++ = ALLOCFILL;
				*tmptr++ = ALLOCFILL;
				*tmptr++ = ALLOCFILL;
			}
		   }
		   return SetupBlock (ptr, amount, fileName, lineNum);
		}
	}
	else {
		ptr = (pointer)malloc(request(amount));
#ifdef DEBUG
fprintf(stderr,"newPtr = x%x\n", ptr);
#endif

		if(ptr)
		{ 
			c = request(amount);
			tmptr = ptr;
			for(;c>0;c-=4) {
				*tmptr++ = ALLOCFILL;
				*tmptr++ = ALLOCFILL;
				*tmptr++ = ALLOCFILL;
				*tmptr++ = ALLOCFILL;
			}
			return SetupBlock (ptr, amount, fileName, lineNum);
		}
	}

	if (Must_have_memory)
		FatalError("Out of memory");

	return (unsigned long *)NULL;
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

membug_free(ptr, fileName, lineNum)
pointer ptr;
char    *fileName;
unsigned long   lineNum;
{
	int c ;
	pointer tmptr;
	mem_bug_free_count++;
	if (MemoryValidate)
		ValidateAllActiveMemory (fileName, lineNum, "xfree");

	if (ptr)
	{
		MallocHeaderPtr	head;
		head = Header(ptr);

                if (!same_magic(head->magic,FIRSTMAGIC))
                        MemError("membug_free : Bad header",
                                  ptr, fileName, lineNum, "membug_free");
                if (!same_magic(Trailer(ptr)->magic,SECONDMAGIC))
                        MemError("membug_free : Bad trailer",
                                  ptr, fileName, lineNum, "membug_free");


		ReleaseBlock(ptr, fileName, lineNum, "xfree");

		set_magic(head->magic, FREEDMAGIC);
		head->fileName = fileName;
		head->lineNum  = lineNum;
		{ 
			c = head->amount;
			tmptr = ptr;
			for(;c>0;c-=4) {
				*tmptr++ = FREEFILL;
				*tmptr++ = FREEFILL;
				*tmptr++ = FREEFILL;
				*tmptr++ = FREEFILL;
			}
		}
#ifdef DEBUG
fprintf(stderr,"XFree : Filename : %s, LineNum : %d,\t\t\t ptr=x%x\n",
						fileName,lineNum,(char *)head);
#endif

		free ((char *) head);
	}
}



/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

static void ReleaseBlock(ptr, fileName, lineNum, callerName)
pointer ptr;
char    *fileName;
unsigned long   lineNum;
{
	MallocHeaderPtr	head = Header(ptr);

	ValidateNode(ptr, fileName, lineNum, callerName);

	if(head->prev)
		head->prev->next = head->next;
	else
		MemoryInUse = head->next;

	if (head->next)
		head->next->prev = head->prev;

	MemoryActive -= head->amount;
}

/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

void DumpAllMemoryInUse()
{
	MallocHeaderPtr	head;

	ErrorF("--------------------- Memory Dump ----------------------------\n");
	ErrorF("Statistics: malloc=%d, realloc=%d, free=%d, mem_active=0x%x\n\n",
	    mem_bug_free_count, mem_bug_alloc_count,
	    mem_bug_realloc_count, MemoryActive);

	for (head = MemoryInUse; head; head = head->next)
	{
		ErrorF("0x%08x %5d %6d %12s %5d\n",
		    head, head->amount, head->time, head->fileName, head->lineNum);
	}
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

void DumpMemoryInUse()
{
	MallocHeaderPtr	head;

	fprintf(stderr,"--------------------- Memory Dump ----------------------------\n");
	fprintf(stderr,"Statistics: malloc=%d, realloc=%d, free=%d, mem_active=0x%x\n\n",
	    mem_bug_free_count, mem_bug_alloc_count,
	    mem_bug_realloc_count, MemoryActive);

	for (head = MemoryInUse; head; head = head->next)
	{
			fprintf(stderr,"0x%x %5d %6d %12s %5d\n",
			    (char *)head, head->amount, head->time, head->fileName, head->lineNum);
	}
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

static unsigned long	MarkedTime;

void MarkMemoryTime ()
{
	MarkedTime = MemoryAllocTime;
}


/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

void DumpMemorySince ()
{
	DumpMemoryInUse (MarkedTime);
}



/*****************************************************************************
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*                                                                            *
*****************************************************************************/

#define UNKNOWN "unknown"

#ifdef NOT_USED
unsigned long *Xalloc(amount)
unsigned long amount;
{
	return(membug_alloc(amount, UNKNOWN, 0));
}

unsigned long *Xcalloc (amount)
unsigned long   amount;
{
	return(membug_calloc(amount, UNKNOWN, 0));
}

unsigned long *Xrealloc (ptr, amount)
register pointer ptr;
unsigned long amount;
{
	return(membug_realloc(ptr,amount, UNKNOWN, 0));
}

void Xfree(ptr)
register pointer ptr;
{
	membug_free(ptr, UNKNOWN, 0);
}
#endif

#else

unsigned long *membug_alloc (amount, fileName, lineNum)
unsigned long amount;
char  *fileName;
unsigned long lineNum;
{
  return Xalloc(amount);
}

unsigned long *membug_realloc (ptr, amount, fileName, lineNum)
pointer ptr;
unsigned long   amount;
char    *fileName;
unsigned long   lineNum;
{
 return Xrealloc(ptr,amount);
}

membug_free(ptr, fileName, lineNum)
pointer ptr;
char    *fileName;
unsigned long   lineNum;
{
  
  Xfree(ptr);
}
#endif

