/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 *
 * SPECIAL PROGRAM TO READ CONFIGURATION ONLY
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include "xvideo.h"
#include "plxv.h"

static char *namestr[] = {
	"not_used",		/* 00 */
	"not_used",		/* 01 */
	"not_used",		/* 02 */
	"not_used",		/* 03 */
	"Sbus 66hz",		/* 04 */
	"Turbo 1280",		/* 05 */
	"Sbus 76hz",		/* 06 */
	"Sbus 60hz",		/* 07 */
	"Sbus P/M Video",	/* 08 */
	"Sbus PURVIS",		/* 09 */
	"VIO (ntsc/pal)",	/* 10 */
	"RGB",			/* 11 */
	"SONYRGB",		/* 12 */
	"SONYRGB2",		/* 13 */
	"not_used",		/* 14 */
	"not_used",		/* 15 */
	"not_used"		/* 16 */
	};


/*******************************************/
/* ALL OF THE GLOBAL POINTERS DEFINED HERE */
/* XVSET DOES NOT LINK TO THE LIBRARY      */
int *xv_regbase,*xv_m1base,*xv_m24base,*xv_c3base,*xv_eprombase;
char *xv_m8base;
XV_write_regs *xv_wreg;
XV_read_regs *xv_rreg;
XV_shdw_regs *xv_shdw;
XV_struct *xv_cp;
XV_addr_maps *xv_ap;
/******************************************/

static XV_knob_rec    Dmsd_1   = {128,128,128,128};    /* initial video knobs */
static char delim_1[] = {" ,:"};
static char delim_2[] = {":"};

static int edit_addr = 0;
static int max_addr;
static char *rp;
static int fd;

typedef struct {
        int *address;
        int len;
        }PLX_MAPS;

PLX_MAPS map_array[3];


main(argc,argv)
int argc;
char *argv[];
{
    int i,j,k;
    int init_flag = 0;
    int write_flag = 0;
    int opt_flag = 0;
    int edit_flag = 0;
    int read_flag = 0;
    int bd_code = 0;
    int compare_flag = 0;
    int serial_number = 0;
    int serial_flag = 0;
    int override_flag = 0;
    int configuration_flag = 0;
    char *device = "/dev/fbs/tvtwo0";
    char *ap;

    if (sizeof(XV_controller_nvram) > 0x100 )
    {
	printf("ERROR: CONTROLLER NVRAM SPACE EXCEEDED\n");
    }
    if (sizeof(XV_option_nvram) > 0x100 )
    {
	printf("ERROR: OPTION NVRAM SPACE EXCEEDED\n");
    }

    if(argc > 2)
    {
	printf("usage: %s [device]\n");
	exit(1);
    }
    if(argc == 2)
	device = argv[1];   

    if((fd = open(device, O_RDWR)) < 0)
    {
        perror(device);
        exit(1);
    }

    xv_cp = (XV_struct *) plx_map_framebuffer(fd, &map_array);

    print_config();
    exit(0);
}
	

print_config()
{
	int i;
	load_from_bd(0);
	load_from_bd(1);
	printf("Main board:  %s         serial number: %d\n",
		namestr[xv_cp->main.board_code],xv_cp->main.serial_number);
	printf("             power cycles: %d  hours of operation: %d\n",
		xv_cp->main.power_cycles,xv_cp->main.power_hours);
	printf("             CCube is ");
	 if ((i =xv_cp->main.c3_rev)) printf("rev %d\n",i);
	    else printf("not installed\n");
	printf("             Hardware library compatability code: %x\n",
	    xv_cp->main.hlib_rev);
	
	if (xv_cp->opt.board_code)
	{
	    printf("\n\nOption  board:  %s  serial number: %d  board rev: %d\n",
		namestr[xv_cp->opt.board_code],
		xv_cp->opt.serial_number,
		xv_cp->opt.board_rev);
	    printf("                  power cycles: %d  hours operation: %d\n",
		xv_cp->opt.power_cycles,xv_cp->opt.power_hours);
	    printf("                  Hardware library compatability code: %x\n"
		,xv_cp->main.hlib_rev);
	} else
	{
	    printf("\n\nNo option board installed\n");
	}
}

compare_data(bd_code,opt_flag)
int bd_code,opt_flag;
{
	int i,size;
	unsigned char *cp1,*cp2;
        XV_struct ref;

	plxv_init_ram(&ref,bd_code,opt_flag);
	if (opt_flag)
	{
	    size = sizeof(XV_option_nvram);
	    cp1 = (unsigned char *)&(ref.opt);
	    cp2 = (unsigned char *)&(xv_cp->opt);
	} else
	{
	    size = sizeof(XV_controller_nvram);
	    cp1 = (unsigned char *)&(ref.main);
	    cp2 = (unsigned char *)&(xv_cp->main);
	}
	for (i=0;i<size;i++)
	{
	    if (*cp1 != *cp2)
		printf("addr: %x  ref: %x  board: %x\n",i,*cp1,*cp2);
	    cp1++; cp2++;
	}
}

load_from_bd(opt)
int opt;
{
    if (opt == 0)
    {
	if (plxv_nv_read(0,&(xv_cp->main),0,sizeof(XV_controller_nvram)))
	{
	    printf("main board nvram not responding\n");
	    return(-1);
	}
    } else
    {
	if (plxv_nv_read(1,&(xv_cp->opt),0,sizeof(XV_option_nvram)))
	{
	    xv_cp->opt.board_code = 0;
	    return(-1);
	}
    }
    return(0);
}

display_ram(opt)
int opt;
{
    unsigned int   i,j,*ip;
    unsigned short *sp;
    unsigned char  *cp;
    if (opt)
    {
	ip = (unsigned int *) &(xv_cp->opt);
	j = (sizeof(XV_option_nvram))/(sizeof(int));
    } else
    {
	ip = (unsigned int *) &(xv_cp->main);
	j = (sizeof(XV_controller_nvram))/(sizeof(int));
    }
    sp = (unsigned short *) ip;
    cp = (unsigned char *) ip;

    for (i=0;i<j;i++)
    {
	printf("%2x: %8x { %4x %4x } [",i*4,*ip++,*sp,*(sp+1));
	printf("%2x %2x %2x %2x]\n",*cp,*(cp+1),*(cp+2),*(cp+3));
	cp += 4; sp +=2;
    }
}

edit_ram(opt)
int opt;
{
    char inline[132],token,*cp1,*cp2,*cp3;
    int i,j,k;
    int go = 1;
    int more = 1;

    if (opt)
    {
	rp = (char *)&(xv_cp->opt);
	max_addr = sizeof(XV_option_nvram);
    } else
    {
	rp = (char *)&(xv_cp->main);
	max_addr = sizeof(XV_controller_nvram);
    }


    printf("Edit mode: [addr:] data [data [data]]  (cr to get out)\n");
    while (go)
    {
        printf("\nEDIT: ");
	gets(inline);
        if ( inline[0] == '\0') go = 0;

        cp2 = strstr(inline,delim_2);
        cp1 = strtok(inline,delim_1);
        more = 1;
        while ( (cp1 != NULL)  && (more != 0))
        {   /* more data is available */
            i = sscanf(cp1,"%x",&k);
            if (i == 0)
            {
		if (strstr(cp1,"p") == 0)
		{
		    display_ram(opt);
		    cp1 = strtok(NULL,delim_1);
		} else
		{
                    printf("bad scan for hex %s\n",cp1);
                    more = 0;
		}
            } else
            {	/* good scan, now see if it was an address spec */
                if (cp2 != NULL) cp3 = strstr(cp2+1,delim_2); else cp3 = NULL;
                cp1 = strtok(NULL,delim_1);
                if (((unsigned int)cp1 > (unsigned int)cp2) && (cp2 != NULL))
                {
                    set_addr(k);
                    cp2 = cp3;
                } else if ((cp1 == NULL) && (cp2 !=NULL))
                {
                    set_addr(k);
                    cp2 = cp3;
                } else
                {
                    set_data(k);
                }
            }
        }
    }
}

static set_addr(k)
int k;
{
	edit_addr = k;
}
static set_data(k)
int k;
{
    if (edit_addr >= max_addr)
    {
        printf("address range exceeded, write not performed\n");
        return(-1);
    }
    *(rp + edit_addr) = k;
    edit_addr++;
}

void ErrorF(t)
char *t;
{
    fprintf(stderr, t);
    exit(1);
}

#if 0
XV_struct *plxv_init_context(reg,m1,m8,m24,c3,eprom)
int *reg,*m1,*m24,*c3,*eprom;
char *m8;
{
    int Xres,Yres,i,*ip,j;
    XV_struct      *cp;
    XV_shdw_regs   *sp;
    XV_addr_maps   *ap;

    if ((cp = (XV_struct *)malloc(sizeof(XV_struct))) == NULL)
    {   /* malloc failed */
	printf("malloc failed on xvset_init_struct:main struct\n");
	exit(-1);
    }
    if ((sp = (XV_shdw_regs *)malloc(sizeof(XV_shdw_regs))) == NULL)
    {   /* malloc failed */
	printf("malloc failed on xvset_init_struct:shadow\n");
	exit(-1);
    }

    /* fill in the structure */
    cp->regbase = reg;
    cp->shadow = (int *)sp;
	
    /* init the REGISTER FILE shadow ram structure */
    j = sizeof(XV_shdw_regs) >> 2;
    for (ip = (int *)sp, i = 0; i++; i<j) *ip++ = 0;

    /* move all the address pointers into the global set */
    /* (this is doing the actual context switch */
    xv_cp = cp;
    xv_regbase = reg;
    xv_wreg = (XV_write_regs *)reg;
    xv_rreg = (XV_read_regs *)reg;
    xv_shdw = sp;

    plxv_load_struct(cp);

    return(cp);
}

junkprint()
{
}
#endif
