
#include <stdio.h>
#include "sun.h"
#include "mpg/mpg.h"
#include "mpg/wid.h"
#include "X.h"
#include "Xmd.h"
#include "Xproto.h"
#define PSZ 32
#include "cfb.h"
#include "fontstruct.h"
#include "dixfontstr.h"
#include "gcstruct.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "scrnintstr.h"
#include "region.h"

#include "mistruct.h"
#include "mibstore.h"

#include "cfbmskbits.h"
#include "cfb8bit.h"
#include "plxInit.h"

void plx32ValidateGC();
extern int cfb32GCPrivateIndex;

void cfbChangeGC(), cfbCopyGC(), plx32DestroyGC();
void cfbChangeClip(), cfbDestroyClip(), cfbCopyClip();

#if PSZ == 8
# define useTEGlyphBlt  cfbTEGlyphBlt8
#else
# ifdef WriteFourBits
#  define useTEGlyphBlt	cfbImageGlyphBlt8
# else
#  define useTEGlyphBlt	cfbTEGlyphBlt
# endif
#endif

#ifdef WriteFourBits
# define useImageGlyphBlt	cfbImageGlyphBlt8
# define usePolyGlyphBlt	cfbPolyGlyphBlt8
#else
# define useImageGlyphBlt	miImageGlyphBlt
# define usePolyGlyphBlt	miPolyGlyphBlt
#endif

#ifdef FOUR_BIT_CODE
# define usePushPixels	cfbPushPixels8
#else
# define usePushPixels	mfbPushPixels
#endif

#ifdef PIXEL_ADDR
# define ZeroPolyArc	cfbZeroPolyArcSS8Copy
#else
# define ZeroPolyArc	miZeroPolyArc
#endif

extern int plxroptbl[];

GCFuncs plx32GCFuncs = {
    plx32ValidateGC,
    cfbChangeGC,
    cfbCopyGC,
    plx32DestroyGC,
    cfbChangeClip,
    cfbDestroyClip,
    cfbCopyClip,
};

extern GCOps *cfbCreateOps ();
extern void pfbPolylines();
extern void pfbPolySegment();

extern void	    PlxPolyFillRect32();

static void	    Plx32FillSpans(),	Plx32SetSpans(),	   Plx32PutImage();
static RegionPtr    Plx32CopyArea(),	Plx32CopyPlane();
static void	    Plx32PolyPoint(),	Plx32Polylines(),    Plx32PolySegment();

static void	    Plx32PolyRectangle(),	Plx32PolyArc(),	   Plx32FillPolygon();
static void	    Plx32PolyFillArc(), Plx32PolyFillRect();
static int	    Plx32PolyText8(),	Plx32PolyText16();
static void	    Plx32ImageText8(),	Plx32ImageText16();
static void	    Plx32ImageGlyphBlt(),	Plx32PolyGlyphBlt();
static void	    Plx32PushPixels(),	Plx32LineHelper();

#ifdef OLDGLYPHBLT
static GCOps plx32TEOps = {
    Plx32FillSpans,	Plx32SetSpans,		Plx32PutImage,	
    Plx32CopyArea,	Plx32CopyPlane,  	Plx32PolyPoint,
    Plx32Polylines,	Plx32PolySegment,	Plx32PolyRectangle,
    Plx32PolyArc,	Plx32FillPolygon,	Plx32PolyFillRect,
    Plx32PolyFillArc,	Plx32PolyText8,  	Plx32PolyText16,
    Plx32ImageText8,	Plx32ImageText16,	Plx32ImageGlyphBlt,
    Plx32PolyGlyphBlt,	Plx32PushPixels, 	Plx32LineHelper,
};
static GCOps plx32NonTEOps = {
    Plx32FillSpans,	Plx32SetSpans,		Plx32PutImage,	
    Plx32CopyArea,	Plx32CopyPlane,  	Plx32PolyPoint,
    Plx32Polylines,	Plx32PolySegment,	Plx32PolyRectangle,
    Plx32PolyArc,	Plx32FillPolygon,	Plx32PolyFillRect,
    Plx32PolyFillArc,	Plx32PolyText8,  	Plx32PolyText16,
    Plx32ImageText8,	Plx32ImageText16,	Plx32ImageGlyphBlt,
    Plx32PolyGlyphBlt,	Plx32PushPixels, 	Plx32LineHelper,
};
#else
static GCOps plx32TEOps = {
    Plx32FillSpans,	Plx32SetSpans,		Plx32PutImage,	
    Plx32CopyArea,	Plx32CopyPlane,  	Plx32PolyPoint,
    Plx32Polylines,	Plx32PolySegment,		Plx32PolyRectangle,
    Plx32PolyArc,	Plx32FillPolygon,	Plx32PolyFillRect,
    Plx32PolyFillArc,	miPolyText8,	  	miPolyText16,
    miImageText8,	miImageText16,		Plx32ImageGlyphBlt,
    Plx32PolyGlyphBlt,	miPushPixels,	 	Plx32LineHelper,
};
static GCOps plx32NonTEOps = {
    Plx32FillSpans,	Plx32SetSpans,		Plx32PutImage,	
    Plx32CopyArea,	Plx32CopyPlane,  	Plx32PolyPoint,
    Plx32Polylines,	Plx32PolySegment,	Plx32PolyRectangle,
    Plx32PolyArc,	Plx32FillPolygon,	Plx32PolyFillRect,
    Plx32PolyFillArc,	miPolyText8,	  	miPolyText16,
    miImageText8,	miImageText16,		Plx32ImageGlyphBlt,
    Plx32PolyGlyphBlt,	miPushPixels,	 	Plx32LineHelper,
};
#endif


GCOps *plx32MatchCommon();


#define GET_PWID(w) (widGetValue(widGetWindowWid(((WindowPtr)(w))->parent)))

#define Plx32Private

Bool
plx32CreateGC(pGC)
    register GCPtr pGC;
{
plxGCPtr	privGC;
plxScreenPtr  pPlxPrivate =  GetPlxPrivate(pGC->pScreen);

    switch(pGC->depth)
    {
    case 24:
         if (!cfbCreateGC(pGC)) return FALSE;
         privGC = (plxGCPtr)xalloc(sizeof(plxGCRec));
         if (!privGC)
           return FALSE;
	 privGC->rop=5;
	 privGC->bg=0;
	 privGC->wid_value=0;
	 privGC->fg=0xffffff;
	 privGC->linebytes=1152;
	 privGC->planemask=0xffffff;
	 privGC->win_type=0;
	 privGC->bpp=24;
	 privGC->video_priv=NULL;
         pGC->devPrivates[plxGCPrivateIndex].ptr= (pointer)privGC;
	 if(pPlxPrivate->privGC[PLX_COLOR_24] == NULL)
	     pPlxPrivate->privGC[PLX_COLOR_24]= privGC;
         pGC->ops = &plx32NonTEOps;
         pGC->funcs = &plx32GCFuncs;
	 break;
    case 1:
    case 8:
	    FatalError("PlxGc32: illegal depth\n");
	 break;
    }
	
    return TRUE;
}

void
plx32DestroyGC(pGC)
    GC 			*pGC;
{
cfbPrivGC *pPriv;
plxGCPtr	privGC = GetPlxGCPrivate(pGC);

    if(privGC)
    {
        if(privGC->video_priv!=NULL)
	    xfree(privGC->video_priv);
        xfree(privGC);
    }

    pPriv = (cfbPrivGC *)(pGC->devPrivates[cfbGCPrivateIndex].ptr);
    if (pPriv->pRotatedPixmap)
	cfbDestroyPixmap(pPriv->pRotatedPixmap);
    if (pPriv->freeCompClip)
	(*pGC->pScreen->RegionDestroy)(pPriv->pCompositeClip);
    cfbDestroyOps (pGC->ops);
}

void
plx32ValidateGC(pGC, changes, pDrawable)
    register GCPtr  pGC;
    Mask	    changes;
    DrawablePtr	    pDrawable;
{
    WindowPtr   pWin;
    int         mask;		/* stateChanges */
    int         index;		/* used for stepping through bitfields */
    int		new_rrop;
    int         new_line, new_text, new_fillspans, new_fillarea;
    int		new_rotate;
    int		xrot, yrot;
    /* flags for changing the proc vector */
    cfbPrivGCPtr devPriv;
    int		oneRect;
    int		vid;
    int		pwid;
    plxGCPtr	privGC,plxPrivGC;
    plxScreenPtr  pPlxPrivate =  GetPlxPrivate(pGC->pScreen);

    privGC = GetPlxGCPrivate(pGC);
    plxPrivGC= pPlxPrivate->privGC[PLX_COLOR_24];

    new_rotate = pGC->lastWinOrg.x != pDrawable->x ||
		 pGC->lastWinOrg.y != pDrawable->y;

    pGC->lastWinOrg.x = pDrawable->x;
    pGC->lastWinOrg.y = pDrawable->y;
    if (pDrawable->type == DRAWABLE_WINDOW)
    {
	pWin = (WindowPtr) pDrawable;
	vid = WindowVisual(pWin);
	plxPrivGC->wid_value= privGC->wid_value= widWinGetValue(pWin);
    }
    else
    {
	pWin = (WindowPtr) NULL;
	vid = 0;
	pwid=0;
    }

    devPriv = ((cfbPrivGCPtr) (pGC->devPrivates[cfb32GCPrivateIndex].ptr));

    new_rrop = FALSE;
    new_line = FALSE;
    new_text = FALSE;
    new_fillspans = FALSE;
    new_fillarea = FALSE;

    /*
     * if the client clip is different or moved OR the subwindowMode has
     * changed OR the window's clip has changed since the last validation
     * we need to recompute the composite clip 
     */

    if ((changes & (GCClipXOrigin|GCClipYOrigin|GCClipMask|GCSubwindowMode)) ||
	(pDrawable->serialNumber != (pGC->serialNumber & DRAWABLE_SERIAL_BITS))
	)
    {
	ScreenPtr pScreen = pGC->pScreen;

	if (pWin) {
	    RegionPtr   pregWin;
	    Bool        freeTmpClip, freeCompClip;

	    pregWin = &pWin->clipList;

	    if (pGC->subWindowMode == IncludeInferiors) {
		pregWin = NotClippedByChildren(pWin);
		freeTmpClip = TRUE;
	    }
	    else {
		pregWin = &pWin->clipList;
		freeTmpClip = FALSE;
	    }
	    freeCompClip = devPriv->freeCompClip;

	    /*
	     * if there is no client clip, we can get by with just keeping
	     * the pointer we got, and remembering whether or not should
	     * destroy (or maybe re-use) it later.  this way, we avoid
	     * unnecessary copying of regions.  (this wins especially if
	     * many clients clip by children and have no client clip.) 
	     */
	    if (pGC->clientClipType == CT_NONE) {
		if (freeCompClip)
		    (*pScreen->RegionDestroy) (devPriv->pCompositeClip);
		devPriv->pCompositeClip = pregWin;
		devPriv->freeCompClip = freeTmpClip;
	    }
	    else {
		/*
		 * we need one 'real' region to put into the composite
		 * clip. if pregWin the current composite clip are real,
		 * we can get rid of one. if pregWin is real and the
		 * current composite clip isn't, use pregWin for the
		 * composite clip. if the current composite clip is real
		 * and pregWin isn't, use the current composite clip. if
		 * neither is real, create a new region. 
		 */

		(*pScreen->TranslateRegion)(pGC->clientClip,
					    pDrawable->x + pGC->clipOrg.x,
					    pDrawable->y + pGC->clipOrg.y);
						  
		if (freeCompClip)
		{
		    (*pGC->pScreen->Intersect)(devPriv->pCompositeClip,
					       pregWin, pGC->clientClip);
		    if (freeTmpClip)
			(*pScreen->RegionDestroy)(pregWin);
		}
		else if (freeTmpClip)
		{
		    (*pScreen->Intersect)(pregWin, pregWin, pGC->clientClip);
		    devPriv->pCompositeClip = pregWin;
		}
		else
		{
		    devPriv->pCompositeClip = (*pScreen->RegionCreate)(NullBox,
								       0);
		    (*pScreen->Intersect)(devPriv->pCompositeClip,
					  pregWin, pGC->clientClip);
		}
		devPriv->freeCompClip = TRUE;
		(*pScreen->TranslateRegion)(pGC->clientClip,
					    -(pDrawable->x + pGC->clipOrg.x),
					    -(pDrawable->y + pGC->clipOrg.y));
						  
	    }
	}			/* end of composite clip for a window */
	else {
	    BoxRec      pixbounds;

	    /* XXX should we translate by drawable.x/y here ? */
	    pixbounds.x1 = 0;
	    pixbounds.y1 = 0;
	    pixbounds.x2 = pDrawable->width;
	    pixbounds.y2 = pDrawable->height;

	    if (devPriv->freeCompClip)
		(*pScreen->RegionReset)(devPriv->pCompositeClip, &pixbounds);
	    else {
		devPriv->freeCompClip = TRUE;
		devPriv->pCompositeClip = (*pScreen->RegionCreate)(&pixbounds,
								   1);
	    }

	    if (pGC->clientClipType == CT_REGION)
	    {
		(*pScreen->TranslateRegion)(devPriv->pCompositeClip,
					    -pGC->clipOrg.x, -pGC->clipOrg.y);
		(*pScreen->Intersect)(devPriv->pCompositeClip,
				      devPriv->pCompositeClip,
				      pGC->clientClip);
		(*pScreen->TranslateRegion)(devPriv->pCompositeClip,
					    pGC->clipOrg.x, pGC->clipOrg.y);
	    }
	}			/* end of composute clip for pixmap */
	oneRect = REGION_NUM_RECTS(devPriv->pCompositeClip) == 1;
	if (oneRect != devPriv->oneRect)
	    new_line = TRUE;
	devPriv->oneRect = oneRect;
    }

    mask = changes;
    while (mask) {
	index = lowbit (mask);
	mask &= ~index;

	/*
	 * this switch acculmulates a list of which procedures might have
	 * to change due to changes in the GC.  in some cases (e.g.
	 * changing one 16 bit tile for another) we might not really need
	 * a change, but the code is being paranoid. this sort of batching
	 * wins if, for example, the alu and the font have been changed,
	 * or any other pair of items that both change the same thing. 
	 */
	switch (index) {
	case GCFunction:
    	    plxPrivGC->rop = privGC->rop= plxroptbl[pGC->alu];
	    new_rrop = TRUE;
	    break;
	case GCForeground:
	    plxPrivGC->fg = privGC->fg=pGC->fgPixel;
	    new_rrop = TRUE;
	    break;
	case GCPlaneMask:
	    new_rrop = TRUE;
	    new_text = TRUE;
	    plxPrivGC->planemask = privGC->planemask=pGC->planemask;
	    break;
	case GCBackground:
	    plxPrivGC->bg = privGC->bg=pGC->bgPixel;
	    break;
	case GCLineStyle:
	case GCLineWidth:
	    new_line = TRUE;
	    break;
	case GCJoinStyle:
	case GCCapStyle:
	    break;
	case GCFillStyle:
	    new_text = TRUE;
	    new_fillspans = TRUE;
	    new_line = TRUE;
	    new_fillarea = TRUE;
	    break;
	case GCFillRule:
	    break;
	case GCTile:
	    new_fillspans = TRUE;
	    new_fillarea = TRUE;
	    break;

	case GCStipple:
	    if (pGC->stipple)
	    {
		int width = pGC->stipple->drawable.width;
		PixmapPtr nstipple;

		if ((width <= 32) && !(width & (width - 1)) &&
		    (nstipple = cfbCopyPixmap(pGC->stipple)))
		{
		    cfbPadPixmap(nstipple);
		    cfbDestroyPixmap(pGC->stipple);
		    pGC->stipple = nstipple;
		}
	    }
	    new_fillspans = TRUE;
	    new_fillarea = TRUE;
	    break;

	case GCTileStipXOrigin:
	    new_rotate = TRUE;
	    break;

	case GCTileStipYOrigin:
	    new_rotate = TRUE;
	    break;

	case GCFont:
	    new_text = TRUE;
	    break;
	case GCSubwindowMode:
	    break;
	case GCGraphicsExposures:
	    break;
	case GCClipXOrigin:
	    break;
	case GCClipYOrigin:
	    break;
	case GCClipMask:
	    break;
	case GCDashOffset:
	    break;
	case GCDashList:
	    break;
	case GCArcMode:
	    break;
	default:
	    break;
	}
    }
    /*
     * If the drawable has changed,  ensure suitable
     * entries are in the proc vector. 
     *
     */
    if (pDrawable->serialNumber != (pGC->serialNumber & (DRAWABLE_SERIAL_BITS))) {
	new_fillspans = TRUE;	/* deal with FillSpans later */
    }

    if (new_rotate || new_fillspans)
    {
	Bool new_pix = FALSE;

	xrot = pGC->patOrg.x + pDrawable->x;
	yrot = pGC->patOrg.y + pDrawable->y;

	switch (pGC->fillStyle)
	{
	case FillTiled:
	    if (!pGC->tileIsPixel)
	    {
		int width = pGC->tile.pixmap->drawable.width * PSZ;

		if ((width <= 32) && !(width & (width - 1)))
		{
		    cfbCopyRotatePixmap(pGC->tile.pixmap,
					&devPriv->pRotatedPixmap,
					xrot, yrot);
		    new_pix = TRUE;
		}
	    }
	    break;
#ifdef FOUR_BIT_CODE
	case FillStippled:
	case FillOpaqueStippled:
	    {
		int width = pGC->stipple->drawable.width;

		if ((width <= 32) && !(width & (width - 1)))
		{
		    mfbCopyRotatePixmap(pGC->stipple,
					&devPriv->pRotatedPixmap, xrot, yrot);
		    new_pix = TRUE;
		}
	    }
	    break;
#endif
	}
	if (!new_pix && devPriv->pRotatedPixmap)
	{
	    cfbDestroyPixmap(devPriv->pRotatedPixmap);
	    devPriv->pRotatedPixmap = (PixmapPtr) NULL;
	}
    }

    if (new_rrop)
    {
	int old_rrop;

	old_rrop = devPriv->rop;
	devPriv->rop = cfbReduceRasterOp (pGC->alu, pGC->fgPixel,
					   pGC->planemask,
					   &devPriv->and, &devPriv->xor);
	if (old_rrop == devPriv->rop)
	    new_rrop = FALSE;
	else
	{
#ifdef PIXEL_ADDR
	    new_line = TRUE;
#endif
#ifdef WriteFourBits
	    new_text = TRUE;
#endif
	    new_fillspans = TRUE;
	    new_fillarea = TRUE;
	}
    }

    if (new_rrop || new_fillspans || new_text || new_fillarea || new_line)
    {
	GCOps	*newops;

	if (newops = plx32MatchCommon (pGC, devPriv))
 	{
	    if (pGC->ops->devPrivate.val)
		cfbDestroyOps (pGC->ops);
	    pGC->ops = newops;
	    new_rrop = new_line = new_fillspans = new_text = new_fillarea = 0;
	}
 	else
 	{
	    if (!pGC->ops->devPrivate.val)
	    {
		pGC->ops = cfbCreateOps (pGC->ops);
		pGC->ops->devPrivate.val = 1;
	    }
	}
    }

    /* deal with the changes we've collected */
    if (new_line)
    {
	pGC->ops->FillPolygon = miFillPolygon;
	if (devPriv->oneRect && pGC->fillStyle == FillSolid)
	{
	       pGC->ops->FillPolygon = Plx32FillPolygon;
	}
	if (pGC->lineWidth == 0)
	{
#ifdef PIXEL_ADDR
	    if ((pGC->lineStyle == LineSolid) && (pGC->fillStyle == FillSolid))
	    {
		switch (devPriv->rop)
		{
		case GXxor:
		    pGC->ops->PolyArc = cfbZeroPolyArcSS8Xor;
		    break;
		case GXcopy:
		    pGC->ops->PolyArc = cfbZeroPolyArcSS8Copy;
		    break;
		default:
		    pGC->ops->PolyArc = cfbZeroPolyArcSS8General;
		    break;
		}
	    }
	    else
#endif
		pGC->ops->PolyArc = miZeroPolyArc;
	}
	else
	    pGC->ops->PolyArc = miPolyArc;

	pGC->ops->PolySegment = Plx32PolySegment;
#ifdef notnow
	switch (pGC->lineStyle)
	{
	case LineSolid:
	    if(pGC->lineWidth == 0)
	    {
		if (pGC->fillStyle == FillSolid)
		{
#ifdef PIXEL_ADDR
		    if (devPriv->oneRect)
		    {
			pGC->ops->Polylines = cfb32LineSS1Rect;
			pGC->ops->PolySegment = cfb32SegmentSS1Rect;
		    } else
		    {
		    	pGC->ops->Polylines = cfbLineSS;
		    	pGC->ops->PolySegment = cfbSegmentSS;
		    }
		}
 		else
		    pGC->ops->Polylines = miZeroLine;
	    }
	    else
		pGC->ops->Polylines = miWideLine;
#endif
	    break;
	case LineOnOffDash:
	case LineDoubleDash:
	    if (pGC->lineWidth == 0 && pGC->fillStyle == FillSolid)
	    {
		pGC->ops->Polylines = cfbLineSD;
		pGC->ops->PolySegment = cfbSegmentSD;
	    } else
		pGC->ops->Polylines = miWideDash;
	    break;
	}
#endif
    }

    if (new_text && (pGC->font))
    {
        if (FONTMAXBOUNDS(pGC->font,rightSideBearing) -
            FONTMINBOUNDS(pGC->font,leftSideBearing) > 32 ||
	    FONTMINBOUNDS(pGC->font,characterWidth) < 0)
        {
#ifdef OLDGLYPHBLT
            pGC->ops->PolyGlyphBlt = Plx32PolyGlyphBlt;
            pGC->ops->ImageGlyphBlt = Plx32ImageGlyphBlt;
#else
            pGC->ops->PolyGlyphBlt = miPolyGlyphBlt;
            pGC->ops->ImageGlyphBlt = miImageGlyphBlt;
#endif
        }
        else
        {
#ifdef WriteFourBits
	    if (pGC->fillStyle == FillSolid)
	    {
		if (devPriv->rop == GXcopy)
		    pGC->ops->PolyGlyphBlt = cfbPolyGlyphBlt8;
		else
#ifdef FOUR_BIT_CODE
		    pGC->ops->PolyGlyphBlt = cfbPolyGlyphRop8;
#else
		    pGC->ops->PolyGlyphBlt = Plx32PolyGlyphBlt;
#endif
	    }
	    else
#endif
#ifdef OLDGLYPHBLT
		pGC->ops->PolyGlyphBlt = Plx32PolyGlyphBlt;
#else
		pGC->ops->PolyGlyphBlt = miPolyGlyphBlt;
#endif
            /* special case ImageGlyphBlt for terminal emulator fonts */
#if !defined(WriteFourBits) || PSZ == 8
	    if (TERMINALFONT(pGC->font) &&
		(pGC->planemask & PMSK) == PMSK
#ifdef FOUR_BIT_CODE
		&& FONTMAXBOUNDS(pGC->font,characterWidth) >= 4
#endif
		)
	    {
		pGC->ops->ImageGlyphBlt = useTEGlyphBlt;
	    }
            else
#endif
	    {
#ifdef WriteFourBits
		if (devPriv->rop == GXcopy &&
		    pGC->fillStyle == FillSolid &&
		    (pGC->planemask & PMSK) == PMSK)
		    pGC->ops->ImageGlyphBlt = Plx32ImageGlyphBlt;
		else
#endif
		    pGC->ops->ImageGlyphBlt = miImageGlyphBlt;
	    }
        }
    }    


    if (new_fillspans) {
	switch (pGC->fillStyle) {
	case FillSolid:
		pGC->ops->FillSpans = Plx32FillSpans;
	    break;
	case FillTiled:
	    if (devPriv->pRotatedPixmap)
	    {
		if (pGC->alu == GXcopy && (pGC->planemask & PMSK) == PMSK)
		    pGC->ops->FillSpans = cfbTile32FSCopy;
		else
		    pGC->ops->FillSpans = cfbTile32FSGeneral;
	    }
	    else
		pGC->ops->FillSpans = cfbUnnaturalTileFS;
	    break;
	case FillStippled:
#ifdef FOUR_BIT_CODE
	    if (devPriv->pRotatedPixmap)
		pGC->ops->FillSpans = cfb8Stipple32FS;
	    else
#endif
		pGC->ops->FillSpans = cfbUnnaturalStippleFS;
	    break;
	case FillOpaqueStippled:
#ifdef FOUR_BIT_CODE
	    if (devPriv->pRotatedPixmap)
		pGC->ops->FillSpans = cfb8OpaqueStipple32FS;
	    else
#endif
		pGC->ops->FillSpans = cfbUnnaturalStippleFS;
	    break;
	default:
	    FatalError("Plx32ValidateGC: illegal fillStyle\n");
	}
    } /* end of new_fillspans */

    if (new_fillarea) {
#ifndef FOUR_BIT_CODE
	pGC->ops->PolyFillRect = miPolyFillRect;

	if (pGC->fillStyle == FillSolid)
	       pGC->ops->PolyFillRect = Plx32PolyFillRect;
	else if (pGC->fillStyle == FillTiled)
	       pGC->ops->PolyFillRect = cfbPolyFillRect;
#endif
#ifdef FOUR_BIT_CODE
	pGC->ops->PushPixels = mfbPushPixels;
	if (pGC->fillStyle == FillSolid && devPriv->rop == GXcopy)
	    pGC->ops->PushPixels = Plx32PushPixels;
#endif
	pGC->ops->PolyFillArc = miPolyFillArc;
	if (pGC->fillStyle == FillSolid)
	{
	    switch (devPriv->rop)
	    {
	    case GXcopy:
		pGC->ops->PolyFillArc = cfbPolyFillArcSolidCopy;
		break;
	    default:
		pGC->ops->PolyFillArc = cfbPolyFillArcSolidGeneral;
		break;
	    }
	}
    }
}

extern RegionPtr cfbCopyPlane(),cfbCopyArea();
#ifdef SPECIAL_CFB
extern RegionPtr plxCopyArea32();
#endif

/****************************************************************************/
/*                             Plx32 GC OPS                                   */
/****************************************************************************/
static void
Plx32FillSpans(pDrawable, pGC, nInit, pptInit, pwidthInit, fSorted)
    DrawablePtr	pDrawable;
    GCPtr	pGC;
    int		nInit;		
    DDXPointPtr pptInit;
    int		*pwidthInit;
    int 	fSorted;
{
    cfbPrivGCPtr devPriv;
    plxGCPtr	privGC = GetPlxGCPrivate(pGC);
    plxScreenPtr  pPlxPrivate =  GetPlxPrivate(pGC->pScreen);
    devPriv = ((cfbPrivGCPtr) (pGC->devPrivates[cfb32GCPrivateIndex].ptr));
    if(privGC->rop == PLXV_ROP_INVERT || privGC->rop == PLXV_ROP_XOR)
    {
            plxv_rop(5);
	    plxv_color_24(privGC->planemask);
	    plxv_tag(PLXV_TAG_MASK,pPlxPrivate->wid24True);
	    plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
            plxv_wmask(PLXV_PIXEL_MASK);
    }
    else
        SetAttributes(pDrawable,pGC);
    switch (devPriv->rop) {
    case GXcopy:
	cfbSolidSpansCopy(pDrawable, pGC, nInit, pptInit, pwidthInit, fSorted);
	break;
    case GXxor:
	cfbSolidSpansXor(pDrawable, pGC, nInit, pptInit, pwidthInit, fSorted);
	break;
    default:
	cfbSolidSpansGeneral (pDrawable, pGC, nInit, pptInit, pwidthInit, fSorted);
	break;
    }
}

static void
Plx32SetSpans(pDrawable, pGC, psrc, ppt, pwidth, nspans, fSorted)
    DrawablePtr	pDrawable;
    GCPtr	pGC;
    int		*psrc;
    DDXPointPtr ppt;
    int		*pwidth;
    int		nspans;
    int		fSorted;
{
    cfbPrivGCPtr devPriv;
    SetAttributes(pDrawable,pGC);
      cfbSetSpans(pDrawable, pGC, psrc, ppt, pwidth, nspans, fSorted);

}

static void
Plx32PutImage(pDrawable, pGC, depth, x, y, w, h, leftPad, format, pBits)
    DrawablePtr	pDrawable;
    GCPtr   	pGC;
    int		depth;
    int	    	x;
    int	    	y;
    int	    	w;
    int	    	h;
    int	    	format;
    char    	*pBits;

{

    SetAttributes(pDrawable,pGC);
    cfbPutImage(pDrawable, pGC, depth, x, y, w, h, leftPad, format, pBits);
}
extern RegionPtr Plx32ScrnCopyArea();

static RegionPtr
Plx32CopyArea(pSrc, pDst, pGC, srcx, srcy, w, h, dstx, dsty)
    DrawablePtr	pSrc;
    DrawablePtr	pDst;
    GCPtr   	pGC;
    int	    	srcx;
    int	    	srcy;
    int	    	w;
    int	    	h;
    int	    	dstx;
    int	    	dsty;
{
    RegionPtr     rgn;

    plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
    plxv_tag(PLXV_TAG_MASK_DISABLE,PLXV_TAG_DISABLE);
    plxv_wmask(PLXV_PIXEL_MASK);
    plxv_rop(PLXV_ROP_COPY);
#ifndef NOTDEF
    if((pDst->type == DRAWABLE_WINDOW))
	SetAttributes(pDst, pGC);
#endif

    if(pSrc == pDst && (pDst->type == DRAWABLE_WINDOW))
        rgn= Plx32ScrnCopyArea(pSrc, pDst, pGC, srcx, srcy, w, h, dstx, dsty);
    else
#ifdef  SPECIAL_CFB
        rgn= plxCopyArea32(pSrc, pDst, pGC, srcx, srcy, w, h, dstx, dsty);
#else
        rgn= cfbCopyArea(pSrc, pDst, pGC, srcx, srcy, w, h, dstx, dsty);
#endif
    return(rgn);
}

static RegionPtr
Plx32CopyPlane(pSrc, pDst, pGC, srcx, srcy, w, h, dstx, dsty, plane)
    DrawablePtr		pSrc;
    DrawablePtr 	pDst;
    GCPtr  	 	pGC;
    int         	srcx;
    int         	srcy;
    int         	w;  
    int         	h;
    int         	dstx;
    int         	dsty;
    unsigned long	plane;
{
    RegionPtr	rgn;
    plxGCPtr	privGC = GetPlxGCPrivate(pGC);
    SetAttributes(pDst,pGC);
    plxv_tag(PLXV_TAG_MASK_DISABLE,PLXV_TAG_DISABLE);
#ifdef NOTDEF
    plxv_stencil(PLXV_TAG_ST_MASK,privGC->wid_value);
#else
    plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
#endif
    plxv_wmask(PLXV_PIXEL_MASK);
    rgn= cfbCopyPlane(pSrc, pDst, pGC, srcx, srcy, w, h, dstx, dsty, plane);
    return(rgn);
}

void
Plx32PolyFillRect(pDrawable, pGC, nrectFill, prectInit)
    DrawablePtr pDrawable;
    register GCPtr pGC;
    int		nrectFill; 	/* number of rectangles to fill */
    xRectangle	*prectInit;  	/* Pointer to first rectangle to fill */
{
    plxGCPtr	privGC = GetPlxGCPrivate(pGC);

         if (pGC->fillStyle == FillSolid)
		PlxPolyFillRect32(pDrawable, pGC, nrectFill, prectInit);
	 else
	 {
		SetAttributes(pDrawable,pGC);
		cfbPolyFillRect(pDrawable, pGC, nrectFill, prectInit);
	 }
}

static void
Plx32PolyPoint(pDrawable, pGC, mode, npt, pptInit)
    DrawablePtr pDrawable;
    GCPtr	pGC;
    int		mode;
    int		npt;
    xPoint	*pptInit;
{
    SetAttributes(pDrawable,pGC);
    miPolyPoint(pDrawable, pGC, mode, npt, pptInit);
}

static void
Plx32Polylines(pDrawable, pGC, mode, npt, pptInit)
    DrawablePtr	pDrawable;
    GCPtr   	pGC;
    int	    	mode;
    int	    	npt;
    DDXPointPtr	pptInit;
{
    void	    (*Polylines)();
    plxGCPtr	privGC = GetPlxGCPrivate(pGC);
    cfbPrivGCPtr devPriv = 
	((cfbPrivGCPtr) (pGC->devPrivates[cfb32GCPrivateIndex].ptr));
    plxScreenPtr  pPlxPrivate =  GetPlxPrivate(pGC->pScreen);

    if(privGC->rop == PLXV_ROP_XOR)
    {
            plxv_rop(5);
	    plxv_color_24(privGC->planemask);
	    plxv_tag(PLXV_TAG_MASK,pPlxPrivate->wid24True);
	    plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
	    plxv_wmask(PLXV_PIXEL_MASK);
    }
    else
        SetAttributes(pDrawable,pGC);
    switch (pGC->lineStyle)
    {
	case LineSolid:
	    if(pGC->lineWidth == 0)
	    {
		if (pGC->fillStyle == FillSolid)
		{
		  if((pDrawable->type == DRAWABLE_WINDOW) ||
		     ((pointer)(pDrawable->pScreen->devPrivate) ==
		      (pointer)pDrawable))
		    Polylines = pfbPolylines;
		  else
		  {
		    if (devPriv->oneRect)
			Polylines = cfb32LineSS1Rect;
		     else
		    	Polylines = cfb32LineSS;
		  }
		}
 		else
		    Polylines = miZeroLine;
	    }
	    else
		Polylines = miWideLine;
	    break;
	case LineOnOffDash:
	case LineDoubleDash:
	    if (pGC->lineWidth == 0 && pGC->fillStyle == FillSolid)
		Polylines =cfbLineSD;
	    else
		Polylines = miWideDash;
	    break;
    }
    (*Polylines)(pDrawable, pGC, mode, npt, pptInit);
}


static void
Plx32PolySegment(pDrawable, pGC, nseg, pSegs)
    DrawablePtr pDrawable;
    GCPtr 	pGC;
    int		nseg;
    xSegment	*pSegs;
{
    void	    (*PolySegment)();
    plxGCPtr	privGC = GetPlxGCPrivate(pGC);
    cfbPrivGCPtr devPriv = 
	((cfbPrivGCPtr) (pGC->devPrivates[cfb32GCPrivateIndex].ptr));
    plxScreenPtr  pPlxPrivate =  GetPlxPrivate(pGC->pScreen);

        if(privGC->rop == PLXV_ROP_XOR || privGC->rop == PLXV_ROP_INVERT)
        {
            plxv_rop(5);
	    plxv_tag(PLXV_TAG_MASK,pPlxPrivate->wid24True);
	    plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
	    plxv_wmask(PLXV_PIXEL_MASK);
        }
        else
            SetAttributes(pDrawable,pGC);

        PolySegment = miPolySegment;
        switch (pGC->lineStyle)
	{
	case LineSolid:
	    if(pGC->lineWidth == 0)
	    {
		if (pGC->fillStyle == FillSolid)
		{
		  if((pDrawable->type == DRAWABLE_WINDOW) ||
		     ((pointer)(pDrawable->pScreen->devPrivate) ==
		      (pointer)pDrawable))
		    	PolySegment = pfbPolySegment;
		  else
		  {
		    if (devPriv->oneRect)
			PolySegment = cfb32SegmentSS1Rect;
		    else
		    	PolySegment = cfb32SegmentSS;
		  }
		}
            }
	    break;
	case LineOnOffDash:
	case LineDoubleDash:
	    if (pGC->lineWidth == 0 && pGC->fillStyle == FillSolid)
		PolySegment = cfb32SegmentSD;
	    break;
	}
        (*PolySegment) (pDrawable, pGC, nseg, pSegs);
}

static void
Plx32PolyRectangle(pDrawable, pGC, nrects, pRects)
    DrawablePtr	pDrawable;
    GCPtr	pGC;
    int		nrects;
    xRectangle	*pRects;
{
    SetAttributes(pDrawable,pGC);
    plxv_rop(5);
	miPolyRectangle(pDrawable, pGC, nrects, pRects);
}

static void Plx32PolyArc(pDrawable, pGC, narcs, parcs)
    DrawablePtr	pDrawable;
    GCPtr	pGC;
    int		narcs;
    xArc	*parcs;
{
    SetAttributes(pDrawable,pGC);
	miPolyArc(pDrawable, pGC, narcs, parcs);
}       

static void 
Plx32FillPolygon(pDrawable, pGC, shape, mode, count, pPts)
    DrawablePtr	pDrawable;
    GCPtr	pGC;
    int		shape, mode;
    int		count;
    DDXPointPtr	pPts;
{
    SetAttributes(pDrawable,pGC);
	miFillPolygon(pDrawable, pGC, shape, mode, count, pPts);
}

static void
Plx32PolyFillArc(pDrawable, pGC, narcs, parcs)
    DrawablePtr	pDrawable;
    GCPtr	pGC;
    int		narcs;
    xArc	*parcs;
{
    SetAttributes(pDrawable,pGC);
	miPolyFillArc(pDrawable, pGC, narcs, parcs);
}

static int
Plx32PolyText8(pDrawable, pGC, x, y, count, chars)
    DrawablePtr pDrawable;
    GCPtr	pGC;
    int		x, y;
    int 	count;
    char	*chars;
{
#ifdef DEBUG
    fprintf(stderr,"Plx32PolyText8\n");
#endif
    SetAttributes(pDrawable,pGC);
    plxv_rop(5);
	miPolyText8(pDrawable, pGC, x, y, count, chars);
}

static int
Plx32PolyText16(pDrawable, pGC, x, y, count, chars)
    DrawablePtr 	pDrawable;
    GCPtr		pGC;
    int			x, y;
    int			count;
    unsigned short 	*chars;
{
#ifdef DEBUG
    fprintf(stderr,"Plx32PolyText16\n");
#endif
    SetAttributes(pDrawable,pGC);
    plxv_rop(5);
	miPolyText16(pDrawable, pGC, x, y, count, chars);
}

static void
Plx32ImageText8(pDrawable, pGC, x, y, count, chars)
    DrawablePtr pDrawable;
    GCPtr	pGC;
    int		x, y;
    int		count;
    char	*chars;
{
#ifdef DEBUG
    fprintf(stderr,"Plx32ImageText8\n");
#endif
    SetAttributes(pDrawable,pGC);
    plxv_rop(5);
	miImageText8(pDrawable, pGC, x, y, count, chars);
}

static void
Plx32ImageText16(pDrawable, pGC, x, y, count, chars)
    DrawablePtr pDrawable;
    GCPtr	pGC;
    int		x, y;
    int		count;
    char	*chars;
{
#ifdef DEBUG
    fprintf(stderr,"Plx32ImageText16\n");
#endif
    SetAttributes(pDrawable,pGC);
    plxv_rop(5);
	miImageText16(pDrawable, pGC, x, y, count, chars);
}

static void
Plx32ImageGlyphBlt(pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
    DrawablePtr		pDrawable;
    GCPtr		pGC;
    int 		x, y;
    unsigned int 	nglyph;
    CharInfoPtr 	*ppci;
    pointer 		pglyphBase;
{
#ifdef DEBUG_TEXT
    fprintf(stderr,"Plx32ImageGlyphBlt\n");
#endif
    SetAttributes(pDrawable,pGC);
    plxv_rop(5);
	miImageGlyphBlt(pDrawable, pGC, x, y, nglyph, ppci, pglyphBase);
}

static void
Plx32PolyGlyphBlt(pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
    DrawablePtr 	pDrawable;
    GCPtr		pGC;
    int 		x, y;
    unsigned int 	nglyph;
    CharInfoPtr 	*ppci;	
    char 		*pglyphBase;
{
#ifdef DEBUG_TEXT
    fprintf(stderr,"Plx32PolyGlyphBlt\n");
#endif
    SetAttributes(pDrawable,pGC);
    plxv_rop(5);
	miPolyGlyphBlt(pDrawable, pGC, x, y, nglyph, ppci, pglyphBase);
}

static void
Plx32PushPixels(pGC, pBitMap, pDst, w, h, x, y)
    GCPtr	pGC;
    PixmapPtr	pBitMap;
    DrawablePtr pDst;
    int		w, h, x, y;
{
plxGCPtr	privGC = GetPlxGCPrivate(pGC);
    SetAttributes(pDst,pGC);
    if(pDst->type == DRAWABLE_WINDOW && !privGC->win_type)
    {
	/*
	 * Set up the hardware for transparent accesses with tag override,
	 * rop = copy, and the enable bit set in the color value.
	 */
        plxv_transp_on();
	plxv_wmask(PLXV_PIXEL_MASK);
        plxv_tag(PLXV_TAG_MASK_DISABLE,PLXV_TAG_DISABLE);
        plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
        plxv_rop(5);
	plxv_colors_1(pGC->fgPixel|0x1000000,0,0);
        plxPushPixels8(pGC, pBitMap, pDst, w, h, x, y);
	plxv_transp_off();
    }
    else
    {
        SetAttributes(pDst,pGC);
	usePushPixels(pGC, pBitMap, pDst, w, h, x, y);
    }
}

static void
Plx32LineHelper()
{
    FatalError("Plx32LineHelper called\n");
}
GCOps *
plx32MatchCommon (pGC, devPriv)
    GCPtr	    pGC;
    cfbPrivGCPtr    devPriv;
{
    if (pGC->lineWidth != 0)
	return 0;
    if (pGC->lineStyle != LineSolid)
	return 0;
    if (pGC->fillStyle != FillSolid)
	return 0;
    if (devPriv->rop != GXcopy)
	return 0;
    if (pGC->font &&
	FONTMAXBOUNDS(pGC->font,rightSideBearing) -
        FONTMINBOUNDS(pGC->font,leftSideBearing) <= 32 &&
	FONTMINBOUNDS(pGC->font,characterWidth) >= 0)
    {
	if (TERMINALFONT(pGC->font)
#ifdef FOUR_BIT_CODE
	    && FONTMAXBOUNDS(pGC->font,characterWidth) >= 4
#endif
	)
	{
	    return &plx32TEOps;
        }
	else
	 {
	    return &plx32NonTEOps;
	 }
    }
    return 0;
}
