/*
 *   Copyright (c) 1987, 88 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 */
#ifndef PPW
#define PPW 4
#endif

#define PSZ 8
#include     "X.h"
#include    "windowstr.h"
#include "miscstruct.h"
#include "misc.h"
#include "cfb.h"
#include "cfbmskbits.h"
#include "pixmapstr.h"
#include <sys/mman.h>
#include "xvideo.h"
#include "plxInit.h"
#include "tv2reg.h"


#define M8VOFF 0x40000
#define M8SIZE (1152*900)
#define M24VOFF 0x140000
#define M24SIZE (1152*900*4)

#ifndef plxPMSK
#define plxPMSK 0xffffff
#endif

#if 0
#define getpagesize() (4096)
#endif

int fbFd = -1;

/* plxroptbl.c translate x rops to plx rops */

int plxroptbl[16] =
{
	PLXV_ROP_CLEAR,
	PLXV_ROP_AND,
	PLXV_ROP_ANDREVERSE,
	PLXV_ROP_COPY,
	PLXV_ROP_ANDINVERTED,
	PLXV_ROP_NOOP,
	PLXV_ROP_XOR,
	PLXV_ROP_OR,
	PLXV_ROP_NOR,
	PLXV_ROP_EQUIV,
	PLXV_ROP_INVERT,
	PLXV_ROP_ORREVERSE,
	PLXV_ROP_COPYINVERTED,
	PLXV_ROP_ORINVERTED,
	PLXV_ROP_NAND,
	PLXV_ROP_SET,
};

xplx_enable_tag ()
{
   plxv_tag(PLXV_TAG_MASK_DISABLE,PLXV_TAG_DISABLE);
}

typedef struct {
	int *address;
	int len;
	}PLX_MAPS;


int *
plx_map_one_segment(fd, offset, length)
int fd;
int offset;
int length;
{
	caddr_t page_address;

	page_address = mmap((caddr_t)0, length,
			PROT_READ|PROT_WRITE, MAP_SHARED,
			fd, offset);
	if (page_address == (caddr_t) -1) {
		perror("mmap0:");
		return(NULL);
		}
	return (int *)page_address;
}

int *c3_wreg;	/* need to move this elsewhere!!! */

#define NEW_C3_VADDR 0x812000

XV_struct *
plx_map_framebuffer( int fd, PLX_MAPS *map_array)
{
	XV_struct *tv2_context;
	int base, i, j, *dat;
	int *ptr;

        xv_regbase = plx_map_one_segment(fd, TV2_VADDR_REG, REGSIZE);
	xv_wreg = (XV_write_regs *) xv_regbase;
	xv_rreg = (XV_read_regs *) xv_regbase;
	xv_eprombase = NULL;	/* don't know where it's mapped, and noone cares anyway */
        xv_c3base = plx_map_one_segment(fd, NEW_C3_VADDR, C3SIZE);
	/*
        xv_c3base = plx_map_one_segment(fd, TV2_VADDR_C3, C3SIZE);
	*/
	c3_wreg = xv_c3base ;

        xv_m1base = plx_map_one_segment(fd, 0 , 0x28000 ) ;
	map_array->address= xv_m1base;
	map_array->len= 0x28000;
	map_array++;

        xv_m8base = (char *)plx_map_one_segment(fd, M8VOFF, M8SIZE);
	map_array->address= (int *)xv_m8base;
	map_array->len= M8SIZE;
	map_array++;

        xv_m24base = plx_map_one_segment(fd, M24VOFF, M24SIZE);
	map_array->address= xv_m24base;
	map_array->len= M24SIZE;

	tv2_context = plxv_init_context(xv_regbase, xv_m1base, 
	      xv_m8base, xv_m24base, xv_c3base, xv_eprombase);
	return(tv2_context);

}

int *plxv_peek(x,y)
int x,y;
{
  int *addr;
  addr = (int *) ( (int)xv_ap->m24[(y)] + ((x)<<2));
  ErrorF("addr(%d,%d) = 0x%x, val = 0x%x\n",x,y,addr, *addr);
  return addr;
}

plxv_reset_screen()
{
	XV_color c;
        plxv_tag(PLXV_TAG_MASK_DISABLE,PLXV_TAG_DISABLE);
        plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
        plxv_init_bt(6);
        plxv_init_disp();
	plxv_transp_on();
#if 0
	plxv_color_24(0x20ffffff);
#else
	plxv_color_24(0);
#endif
	plxv_wmask(0xffffffff);
	plxv_box(0,0,1152,900);
	plxv_wmask(0xffffff);
	plxv_mode_8();

/*	restore color table to console mode */

	c.red = 0xff;
	c.green = 0xff;
	c.blue = 0xff;
	plxv_load_clt(0, 0, 1, &c);	/* color 0 = white */

	c.red = 0;
	c.green = 0;
	c.blue = 0;
	plxv_load_clt(0, 255, 1, &c);	/* color 255 = black */

}
plxv_init_hw()
{
        plxv_init_bt(6);
	plxv_init_draw();
        plxv_init_bus();
        plxv_init_disp();
        plxv_init_video();
        plxv_init_dmsd();
        plxv_28_now(0);
	/*
	plxv_disp_blank(1);
	*/
	/*
	 * Clear 24 bit planegroup to black.
	 */
        plxv_transp_off();
        plxv_tag(PLXV_TAG_MASK_DISABLE,PLXV_TAG_DISABLE);
        plxv_stencil(PLXV_ST_MASK_DISABLE,PLXV_ST_DISABLE);
        plxv_rop(PLXV_ROP_COPY);
	plxv_wmask(PLXV_PIXEL_MASK);
	plxv_color_24(0);
	plxv_box(0,0,1152,900);
	/*
	 * Set Cursor bits to white.
	 */
	plxv_wmask(PLXV_CURSOR_MASK_SHIFTED);
	plxv_color_24(0xffffff);
	plxv_box(0,0,1152,900);
	/*
	 * Clear the 8 bit planegroup to Blue.
        plxv_mode_8();
	plxv_color_8(6);
	plxv_box(0,0,1152,900);
	 */
	/*
	plxv_disp_blank(0);
	*/
}
