/************************************************************
Copyright 1989 by The Massachusetts Institute of Technology

Permission to use, copy, modify, and distribute this
software and its documentation for any purpose and without
fee is hereby granted, provided that the above copyright
no- tice appear in all copies and that both that copyright
no- tice and this permission notice appear in supporting
docu- mentation, and that the name of MIT not be used in
advertising or publicity pertaining to distribution of the
software without specific prior written permission.
M.I.T. makes no representation about the suitability of
this software for any purpose. It is provided "as is"
without any express or implied warranty.

********************************************************/

/* THIS IS NOT AN X CONSORTIUM STANDARD */

/* $XConsortium: shm.c,v 1.16 93/01/28 19:29:19 rws Exp $ */

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#define NEED_REPLIES
#define NEED_EVENTS
#include "X.h"
#include "Xproto.h"
#include "misc.h"
#include "os.h"
#include "dixstruct.h"
#include "resource.h"
#include "scrnintstr.h"
#include "windowstr.h"
#include "pixmapstr.h"
#include "gcstruct.h"
#include "extnsionst.h"
#include "servermd.h"
#define _XSHM_SERVER_
#include "shmstr.h"
#include "Xfuncproto.h"

typedef struct _ShmDesc {
    struct _ShmDesc *next;
    int shmid;
    int refcnt;
    char *addr;
    Bool writable;
    unsigned long size;
} ShmDescRec, *ShmDescPtr;

#ifdef SUNSOFT_BUYBACK

#ifndef SVR4  /* SVR4 defines shmat() in shm.h */
#if NeedFunctionPrototypes
#if !defined(sgi) && !defined(hpux)
char *shmat(int, char*, int);
#endif
#else
char *shmat();
#endif
#endif /* SVR4 */

#else /* !SUNSOFT_BUYBACK */

#if NeedFunctionPrototypes

#if defined(SVR4)
void *shmat(int, void*, int);
#else
#if !defined(sgi) && !defined(hpux)
char *shmat(int, char*, int);
#endif
#endif

#else
char *shmat();
#endif

#endif /* SUNSOFT_BUYBACK */

ExtensionEntry *AddExtension();
static int ProcPlxShmDispatch(), SProcPlxShmDispatch();
static int PlxShmDetachSegment();
static void PlxShmResetProc(), SPlxShmCompletionEvent();

unsigned char PlxShmReqCode;
int PlxShmCompletionCode;
int PlxBadShmSegCode;
RESTYPE PlxShmSegType;
static ShmDescPtr Shmsegs;
static ShmFuncsPtr shmFuncs[MAXSCREENS];

#define VERIFY_SHMSEG(shmseg,shmdesc,client) \
{ \
    shmdesc = (ShmDescPtr)LookupIDByType(shmseg, PlxShmSegType); \
    if (!shmdesc) \
    { \
	client->errorValue = shmseg; \
	return PlxBadShmSegCode; \
    } \
}

#define VERIFY_SHMPTR(shmseg,offset,needwrite,shmdesc,client) \
{ \
    VERIFY_SHMSEG(shmseg, shmdesc, client); \
    if ((offset & 3) || (offset > shmdesc->size)) \
    { \
	client->errorValue = offset; \
	return BadValue; \
    } \
    if (needwrite && !shmdesc->writable) \
	return BadAccess; \
}

#define VERIFY_SHMSIZE(shmdesc,offset,len,client) \
{ \
    if ((offset + len) > shmdesc->size) \
    { \
	return BadAccess; \
    } \
}

void
PlxShmExtensionInit()
{
    ExtensionEntry *extEntry;
    int i;

    PlxShmSegType = CreateNewResourceType(PlxShmDetachSegment);
    if (PlxShmSegType &&
	(extEntry = AddExtension(PLXSHMNAME, ShmNumberEvents, ShmNumberErrors,
				 ProcPlxShmDispatch, SProcPlxShmDispatch,
				 PlxShmResetProc, StandardMinorOpcode)))
    {
	PlxShmReqCode = (unsigned char)extEntry->base;
	PlxShmCompletionCode = extEntry->eventBase;
	PlxBadShmSegCode = extEntry->errorBase;
	EventSwapVector[PlxShmCompletionCode] = SPlxShmCompletionEvent;
    }
}

/*ARGSUSED*/
static void
PlxShmResetProc (extEntry)
ExtensionEntry	*extEntry;
{
    int i;

    for (i = 0; i < MAXSCREENS; i++)
    {
	shmFuncs[i] = (ShmFuncsPtr)NULL;
    }
}


static int
ProcPlxShmQueryVersion(client)
    register ClientPtr client;
{
    REQUEST(xShmQueryVersionReq);
    xShmQueryVersionReply rep;
    register int n;

    REQUEST_SIZE_MATCH(xShmQueryVersionReq);
    rep.type = X_Reply;
    rep.length = 0;
    rep.sequenceNumber = client->sequence;
    /*
    rep.sharedPixmaps = sharedPixmaps;
    rep.pixmapFormat = pixmapFormat;
    */
    rep.sharedPixmaps = NULL;
    rep.pixmapFormat = NULL;
    rep.majorVersion = SHM_MAJOR_VERSION;
    rep.minorVersion = SHM_MINOR_VERSION;
    rep.uid = geteuid();
    rep.gid = getegid();
    if (client->swapped) {
    	swaps(&rep.sequenceNumber, n);
    	swapl(&rep.length, n);
	swaps(&rep.majorVersion, n);
	swaps(&rep.minorVersion, n);
	swaps(&rep.uid, n);
	swaps(&rep.gid, n);
    }
    WriteToClient(client, sizeof(xShmQueryVersionReply), (char *)&rep);
    return (client->noClientException);
}


static int
SProcPlxShmQueryVersion(client)
    register ClientPtr	client;
{
    register int n;
    REQUEST(xShmQueryVersionReq);

    swaps(&stuff->length, n);
    return ProcPlxShmQueryVersion(client);
}


static int
ProcPlxShmAttach(client)
    register ClientPtr client;
{
    struct shmid_ds buf;
    ShmDescPtr shmdesc;
    REQUEST(xPlxShmAttachReq);

    REQUEST_SIZE_MATCH(xPlxShmAttachReq);
    LEGAL_NEW_RESOURCE(stuff->shmseg, client);
    if ((stuff->readOnly != xTrue) && (stuff->readOnly != xFalse))
    {
	client->errorValue = stuff->readOnly;
        return(BadValue);
    }
    for (shmdesc = Shmsegs;
	 shmdesc && (shmdesc->shmid != stuff->shmid);
	 shmdesc = shmdesc->next)
	;
    if (shmdesc)
    {
	if (!stuff->readOnly && !shmdesc->writable)
	    return BadAccess;
	shmdesc->refcnt++;
    }
    else
    {
	shmdesc = (ShmDescPtr) xalloc(sizeof(ShmDescRec));
	if (!shmdesc)
	    return BadAlloc;
	shmdesc->addr = shmat(stuff->shmid, 0,
			      stuff->readOnly ? SHM_RDONLY : 0);
	if ((shmdesc->addr == ((char *)-1)) ||
	    shmctl(stuff->shmid, IPC_STAT, &buf))
	{
	    xfree(shmdesc);
	    return BadAccess;
	}
	shmdesc->shmid = stuff->shmid;
	shmdesc->refcnt = 1;
	shmdesc->writable = !stuff->readOnly;
	shmdesc->size = buf.shm_segsz;
	shmdesc->next = Shmsegs;
	Shmsegs = shmdesc;
    }
    if (!AddResource(stuff->shmseg, PlxShmSegType, (pointer)shmdesc))
	return BadAlloc;
    return(client->noClientException);
}

/*ARGSUSED*/
static int
PlxShmDetachSegment(shmdesc, shmseg)
    ShmDescPtr shmdesc;
    ShmSeg shmseg;
{
    ShmDescPtr *prev;

    if (--shmdesc->refcnt)
	return TRUE;
    shmdt(shmdesc->addr);
    for (prev = &Shmsegs; *prev != shmdesc; prev = &(*prev)->next)
	;
    *prev = shmdesc->next;
    xfree(shmdesc);
}

static int
ProcPlxShmDetach(client)
    register ClientPtr client;
{
    ShmDescPtr shmdesc;
    REQUEST(xPlxShmDetachReq);

    REQUEST_SIZE_MATCH(xPlxShmDetachReq);
    VERIFY_SHMSEG(stuff->shmseg, shmdesc, client);
    FreeResource(stuff->shmseg, RT_NONE);
    return(client->noClientException);
}


static int
ProcPlxShmDispatch (client)
    register ClientPtr	client;
{
    REQUEST(xReq);
    switch (stuff->data)
    {
    case X_ShmQueryVersion:
	return ProcPlxShmQueryVersion(client);
    case X_ShmAttach:
	return ProcPlxShmAttach(client);
    case X_ShmDetach:
	return ProcPlxShmDetach(client);
    default:
	return BadRequest;
    }
}

static void
SPlxShmCompletionEvent(from, to)
    xPlxShmCompletionEvent *from, *to;
{
    to->type = from->type;
    cpswaps(from->sequenceNumber, to->sequenceNumber);
    cpswapl(from->drawable, to->drawable);
    cpswaps(from->minorEvent, to->minorEvent);
    to->majorEvent = from->majorEvent;
    cpswapl(from->shmseg, to->shmseg);
    cpswapl(from->offset, to->offset);
}


static int
SProcPlxShmAttach(client)
    ClientPtr client;
{
    register int n;
    REQUEST(xPlxShmAttachReq);
    swaps(&stuff->length, n);
    REQUEST_SIZE_MATCH(xPlxShmAttachReq);
    swapl(&stuff->shmseg, n);
    swapl(&stuff->shmid, n);
    return ProcPlxShmAttach(client);
}

static int
SProcPlxShmDetach(client)
    ClientPtr client;
{
    register int n;
    REQUEST(xPlxShmDetachReq);
    swaps(&stuff->length, n);
    REQUEST_SIZE_MATCH(xPlxShmDetachReq);
    swapl(&stuff->shmseg, n);
    return ProcPlxShmDetach(client);
}

static int
SProcPlxShmDispatch (client)
    register ClientPtr	client;
{
    REQUEST(xReq);
    switch (stuff->data)
    {
    case X_ShmQueryVersion:
	return SProcPlxShmQueryVersion(client);
    case X_ShmAttach:
	return SProcPlxShmAttach(client);
    case X_ShmDetach:
	return SProcPlxShmDetach(client);
    default:
	return BadRequest;
    }
}
