#pragma ident "@(#)plxWid.c     1.12 94/03/31   SMI"

#include    <stdio.h>
#include    "sun.h"
#include     "mpg/mpg.h"
#include     "mpg/wid.h"
#include    "cmap/cmap.h"
#include     "X.h"
#include    "windowstr.h"
#include    <sys/mman.h>
#include    <sys/memfb.h>
#include    <sys/fbio.h>

#include    <sys/ioccom.h>
#include    <sys/fcntl.h>

#include    "colormap.h"
#include    "colormapst.h"
#include    "resource.h"
#include    "mibstore.h"
#include    "xvideo.h"
#include    "plxInit.h"
#define XV_DIAG 1
#include    "xvdebug.h"

/*  FORWARD REFERENCES: */

int    allocateWid();
void   plxFreeWindowWids();
int    plxGetWid();
void   plxSetColorLut();
WidPtr plxVideoWidAlloc();
void   plxWidFree();

/*-------------------------------------------------------------------------*
 |"allocateWid"
 |
 |   Internal routine to find a free window identifier of a specific 
 |   visual class.
 |
 |  = window id on success, -1 if no wids available for class
 *-------------------------------------------------------------------------*/
int allocateWid(
    ScreenPtr pScreen,  /* -> screen this window id is for. */
    int class           /* =  visual class of window id to allocate. */
)
{
    int             i_wid; 		/* Wid entry win WidPtrXXX accessing. */
    plxScreenPtr    pPlx;		/* Per screen context for Parallax boards. */
    plxWidTableRec* table; 		/* Fast access to pPlx->WidPtrXXX. */

    pPlx =  GetPlxPrivate(pScreen);
    switch(class)
    {
      case DirectColor: 
        table = pPlx->WidPtrDirectVideo; 
    	for ( i_wid = 0; i_wid < MAX_WIDS; i_wid++ )
    	{
			if(!table[i_wid].pWid) return i_wid;
    	}
		ErrorF("Wid Table exceeded...\n");
		return -1;
      case TrueColor:   
		table = pPlx->WidPtrTrueVideo;   
    	for ( i_wid = 0; i_wid < MAX_WIDS; i_wid++ )
    	{
			if(!table[i_wid].pWid) return i_wid;
    	}
		ErrorF("Wid Table exceeded...\n");
		return -1;
	}

	ErrorF("Invalid class for Wid allocation\n"); 
	return -1;
}


/*-------------------------------------------------------------------------*
 |"plxFreeWindowWids"
 |
 |  Free any window ids associated with the specified window.
 *-------------------------------------------------------------------------*/
void plxFreeWindowWids(
        WindowPtr pWin
)
{
    int          i_vis;     /* Element in pScreen->visuals accessing.  */
    int          i_win;     /* Element in window id table accessing.  */
    plxScreenPtr pPlx;		/* Per screen context for Parallax boards. */
    ScreenPtr 	 pScreen;
    WidPtr    	 pWid;
    VisualID  	 vid;



    pScreen  = pWin->drawable.pScreen;
	pPlx =  GetPlxPrivate(pScreen);
    vid = wVisual(pWin);
    for ( i_vis =0;  i_vis <pScreen->numVisuals;  i_vis++) 
	{
        if (pScreen->visuals[i_vis].vid == vid) break;
    }
	if ( i_vis >= pScreen->numVisuals )
	{
		ErrorF("plxFreeWindowWids: Unknown vid for window\n");
		return;
	}

    switch (pScreen->visuals[i_vis].class)
	{
      case DirectColor :
        for( i_win=0; i_win<MAX_WIDS; i_win++)
		{
			if(pPlx->WidPtrDirectVideo[i_win].pWin == pWin)
				plxWidFree(pPlx->WidPtrDirectVideo[i_win].pWid);
		}
        break;
   case PseudoColor :
        break;  /* do nothing */
   case TrueColor   :
        for( i_win=0; i_win<MAX_WIDS; i_win++)
		{
            if(pPlx->WidPtrTrueVideo[i_win].pWin == pWin)
                plxWidFree(pPlx->WidPtrTrueVideo[i_win].pWid);
		}
        break;
      default :
        ErrorF("Unknown Visual type\n");
		break;
   }
}

/*-------------------------------------------------------------------------*
 |"plxGetWid"
 |
 |  Allocate a window id for a drawable.
 |   
 *-------------------------------------------------------------------------*/
int plxGetWid(
        ScreenPtr   pScreen, 
        DrawablePtr pDraw,
		WidPtr*		pPwid
)
{
    WidPtr        new_wid;
    plxPrivWinPtr plxWinPtr;
    plxScreenPtr  pPlx;			/* Per screen context for Parallax boards. */
    int           vid;
    int           wid;

    pPlx =  GetPlxPrivate(pScreen);
 
    if (pDraw->type == DRAWABLE_WINDOW)
    {
        int index;
        plxWinPtr = (plxPrivWinPtr)plxGetPwinPriv(pDraw);
        if ( plxWinPtr->video_wid_set )
        {
            new_wid = (WidPtr)plxWinPtr->widptr;
        }
        else
        {
            new_wid   = plxVideoWidAlloc(pScreen,pDraw);
            widSetWindowWid((WindowPtr)pDraw, new_wid, TRUE);
            plxWinPtr->video_wid_set = 1;
        }
		if ( pPwid ) *pPwid = new_wid;
        wid       = widGetValue(new_wid);
						/* This needs to be augmented when we add DirectColor */
                        /* index is first four bits of wid */
        index = wid&0xf;
        pPlx->WidPtrTrueVideo[index].pWin = (WindowPtr)pDraw; 
        plxWinPtr->wid = wid;
        plxWinPtr->widptr = (int *)new_wid;
        return(wid);
    }
    else
    {
        ErrorF("pDraw->type != DRAWABLE_WINDOW\n");
        wid=pPlx->wid24True;
        return(wid);
    }
}


/*-------------------------------------------------------------------------*
 |"plxSetColorLut"
 |
 |  Sun X server transfer vector routine to set the color look up table
 |  associated with a window id.
 *-------------------------------------------------------------------------*/
void plxSetColorLut(
        WidPtr pWid, 
        int    clutId
)
{
    int       i_vis;
    ScreenPtr pScreen;
    VisualID  visual;

    TRACE(("plxSetColorLut...\n"));

    visual = widGetVisual(pWid);
    pScreen = widGetScreen(pWid);
    for ( i_vis = 0;  i_vis < pScreen->numVisuals;  i_vis ++) 
    {
       if (pScreen->visuals[i_vis].vid == visual)
          break;
    }

   switch (pScreen->visuals[i_vis].class)
   {
        case PseudoColor:
            widSetColorLut(pWid, PLX_PSEUDO_COLOR_CLUT);
            break;
        case DirectColor:
            widSetColorLut(pWid, PLX_DIRECT_COLOR_CLUT);
            break;
   }
}


/*-------------------------------------------------------------------------*
 |"plxVideoWidAlloc"
 |
 |   
 *-------------------------------------------------------------------------*/
WidPtr plxVideoWidAlloc(
        ScreenPtr   pScreen,
        DrawablePtr pDraw
)
{
   plxPrivWinPtr   plxWinPtr;
   WidPtr          pWid;
   VisualID        vid;

   plxWinPtr = plxGetPwinPriv(pDraw);
   vid = WindowVisual((WindowPtr)pDraw);
   pWid = widAllocate(pScreen,vid,1,TRUE,0);

   return pWid;
}

/*-------------------------------------------------------------------------*
 |"plxWidAlloc"
 |
 |   Sun X server transfer vector entry to allocate window identifiers.
 |
 |  -> window id structure on success, 0 on failure
 *-------------------------------------------------------------------------*/
WidPtr plxWidAlloc( 
	ScreenPtr pScreen,      /* -> screen to allocate window id for. */
    VisualID  visual,       /* =  visual id to allocate wid for. */
    int       numContig,    /* */
    Bool      unique,       /* =  1 if window id cannot be shared. */
    int       flavor        /* */
)
{
    int          i_visual;      /* entry in pScreen->visuals examining. */
    plxScreenPtr pPlx;			/* Per screen context for Parallax boards. */
    WidPtr       pWid;
    int          wid;

   TRACE(("plxWidAlloc...\n"));

    pPlx =  GetPlxPrivate(pScreen);
    for ( i_visual = 0; i_visual  < pScreen->numVisuals; i_visual++) 
    {
        if (pScreen->visuals[i_visual].vid == visual) break;
    }
    if ( i_visual >= pScreen->numVisuals )
    {
		 ErrorF("Unknown visual type requested\n");
		 return (NULL);
    }

	switch (pScreen->visuals[i_visual].class)
    {
      case PseudoColor:
		if ( pPlx->WidPtr8 )
		{
			if ( unique ) TRACE(("Warning: WidPtr8 already allocated\n"));
				return pPlx->WidPtr8;
		}
		if( !(pWid = widAllocObj()) ) return (NULL);
		widSetValue(pWid, PLX_PSEUDO_COLOR_TAG << 4); 
		break;

      case TrueColor:  
		if( !unique && pPlx->WidPtrTrue24) return pPlx->WidPtrTrue24;
                
		if( !(pWid = widAllocObj()) ) return (NULL);
#ifdef ANDY_KLUDGE
		if( (wid = allocateWid(pScreen,TrueColor)) <0 ) return NULL;
#else
		if( (wid = allocateWid(pScreen,TrueColor)) <0 ) 
		{
			widFreeObj(pWid);
			pWid = pPlx->WidPtrTrue24;
			wid = widGetValue(pWid);
		}
#endif /* ANDY_KLUDGE */
		widSetValue(pWid, (PLX_TRUE_COLOR_TAG<<4) | wid); 
        pPlx->WidPtrTrueVideo[wid].pWid = pWid;       
            break;

	  case DirectColor:
		if(!unique && pPlx->WidPtrDirect24) return pPlx->WidPtrDirect24;
           
		if( !(pWid = widAllocObj()) ) return (NULL);
#ifdef ANDY_KLUDGE
		if((wid = allocateWid(pScreen,DirectColor))<0) return (NULL);
#else
		if( (wid = allocateWid(pScreen,DirectColor)) <0 ) 
		{
			widFreeObj(pWid);
			pWid = pPlx->WidPtrDirect24;
			wid = widGetValue(pWid);
		}
#endif /* ANDY_KLUDGE */
		widSetValue(pWid,(PLX_DIRECT_COLOR_TAG<<4)|wid); 
		pPlx->WidPtrDirectVideo[wid].pWid = pWid;     
		break;
   }
   return (pWid);
}


/*-------------------------------------------------------------------------*
 |"plxWidFree"
 |
 |   Sun X server transfer vector routine to free a window id object, and 
 |   remove it from wid list maintained for its visual type.
 *-------------------------------------------------------------------------*/
void plxWidFree(
	WidPtr pWid             /* -> window id to be freed. */
)
{
    int       	 i_wid;    /* Element of window id table examining. */
    plxScreenPtr pPlx;	   /* Per screen context for Parallax boards. */
    ScreenPtr 	 pScreen;  /* -> screen to allocate window id for. */

    TRACE(("plxWidFree...\n"));

    pScreen = widGetScreen(pWid);
	pPlx =  GetPlxPrivate(pScreen);

    switch(widGetValue(pWid)>>4)
    {
      case PLX_DIRECT_COLOR_TAG :
        if(pWid == pPlx->WidPtrDirect24) return;
        for( i_wid =0; i_wid <MAX_WIDS; i_wid++)
        {
            if(pPlx->WidPtrDirectVideo[i_wid].pWid == pWid)
            {
                pPlx->WidPtrDirectVideo[i_wid].pWid = NULL;
                pPlx->WidPtrDirectVideo[i_wid].pWin = NULL;
            }
        }
        break;
      case PLX_PSEUDO_COLOR_TAG :
        if(pWid == pPlx->WidPtr8) return;
        break;
      case PLX_TRUE_COLOR_TAG   :
      case PLX_TRUE_COLOR_TAG | 1   :
        if(pWid == pPlx->WidPtrTrue24) return;
        for(i_wid=0;i_wid<MAX_WIDS;i_wid++)
		{
            if(pPlx->WidPtrTrueVideo[i_wid].pWid == pWid)
			{
				pPlx->WidPtrTrueVideo[i_wid].pWid = NULL;
				pPlx->WidPtrTrueVideo[i_wid].pWin = NULL;
			}
		 }
         break;
       default :
         ErrorF("Unknown Wid type\n");
         return;
   }
   widFreeObj(pWid);
}
