/* $Header: /Development/Plx2.5/xserver/server/ddx/solaris/reference/test/rtvModule.c,v 1.1 1995/11/27 22:46:59 dist Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: rtvModule.c,v $
 * Revision 1.1  1995/11/27  22:46:59  dist
 * Initial revision
 *
 * Revision 1.1  1995/10/13  21:35:04  kevin
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /Development/Plx2.5/xserver/server/ddx/solaris/reference/test/rtvModule.c,v 1.1 1995/11/27 22:46:59 dist Exp $";

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>


/*  XV_DIAG FLAGS:
 *
 *    0x80 - RTV operations trace.
 *    1    - Normal one-shot debugging
 */
#define XV_DIAG 0x80
#include "xvdebug.h"

#include "rtv.h"

static int rtv_connections=0;
static RTV_OBJECT *rtv_handle;

char* GetHostname(void);

void InitializeRTV(RTV_STRUCT* p_rtv)
{
    p_rtv->status = RTV_MODULE_DISCONNECTED;
    p_rtv->handle = (RTV_OBJECT *)NULL;
    p_rtv->attributes.interlace = RTV_MODULE_FULL_SIZE;
    p_rtv->attributes.win_x = 0;
    p_rtv->attributes.win_y = 0;
    p_rtv->attributes.fps = 30;
    p_rtv->attributes.win_w = 640;
    p_rtv->attributes.win_h = 480;
    p_rtv->hostname = (char *)NULL;
    p_rtv->fileName = (char *)NULL;
}


int ConnectRTV(RTV_STRUCT* p_rtv)
{
    if(p_rtv->hostname==NULL)
        p_rtv->hostname = (char *)malloc(30);
	gethostname(p_rtv->hostname,30);

    if(rtv_connections==0)
	{
        rtv_handle = (RTV_OBJECT *)RtvConnect(p_rtv->hostname);
	}
	rtv_connections++;
    p_rtv->handle = rtv_handle;

    p_rtv->status = RTV_MODULE_CONNECTED;
	return(1); /* ABW fix to return success for connection to RTV server */
}


int DisconnectRTV(RTV_STRUCT* p_rtv)
{
    if(p_rtv->handle==NULL)
	{
		fprintf(stderr,"Not Connected to Rtv Server- No handle\n");
		return(0);
	}
    RtvRequest(p_rtv->handle, CLIENT_EXIT); 
    if(p_rtv->hostname != NULL)
    {
        free(p_rtv->hostname);
        p_rtv->hostname = NULL;
    }
    p_rtv->status = RTV_MODULE_DISCONNECTED;
	if(rtv_connections==2)
		rtv_connections=1;
	else
	{
        RtvDisconnect(rtv_handle); /* ABW 10-20-95 must disconnect shm handle */
		rtv_connections=0;
	}
  return(1);
}
int SetRTVRecordPosition(RTV_STRUCT* p_rtv, int x, int y )
{
    	p_rtv->attributes.win_x = x;
    	p_rtv->attributes.win_y = y;
        RtvSetup(p_rtv->handle, RTV_SET_RECORD_ORIGIN, &p_rtv->attributes);
    return(1);
}
int SetRTVPlayPosition(RTV_STRUCT* p_rtv, int x, int y )
{
    	p_rtv->attributes.win_x = x;
    	p_rtv->attributes.win_y = y;
        RtvSetup(p_rtv->handle, RTV_SET_PLAY_ORIGIN, &p_rtv->attributes);
    return(1);
}


int SetRTVRecordWindow(RTV_STRUCT* p_rtv, int x, int y, int w , int h)
{
#ifdef RTV_DEBUG
	fprintf(stderr,"SetRTVRecordWindow: x=%d y=%d w=%d h=%d\n", x, y,w,h);
#endif
    	p_rtv->attributes.win_x = x;
    	p_rtv->attributes.win_y = y;
		p_rtv->attributes.win_w = w;
		p_rtv->attributes.win_h = h;
        RtvSetup(p_rtv->handle, RTV_SET_RECORD_WINDOW, &p_rtv->attributes);
    return(1);
}

int SetRTVPlayWindow(RTV_STRUCT* p_rtv, int x, int y, int w , int h)
{
#ifdef RTV_DEBUG
	fprintf(stderr,"SetRTVPlayWindow: x=%d y=%d w=%d h=%d\n", x, y,w,h);
#endif
    	p_rtv->attributes.win_x = x;
    	p_rtv->attributes.win_y = y;
		p_rtv->attributes.win_w = w;
		p_rtv->attributes.win_h = h;
        RtvSetup(p_rtv->handle, RTV_SET_PLAY_WINDOW, &p_rtv->attributes);
    return(1);
}


void RTVRecord(RTV_STRUCT* p_rtv)
{
    RtvRequest(p_rtv->handle, INIT_RTV_RECORD);
    RtvRequest(p_rtv->handle, START_RTV_RECORD);
    p_rtv->status = RTV_MODULE_RECORD;
}


void RTVPlay(RTV_STRUCT* p_rtv)
{
    switch(p_rtv->status)
    {
      case RTV_MODULE_CONNECTED:
      case RTV_MODULE_DISCONNECTED:
      case RTV_MODULE_RECORD:
      case RTV_MODULE_PLAY:
      case RTV_MODULE_PAUSE:
        PrintError("Error!  Must be idle to call RTVPlay");
        return;
        break;
    }      
    
    RtvRequest(p_rtv->handle, START_RTV_PLAY);
    p_rtv->status = RTV_MODULE_PLAY;
}

int RTVPauseWindow(RTV_STRUCT* p_rtv)
{
  RtvRequest(p_rtv->handle, RTV_PAUSE_WINDOW);
}
int RTVResumeWindow(RTV_STRUCT* p_rtv)
{
  RtvRequest(p_rtv->handle, RTV_RESUME_WINDOW);
}

int RTVPausePlay(RTV_STRUCT* p_rtv)
{
  RtvRequest(p_rtv->handle, PAUSE_RTV_PLAY);
  return(1);
}


int RTVPauseRecord(RTV_STRUCT* p_rtv)
{
  RtvRequest(p_rtv->handle, PAUSE_RTV_RECORD);
  return(1);
}


int RTVResumePlay(RTV_STRUCT* p_rtv)
{
  RtvRequest(p_rtv->handle, RESUME_RTV_PLAY);
  return(1);
}


int RTVResumeRecord(RTV_STRUCT* p_rtv)
{
  RtvRequest(p_rtv->handle, RESUME_RTV_RECORD); 
  return(1);
}


void RTVStop(RTV_STRUCT* p_rtv)
{
  switch(p_rtv->status)
  {
    case RTV_MODULE_PLAY:
    {
      RtvRequest(p_rtv->handle, STOP_RTV_PLAY);
      p_rtv->status = RTV_MODULE_IDLE;
      break;
    }

    case RTV_MODULE_RECORD:
    { 
      RtvRequest(p_rtv->handle, STOP_RTV_RECORD);  
      p_rtv->status = RTV_MODULE_IDLE;
      break;
    }
    default:
    {
      PrintDiagnostics("Internal error!  Must be playing or recording to call RTVStop!");
      break;
    }
  }
}


char *GetHostname(void)
{
  char command[100];
  char localHostname[100];
  static char *localHostnamePtr = NULL;
  char *tmpFileName;
  FILE *tmpFilePtr;
  int index;

  if(localHostnamePtr != NULL)
  {
    free(localHostnamePtr);
  }
  
  /* Check to see if there is an environment variable 'HOSTNAME' */
  if(getenv("LOCAL_HOSTNAME") != NULL)
  {
    localHostnamePtr = (char *)malloc(sizeof(char) * (strlen(getenv("LOCAL_HOSTNAME")) + 1));
    if(localHostnamePtr != NULL)
    {
      strcpy(localHostnamePtr, getenv("LOCAL_HOSTNAME"));
    }  
    return(localHostnamePtr);
  }
  
  /*
   * There was no environment variable, get local hostname through a system
   * call to 'hostname'.
   */
  tmpFileName = tmpnam(NULL);
  sprintf(command, "hostname > %s", tmpFileName);
  system(command);
  tmpFilePtr = fopen(tmpFileName, "r");
  fgets(localHostname, MAX_HOSTNAME_LENGTH, tmpFilePtr);
  remove(tmpFileName);
  
  /*
   * Remove the unwanted characters in the hostname string.
   */
  index = 0;
  while(index < strlen(localHostname))
  {
    if(localHostname[index] == '\n')
    {
      localHostname[index] = '\0';
    }
    index++;
  }
  localHostnamePtr = (char *)malloc(sizeof(char) * (strlen(localHostname) + 1));
  strcpy(localHostnamePtr, localHostname);
  return(localHostnamePtr);
}

int RTVGetServerStatus(int *currentFrame)
{
  /*
   * Get the current status of the Rtv server.  If the
   * Rtv server is still functioning, the current frame
   * is returned in handle->next_frame.  If not, there
   * will be an error code.
   */
#ifdef USE_RTV_CODE
  RtvRequest(p_rtv->handle, RTV_GET_STATUS);
  *currentFrame = p_rtv->handle->next_frame;
#endif
  if(*currentFrame < 0)
    return(0);
  else
    return(1);
}
