/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: xv_defaults.c,v $
 * Revision 1.1  1995/11/27  22:46:59  dist
 * Initial revision
 *
 * Revision 1.1  1995/10/30  17:27:33  kevin
 * Initial revision
 *
 * Revision 1.1  1995/10/13  21:35:04  kevin
 * Initial revision
 *
 * Revision 1.1  1995/03/30  19:42:32  kevin
 * Initial revision
 *
 * Revision 1.20  93/09/30  02:37:46  rocky
 * SQUASH, 7192A
 * 
 * Revision 1.19  92/11/19  14:59:45  rocky
 * new values needed for 76Hz serration
 * 
 * Revision 1.18  92/11/18  12:49:36  rocky
 * updates for 76Hz serration pulses
 * 
 * Revision 1.17  92/11/12  15:55:52  rocky
 * updates for 76 Hz serration
 * 
 * Revision 1.16  92/10/02  18:01:35  rocky
 * updates for SONY 60hz
 * 
 * Revision 1.15  92/10/01  16:01:07  rocky
 * temp version for SONYRGB
 * 
 * Revision 1.14  92/09/16  13:10:01  rocky
 * new option modes for SONY camera
 * 
 * Revision 1.13  92/01/24  11:14:38  kau
 * sun 76hz display parameters
 * 
 * Revision 1.12  91/10/23  09:56:05  mears
 * new rev
 * 
 * 
 * Revision 1.11  91/10/14  13:46:10  mears
 * restored auto-versioning of hlib to driver
 * 
 * Revision 1.10  91/10/04  12:47:59  mears
 * ifdef DRIVER
 * 
 * Revision 1.9  91/09/30  16:34:45  mears
 * many changes
 * 
 * Revision 1.8  91/08/20  20:38:03  mears
 * enable vtr mode
 * 
 * Revision 1.7  91/08/14  12:24:46  mears
 * blanking info in nvram
 * 
 * Revision 1.6  91/08/10  14:46:03  mears
 * clear @ passed ptr, not at xv_cp
 * 
 * Revision 1.5  91/08/09  16:58:17  mears
 * new nvram setup
 * 
 * Revision 1.4  91/08/06  10:41:07  eric
 * include plxv_xpoint.h
 * 
 * Revision 1.3  91/07/28  17:08:19  eric
 * added xpoint stuff to VIO card
 * 
 * Revision 1.2  91/07/26  19:07:03  eric
 * new from bill
 * 
 * Revision 1.1  91/06/12  16:06:35  eric
 * Initial revision
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: xv_defaults.c,v 1.1 1995/11/27 22:46:59 dist Exp $ Parallax Graphics Inc";
#endif

#include "plxv.h"
#include "plxv_xpoint.h"

#ifdef DEBUG
#define dprintf printf
#else
#define dprintf junk_print
#endif

#ifdef sun

static XV_disp_format Sbus_1_h = {287,12,31,42,0xbe,0x8000}; /*horozdisp 1152/66hz */
static XV_disp_format Sbus_1_v = {899,1, 3,30,0,0};    /* vert disp 900/66hz */

/* Note: 0xc5 for hsync_ser will be expanded to 0x145 */
static XV_disp_format Sbus_2_h = {287,9,31,37,0xbe,0x8000}; /* horozdisp 1152/76hz */
static XV_disp_format Sbus_2_v = {899,1, 7,32,0,0};    /* vert disp 900/76hz */
static XV_disp_format Sbus_3_h = {287,13,31,35,31,287}; /*horozdisp 1152/60hz */
static XV_disp_format Sbus_3_v = {899,1, 3,29,0,0};    /* vert disp 900/60hz */

#define DEFAULT_BD SBUS_1
#else
static XV_disp_format Tbus_1_h = {255,5,31,44,31,255};  /*horozdisp 1280/66hz */
static XV_disp_format Tbus_1_v = {1023,2, 2,32,0,0};   /* vert disp 1024/66hz */
#define DEFAULT_BD TCHAN_1
#endif

#define DEFAULT_OPT OPT_1
#define DEF_RAS_MODE ~0x10

static XV_knob_rec    knob_1   = {128,128,128,128};    /* initial video knobs */
static XV_dmsd_config dmsd_setup = {DMSDREV_1,0,
				0x50,0x30,0,0xe8,0xb6,0xe9,0x7,0,
				0xf8,0xf8,0x90,0x90,0,
				0x84, /* HREF fix */
				0x7a,0x59,
				0,0x2c,0x34,0xa,0xf4,0xce,0xe7,
				0x0,0x0,0};

static XV_bt261_init rgbin_format[2] = {{780,640,0x78,0,0,0},
				      {944,768,0x9c,0,0,0}};

static XV_bt261_config rgbin_config = {0x9c,0x87,0xf6,0x7f,
				0xd2,0x276,31,40,0x39,0x42,0x60,
				0,0,0};

/* dac loads to cal the ADC in the 253 */
/* rw,rb,gw,gb,bw,bb */
/* ryp,rym,yw,yb,byp,bym */
static XV_bt253_config rgbin_cal[2] = {
		0x98,0,0x98,0,0x98,0,0,0,
		0x98,0x4c,0x98,0,0x98,0x4c,0,0};

static XV_video_format output_format[2] = {
		{640,482,43,7,21},
		{768,574,51,7,34}};
static XV_video_format input_format_dmsd[2] = {
		{640,482,43,10,20},
		{768,574,51,7,23}};
static XV_video_format input_format_rgb[2] = {
		{640,482,43,10,20},
		{768,574,51,7,23}};

plxv_load_struct(cp)
XV_struct *cp;
{
    char junk[132];
    dprintf("loading defaults for board: %d\n",DEFAULT_BD);
    plxv_init_ram(cp,DEFAULT_BD,0);
    plxv_init_ram(cp,DEFAULT_OPT,1);
    dprintf("checking for option card .....");
    if (plxv_nv_read(1,junk,0,10))
    {   /* read error on nvram */
        dprintf("No option card present\n");
        cp->opt.board_code = 0;
	cp->opt.clt_type = 0;
    } else dprintf(" option board installed (assume %x)\n",cp->opt.board_code);
}

plxv_init_ram(cp,bdcode,opt)
XV_struct *cp;
int opt,bdcode;
{
    int i,k;
    char *j;
/*
    printf("plxv_init_ram: cp,bdcode,opt = 0x%x,%d,%d\n",cp,bdcode,opt);
*/
    /* clear the part of the structure being used */
    if (opt)
    {
	j = (char *)&(cp->opt);
	k = sizeof(XV_option_nvram);
    } else
    {
	j = (char *)&(cp->main);
	k = sizeof(XV_controller_nvram);
    }
    for (i=0;i<k;i++) *j++ = 0;

    /* load in the "factory" values */

    switch(bdcode)
    {
#ifdef sun
      case SBUS_1:		/* 1152 x 900 66 hz uses 92.9405 mhz clk */
      case SBUS_3:		/* 1152 x 900 60 hz uses 83.0204 mhz clk */
      case SBUS_4:		/* squash */
      case SBUS_5:		/* purvis */
	cp->main.board_code = bdcode;
	cp->main.ga_rev = GAREV_2;
	cp->main.c3_rev = 0;	/* load through xv_set */
	cp->main.clt_type = BT463;
	cp->main.hlib_rev = HLIB_REV;
	cp->main.power_hours = 0;
	cp->main.power_cycles = 1;
	cp->main.xsize = 1152;
	cp->main.ysize = 900;
	cp->main.Attr = 0x80c0;
/*
	cp->main.RasMode = DEF_RAS_MODE;
*/
	cp->main.RasMode = 0xffef;
	cp->main.DispFlgs = 0xa28;
/*
	cp->main.DispFlgs = 0x1e2a;
	cp->main.StartUp = 0x608 | 0x8000000;
*/
	cp->main.StartUp = 0x8000608;
	cp->main.TmrInit = 0x171;
	cp->main.DispKludge = 0xc41f;

	if(bdcode == SBUS_1 || bdcode == SBUS_4)
	{
		cp->main.h = Sbus_1_h;
		cp->main.v = Sbus_1_v;
	}
	else if(bdcode == SBUS_5)
	{
		cp->main.xsize = 640;
		cp->main.ysize = 480;
	}
	else
	{
/*
		printf("assigning SBUS3 H&V\n");
*/
		cp->main.h = Sbus_3_h;
		cp->main.v = Sbus_3_v;
	}

	cp->main.OLDI2C = 0;
	cp->main.d.flags = IO_DMSD;
	cp->main.d.factory = knob_1;
	cp->main.d.u1 = knob_1;
	cp->main.d.u2 = knob_1;
	cp->main.d.format[0] = input_format_dmsd[0];
	cp->main.d.format[1] = input_format_dmsd[1];
	cp->main.d.config.dmsd = dmsd_setup;
/* squash */
	cp->main.icd_clock = 0x13300f;
	cp->main.linebytes = 1152;
	cp->main.misc_flags = 0;
	cp->main.misc_flags2 = 0;
	break;
      case SBUS_2:		/* 1152 x 900 76 hz uses 105.561 mHz clk */
	cp->main.board_code = bdcode;
	cp->main.ga_rev = GAREV_2;
	cp->main.c3_rev = C3REV_1;
	cp->main.clt_type = BT463;
	cp->main.hlib_rev = HLIB_REV;
	cp->main.power_hours = 0;
	cp->main.power_cycles = 1;
	cp->main.xsize = 1152;
	cp->main.ysize = 900;
	cp->main.Attr = 0x80c0;
	cp->main.RasMode = DEF_RAS_MODE;
	cp->main.DispKludge = 0xc41f;
	cp->main.DispFlgs = 0xa28;
/*
	cp->main.DispFlgs = 0x1f2a;
	cp->main.StartUp = 0x60a | 0x8000000;
*/
	cp->main.StartUp = 0x8000608;
	cp->main.TmrInit = 0x171;
	cp->main.h = Sbus_2_h;
	cp->main.v = Sbus_2_v;
	cp->main.OLDI2C = 0;
	cp->main.d.flags = IO_DMSD;
	cp->main.d.factory = knob_1;
	cp->main.d.u1 = knob_1;
	cp->main.d.u2 = knob_1;
	cp->main.d.format[0] = input_format_dmsd[0];
	cp->main.d.format[1] = input_format_dmsd[1];
	cp->main.d.config.dmsd = dmsd_setup;
	cp->main.misc_flags = 0;
	cp->main.misc_flags2 = 0;
	break;
#else
      case TCHAN_1:
	cp->main.board_code = bdcode;
	cp->main.ga_rev = GAREV_2;
	cp->main.c3_rev = 0;
	cp->main.clt_type = BT463;
	cp->main.hlib_rev = HLIB_REV;
	cp->main.power_hours = 0;
	cp->main.power_cycles = 1;
	cp->main.xsize = 1280;
	cp->main.ysize = 1024;
	cp->main.Attr = 0x80c0;
	cp->main.RasMode = DEF_RAS_MODE;
	cp->main.DispKludge = 0xc41f;
	cp->main.DispFlgs = 0x1e0a;
	cp->main.StartUp = 0x5 | 0x8000000;
	cp->main.TmrInit = 0x171;
	cp->main.h = Tbus_1_h;
	cp->main.v = Tbus_1_v;
	cp->main.OLDI2C = 1;
	cp->main.d.flags = IO_DMSD;
	cp->main.d.factory = knob_1;
	cp->main.d.u1 = knob_1;
	cp->main.d.u2 = knob_1;
	cp->main.d.format[0] = input_format_dmsd[0];
	cp->main.d.format[1] = input_format_dmsd[1];
	cp->main.d.config.dmsd = dmsd_setup;
	break;
#endif
      case OPT_1:			/* the VIO option card */
	/* general stuff */
	cp->opt.board_code = bdcode;
	cp->opt.clt_type = BT473;
	cp->opt.hlib_rev = HLIB_REV;
	cp->opt.power_hours = 0;
	cp->opt.power_cycles = 1;

	/* b port is input (the DMSD chipset) */
	cp->opt.b.flags = IO_DMSD;
	cp->opt.b.factory = knob_1;
	cp->opt.b.u1 = knob_1;
	cp->opt.b.u2 = knob_1;
	cp->opt.b.format[0] = input_format_dmsd[0];
	cp->opt.b.format[1] = input_format_dmsd[1];
	cp->opt.b.config.dmsd = dmsd_setup;
	cp->opt.b.config.dmsd.unit = 1;

	/* a port is output */
	cp->opt.a.flags = IO_COMP_OUT;
	cp->opt.a.format[0] = output_format[0];
	cp->opt.a.format[1] = output_format[1];
	cp->opt.a.config.gen.xpoint_bits = A_DATA_OUTPUT_W | OUTPUT_EN_DATA |
					CE_HIGH | HSYNC_HIGH | VSYNC_HIGH ;
	cp->opt.misc_flags = 0;
	cp->opt.misc_flags2 = 0;
	break;
      case OPT_2:			/* the RGB card */
      case OPT_3:			/* the SONYRGB card */
      case OPT_4:			/* the SONYRGB2 card */
	/* general stuff */
	cp->opt.board_code = bdcode;
	cp->opt.clt_type = BT473;
	cp->opt.hlib_rev = HLIB_REV;
	cp->opt.power_hours = 0;
	cp->opt.power_cycles = 1;

	/* b port is rgb output */
	cp->opt.b.flags = IO_RGB_OUT;
	cp->opt.b.format[0] = output_format[0];
	cp->opt.b.format[1] = output_format[1];
	cp->opt.b.config.gen.xpoint_bits = 0x52535;	/* stuff of xpoint_b */

	/* a port is input */
	cp->opt.a.flags = (bdcode == OPT_3) || (bdcode == OPT_4) ? IO_SONYRGB_IN : IO_RGB_IN;
	cp->opt.a.format[0] = input_format_rgb[0];
	cp->opt.a.format[1] = input_format_rgb[1];
	cp->opt.a.factory = knob_1;
	cp->opt.a.u1 = knob_1;
	cp->opt.a.u2 = knob_1;
	cp->opt.a.config.rgbin.xpoint_bits = (HSYNC_HIGH | VSYNC_HIGH);
	cp->opt.a.config.rgbin.timing = rgbin_config;
	cp->opt.a.config.rgbin.format[0] = rgbin_format[0];
	cp->opt.a.config.rgbin.format[1] = rgbin_format[1];
	cp->opt.a.config.rgbin.cal[0] = rgbin_cal[0];
	cp->opt.a.config.rgbin.cal[1] = rgbin_cal[1];
	cp->opt.misc_flags = 0;
	cp->opt.misc_flags2 = 0;
	break;
      default:
	printf("init_struct: don't know about board %x\n",bdcode);
    }
}
junk_print()
{
}
