/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
#ifndef XVLIB_H
#define XVLIB_H
/*
** File: 
**
**   Xvlib.h --- Xv library public header file
**
** Author: 
**
**   David Carver (Digital Workstation Engineering/Project Athena)
**
** Revisions:
**
**   26.06.91 Carver
**     - changed XvFreeAdaptors to XvFreeAdaptorInfo
**     - changed XvFreeEncodings to XvFreeEncodingInfo
**
**   11.06.91 Carver
**     - changed SetPortControl to SetPortAttribute
**     - changed GetPortControl to GetPortAttribute
**     - changed QueryBestSize
**
**   05.15.91 Carver
**     - version 2.0 upgrade
**
**   01.24.91 Carver
**     - version 1.4 upgrade
**
*/

#include <X11/extensions/Xv.h>
#include <X11/extensions/XShm.h>
#include <X11/extensions/shmstr.h>
#include <X11/extensions/Xext.h>
#include <X11/extensions/extutil.h>
#include <XPlxExt.h>

typedef struct {
  int numerator;
  int denominator;
} XvRational;

typedef struct {
  XvEncodingID encoding_id;
  char *name;
  unsigned long width;
  unsigned long height;
  XvRational rate;
  unsigned long num_encodings;
  unsigned	blankIntervalHeight;				    /* New fieldadded for Plx->Xv compatibility */
  unsigned	interlaced;					    /* New fieldadded for Plx->Xv compatibility */
  unsigned	sync_ok;					    /* New fieldadded for Plx->Xv compatibility */
} XvEncodingInfo;

typedef struct {
  char depth;
  unsigned long visual_id;
} XvFormat;

typedef struct {
  XvPortID base_id;
  unsigned long num_ports;
  char type;
  char *name;
  unsigned long num_formats;
  XvFormat *formats;
  unsigned long num_adaptors;
} XvAdaptorInfo;

typedef struct {
  int type;
  unsigned long serial;	   /* # of last request processed by server */
  Bool send_event;	   /* true if this came from a SendEvent request */
  Display *display;	   /* Display the event was read from */
  Drawable drawable;       /* drawable */
  unsigned long reason;    /* what generated this event */
  XvPortID port_id;        /* what port */
  Time time;		   /* milliseconds */
} XvVideoNotifyEvent;

typedef struct {
  int type;
  unsigned long serial;	   /* # of last request processed by server */
  Bool send_event;	   /* true if this came from a SendEvent request */
  Display *display;	   /* Display the event was read from */
  XvPortID port_id;        /* what port */
  Time time;		   /* milliseconds */
  Atom attribute;           /* atom that identifies attribute */
  long value;              /* value of attribute */
} XvPortNotifyEvent;

typedef union {
  int type;
  XvVideoNotifyEvent xvvideo;
  XvPortNotifyEvent xvport;
  long pad[24];
} XvEvent;

extern int XvQueryExtension(
#if NeedFunctionPrototypes
  Display*                 /* display */,
  unsigned int*            /* p_version */,
  unsigned int*            /* p_revision */,
  unsigned int*            /* p_requestBase */,
  unsigned int*            /* p_eventBase */, 
  unsigned int*            /* p_errorBase */
#endif
);

extern int XvQueryAdaptors(
#if NeedFunctionPrototypes
  Display*                 /* display */,
  Window                   /* window */,
  unsigned int*            /* p_nAdaptors */,
  XvAdaptorInfo**          /* p_pAdaptors */
#endif
);

extern int XvQueryEncodings(
#if NeedFunctionPrototypes
  Display*                 /* display */,
  XvPortID                 /* port */,
  unsigned int*            /* p_nEncoding */,
  XvEncodingInfo**         /* p_pEncoding */
#endif
);

extern int XvPutVideo(
#if NeedFunctionPrototypes
  Display*                 /* display */,
  XvPortID                 /* port */,
  Drawable                 /* d */,
  GC                       /* gc */,
  int                      /* vx */, 
  int                      /* vy */,
  unsigned int             /* vw */, 
  unsigned int             /* vh */,
  int                      /* dx */, 
  int                      /* dy */,
  unsigned int             /* dw */,
  unsigned int             /* dh */
#endif
);

extern int XvPutStill(
#if NeedFunctionPrototypes
  Display*                 /* display */,
  XvPortID                 /* port */,
  Drawable                 /* d */,
  GC                       /* gc */,
  int                      /* vx */, 
  int                      /* vy */,
  unsigned int             /* vw */, 
  unsigned int             /* vh */,
  int                      /* dx */, 
  int                      /* dy */,
  unsigned int             /* dw */,
  unsigned int             /* dh */
#endif
);

extern int XvGetVideo(
#if NeedFunctionPrototypes
  Display*                 /* display */,
  XvPortID                 /* port */,
  Drawable                 /* d */,
  GC                       /* gc */,
  int                      /* vx */, 
  int                      /* vy */,
  unsigned int             /* vw */, 
  unsigned int             /* vh */,
  int                      /* dx */, 
  int                      /* dy */,
  unsigned int             /* dw */,
  unsigned int             /* dh */
#endif
);

extern int XvGetStill(
#if NeedFunctionPrototypes
  Display*                 /* display */,
  XvPortID                 /* port */,
  Drawable                 /* d */,
  GC                       /* gc */,
  int                      /* vx */, 
  int                      /* vy */,
  unsigned int             /* vw */, 
  unsigned int             /* vh */,
  int                      /* dx */, 
  int                      /* dy */,
  unsigned int             /* dw */,
  unsigned int             /* dh */
#endif
);

extern int XvStopVideo(
#if NeedFunctionPrototypes
  Display*                /* display */,
  XvPortID                /* port */,
  Drawable                /* drawable */
#endif
);

extern int XvGrabPort(
#if NeedFunctionPrototypes
  Display*                /* display */,
  XvPortID                /* port */,
  Time                    /* time */
#endif
);

extern int XvUngrabPort(
#if NeedFunctionPrototypes
  Display*                /* display */,
  XvPortID                /* port */,
  Time                    /* time */
#endif
);

extern int XvSelectVideoNotify(
#if NeedFunctionPrototypes
  Display*                /* display */,
  Drawable                /* drawable */,
  Bool                    /* onoff */
#endif
);

extern int XvSelectPortNotify(
#if NeedFunctionPrototypes
  Display*                /* display */,
  XvPortID                /* port */,
  Bool                    /* onoff */
#endif
);

extern int XvSetPortAttribute(
#if NeedFunctionPrototypes
  Display*                /* display */,
  XvPortID                /* port */,
  Atom                    /* attribute */,
  int                     /* value */
#endif
);

extern int XvGetPortAttribute(
#if NeedFunctionPrototypes
  Display*                /* display */,
  XvPortID                /* port */,
  Atom                    /* attribute */,
  int*                    /* p_value */
#endif
);

extern int XvQueryBestSize(
#if NeedFunctionPrototypes
  Display*                /* display */,
  XvPortID                /* port */,
  Bool                    /* motion */,
  unsigned int            /* vid_w */, 
  unsigned int            /* vid_h */,
  unsigned int            /* drw_w */, 
  unsigned int            /* drw_h */,
  unsigned int*           /* p_actual_width */, 
  unsigned int*           /* p_actual_width */
#endif
);

extern void XvFreeAdaptorInfo(
#if NeedFunctionPrototypes
  XvAdaptorInfo*          /* adaptors */
#endif
);

extern void XvFreeEncodingInfo(
#if NeedFunctionPrototypes
  XvEncodingInfo*         /* encodings */
#endif
);

/* Prototypes added for Parallax extensions to the protocol. */

extern void XPlxScaleCopyArea(
#if NeedFunctionPrototypes
			      Display*				    /* Display */,
			      Drawable				    /* Source window */,
			      Drawable				    /* Destination window */,
			      GC				    /* gc */,
			      int				    /* Source x */,
			      int				    /* Source y */,
			      int				    /* Source width */,
			      int				    /* Source height */,
			      int				    /* Dest x */,
			      int				    /* Dest y */,
			      int				    /* Dest width */,
			      int				    /* Dest height */
#endif			      
);

extern XPlxCImage* XPlxCreateCImage(
#if NeedFunctionPrototypes
				    Display*			    /* Display */,
				    char*			    /* JPEG data pointer */,
				    int				    /* Size of JPEG data area */,
				    unsigned int		    /* Width of JPEG image */,
				    unsigned int		    /* Height of JPEG image */
#endif
				    );

extern int XPlxDestroyCImage(
#if NeedFunctionPrototypes
			     XPlxCImage*			    /* JPEG Image data structure */
#endif
			     );

extern XPlxCImage* XPlxGetCImage(
#if NeedFunctionPrototypes
				 Display*			    /* Display */,
				 Drawable			    /* Source window */,
				 GC				    /* gc */,
				 int				    /* Source x */,
				 int				    /* Source y */,
				 unsigned int			    /* Source width */,
				 unsigned int			    /* Source height */,
				 unsigned int			    /* Dest width */,
				 unsigned int			    /* Dest height */
#endif
				 );

extern void XPlxPutCImage(
#if NeedFunctionPrototypes
			  Display*				    /* Display */,
			  Drawable				    /* Destination window */,
			  GC					    /* gc */,
			  XPlxCImage*				    /* Source JPEG image */,
			  int					    /* Source x */,
			  int					    /* Source y */,
			  unsigned int				    /* Source width */,
			  unsigned int				    /* Source height */,
			  int					    /* Dest x */,
			  int					    /* Dest y */,
			  unsigned int				    /* Dest width */,
			  unsigned int				    /* Dest height */,
			  int					    /* Crop-or-Squeeze flag */
#endif
			  );

extern void XPlxPutTable(
#if NeedFunctionPrototypes
			 Display*				    /* Display */,
			 Drawable				    /* Source or Dest window */,
			 GC					    /* gc */,
			 short*					    /* Pointer to QTable data */,
			 int					    /* Size of data */,
			 int					    /* Format */
			 );
#endif

extern XPlxCImage* XPlxShmCreateCImage(
#if NeedFunctionPrototypes
				       Display*			    /* Display */,
				       char*			    /* Pointer to JPEG data (shared memory) */,
				       XShmSegmentInfo*		    /* Shared memory info */,
				       int			    /* Size of shared memory segment */,
				       unsigned int		    /* Width of JPEG image (hint only) */,
				       unsigned int		    /* Height of JPEG image (hint only) */
				       );
#endif

extern Status XPlxShmGetCImage(
#if NeedFunctionPrototypes
			       Display*				    /* Display */,
			       Drawable				    /* Source window */,
			       GC				    /* gc */,
			       XPlxCImage*			    /* Pointer to JPEG image memory */,
			       int				    /* Source x */,
			       int				    /* Source y */,
			       unsigned int			    /* Source width */,
			       unsigned int			    /* Source height */
			       );
#endif

extern Status XPlxShmPutCImage(
#if NeedFunctionPrototypes
			       Display*				    /* Display */,
			       Drawable				    /* Dest window */,
			       GC				    /* gc */,
			       XPlxCImage*			    /* Pointer to JPEG image data */,
			       int				    /* Source x */,
			       int				    /* Source y */,
			       unsigned int			    /* Source width */,
			       unsigned int			    /* Source height */,
			       int				    /* Dest x */,
			       int				    /* Dest y */,
			       unsigned int			    /* Dest width */,
			       unsigned int			    /* Dest height */,
			       int				    /* Crop-or-Squeeze flag */,
			       Bool				    /* Send completion event or not */
			       );
#endif

extern XShmSegmentInfo* XPlxGetShmInfo(
#if NeedFunctionPrototypes
				       void);
#endif

extern void XPlxFreeShmInfo(
#if NeedFunctionPrototypes
			    XShmSegmentInfo*			    /* Shared memory segment info */
			    );
#endif

#endif XVLIB_H
