/***********************************************************
Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
** File: 
**
**   xvplx.h --- Xv Parallax device dependent header file
**
** Author: 
**
**   David Carver (Digital Workstation Engineering/Project Athena)
**
** Revisions:
**
**   26.11.92 Dave Arnold (Parallax Graphics)
**     - Initial Parallax port
**
**   26.11.91 Carver
**     - optimized PutStill to not redraw enable plane between like stills.
**       added fields to XvropPortRec to cache info from last request.
**
**   29.08.91 Carver
**     - added support for video in StaticGray windows
**     - added field to Port priv to store handle to rop and libpip: prop
**
**   26.06.91 Carver
**     - fixed GC wrappers
**
**   04.06.91 Carver
**     - changed interface to libpip.c, use new libpip.h header file
**     - changed assignment of encoding id's
**
**   31.05.91 Carver
**     - made big fixes to occlusion stuff
**
**   29.05.91 Carver
**     - added serialNumber to port private structure.
**
**   15.05.91 Carver
**     - version 2.0 upgrade
**
**   19.03.91 Carver
**     - original port for v1r4
**
**
*/
#ifndef XVPLX_H
#define XVPLX_H

#include "Xvproto.h"
#include "region.h"
#include "XPlxExt.h"

#ifdef GLOBAL
#define EXTERNAL
#define INIT(i) = {i}
#define INIT2(i, j) = {i, j}
#else GLOBAL
#define EXTERNAL extern
#define INIT(i)
#define INIT2(i, j)
#endif

#ifdef GLOBAL
int xvplx_num_adaptors = 1;
#else
extern int xvplx_num_adaptors;
#endif

typedef struct _XvplxPortRec {
  GCPtr pGC;
  int channel;
  int vx,vy,dx,dy;
  unsigned int vw,vh,dw,dh;
  BoxRec enabled_box;
  XvEncodingPtr pEncoding;
  int hue,saturation,brightness,contrast;
  Bool Genlock,Inverted,Crop;
  int sync_ok,interlaced,BlankIntervalHeight;
  unsigned long serialNumber;
  GCFuncs *wrapFuncs;
  Bool gray;
  unsigned int vis;
  unsigned int Dx,Dy;
  DrawablePtr pDrawStill;
  int cvx,cvy,cdx,cdy;
  unsigned int cvw,cvh,cdw,cdh;
} XvplxPortRec, *XvplxPortPtr;

typedef struct _XvplxScreenRec {
  void (* ClipNotify)();
  void (* CopyWindow)();
  void (* WindowExposures)();
  Bool (* UnrealizeWindow)();
} XvplxScreenRec, *XvplxScreenPtr;

EXTERNAL int XvplxNumEncodings[2] INIT2(3, 6);
EXTERNAL int XvplxNumPorts[2] INIT2(1, 2);
EXTERNAL int XvplxNumFormats[2] INIT2(2, 2);

/* PLX_xxx SYMBOLS TAKEN FROM XplxExt.h */

#define PLXV_NTSC (PLX_NTSC << 16)
#define PLXV_PAL (PLX_PAL << 16)
#define PLXV_SECAM (PLX_SECAM << 16)

#ifdef GLOBAL
XvEncodingRec XvplxBaseEncodings[1][3] = 
{
  {
    {PLX_COMP | PLXV_NTSC, (ScreenPtr)NULL, 
       "ntsc-composite", 640, 525, 5994, 100},
    {PLX_COMP | PLXV_PAL, (ScreenPtr)NULL, 
       "pal-composite", 768, 625, 50, 1}, 
    {PLX_COMP | PLXV_SECAM, (ScreenPtr)NULL, 
       "secam-composite", 768, 625, 50, 1},
  }
};

XvEncodingRec XvplxOpt1Encodings[1][6] = 
{
  {
    {PLX_COMP | PLXV_NTSC, (ScreenPtr)NULL, 
       "ntsc-composite", 640, 525, 5994, 100},
    {PLX_COMP | PLXV_PAL, (ScreenPtr)NULL, 
       "pal-composite", 768, 625, 50, 1}, 
    {PLX_COMP | PLXV_SECAM, (ScreenPtr)NULL, 
       "secam-composite", 768, 625, 50, 1},
    {PLX_YC | PLXV_NTSC, (ScreenPtr)NULL, 
       "ntsc-svideo", 640, 525, 5994, 100},
    {PLX_YC | PLXV_PAL, (ScreenPtr)NULL, 
       "pal-svideo", 768, 625, 50, 1}, 
    {PLX_YC | PLXV_SECAM, (ScreenPtr)NULL, 
       "secam-svideo", 768, 625, 50, 1},
  }
};

XvEncodingRec XvplxOpt2Encodings[1][6] = 
{
  {
    {PLX_RGB | PLXV_NTSC, (ScreenPtr)NULL, 
       "ntsc-rgb", 640, 525, 5994, 100},
    {PLX_RGB | PLXV_PAL, (ScreenPtr)NULL, 
       "pal-rgb", 768, 625, 50, 1}, 
    {PLX_RGB | PLXV_SECAM, (ScreenPtr)NULL, 
       "secam-rgb", 768, 625, 50, 1},
    {PLX_YUV | PLXV_NTSC, (ScreenPtr)NULL, 
       "ntsc-yuv", 640, 525, 5994, 100},
    {PLX_YUV | PLXV_PAL, (ScreenPtr)NULL, 
       "pal-yuv", 768, 625, 50, 1}, 
    {PLX_YUV | PLXV_SECAM, (ScreenPtr)NULL, 
       "secam-yuv", 768, 625, 50, 1},
  }
};

#else
XvEncodingRec XvplxBaseEncodings[1][3];
XvEncodingRec XvplxOpt1Encodings[1][6];
XvEncodingRec XvplxOpt2Encodings[1][6];
#endif GLOBAL

#define PLX_SCREEN_PROLOGUE(pScreen, props, field)\
  ((pScreen)->field = props->field)

#define PLX_SCREEN_EPILOGUE(pScreen, field, wrapper)\
  ((pScreen)->field = wrapper)

/* LIST OF PARALLAX PORT CONTROLS XXX --- MORE LATER */

EXTERNAL Atom XvplxEncoding;
EXTERNAL Atom XvplxHue;
EXTERNAL Atom XvplxSaturation;
EXTERNAL Atom XvplxBrightness;
EXTERNAL Atom XvplxContrast;
EXTERNAL Atom XvplxValueRecall;
EXTERNAL Atom XvplxValueSave;
EXTERNAL Atom XvplxCrop;
EXTERNAL Atom XvplxParameterString;

#undef EXTERNAL
#undef INIT

#endif XVPLX_H
